maak_datasets = function(ziekenhuis){
  if(ziekenhuis == 'rs'){
    
    
    # laad Rijnstate data
    bestand_rs_opnames <- list.files("data/rs_EXCEL/",
                                     # pattern = 'Corona data voorspelmodel corona', 
                                     full.names = TRUE
    )%>% tail(n=1)
    print(paste("Bestand RS opnames:", bestand_rs_opnames))
    
    #reading data, sorting on time of entry, dropping irrelevant columns, renaming columns to a better interpretable name
    rs_excel = read_csv(bestand_rs_opnames) %>%
      select( -Diagnose, -`DiagnoseVolgNr`, -OpnameBehandelBeperkingDatum)%>% 
      rename(BehandelBeperking = OpnameBehandelBeperking, Bestemming = OntslagBestemmingOms, Herkomst = OpnameHerkomstOms, startdatumtijd = OpnameMutatieBeginDatumTijd,
             einddatumtijd = OpnameMutatieEindDatumTijd, opname_nummer = PK_OpnameNr, geslacht = Geslacht, status_covid = TypeCorona,
             tijd_opname = OpnameDatumTijd, tijd_ontslag = OntslagDatumTijd
      ) %>% filter(status_covid != 'Geen')
    
    #removing consequtive identical rows (based on patient no and IC info), making the origin/destination more detailed in the case
    # of patients going from/to the IC. 
    result = NULL
    patients = unique(rs_excel$opname_nummer)
    for(patient in patients){
      #get data concerning patient and sort
      data_patient = rs_excel %>% filter(opname_nummer == patient) %>% arrange(startdatumtijd) 
      if(!(any(data_patient$status_covid =='Geen') | any(as.POSIXct(data_patient$tijd_opname)<as.POSIXct("2020-02-27 00:00:00")) )){ #delete patients that didn't have Covid- 19 or arrived at weird time
        #dropping rows if the entries for IC herkomst etc are the same. In this case, the enddate for mutation should be set to the last value.
        i=2
        IC_cols_prev = data_patient[1,] %>% select(HerkomstIC, BestemmingIC, ICBeginDatumTijd, ICEindDatumTijd)
        while(i <= nrow(data_patient)){
          IC_cols_now = data_patient[i,] %>% select(HerkomstIC, BestemmingIC,ICBeginDatumTijd, ICEindDatumTijd)
          if(all(IC_cols_prev == IC_cols_now)){
            data_patient[i-1,]$einddatumtijd = data_patient[i,]$einddatumtijd #set the mutation enddate to the enddate of the removed row
            data_patient = data_patient[-i,]                                  #remove row
          }else{
            #if the entries are not the same, we are dealing with a transition for this patient
            if(any(IC_cols_prev == 'NULL')){
              data_patient[i-1, 'Bestemming'] = 'IC'
              data_patient[i, 'Herkomst'] = 'Kliniek'
            }else{
              data_patient[i,'Herkomst'] = 'IC'
              data_patient[i-1, 'Bestemming'] = 'Kliniek'
            }
            i = i+1
            IC_cols_prev = IC_cols_now
          }
        }
        result = bind_rows(result, data_patient)
      }
    }
    rs_excel = result
    rs_excel[ , -c(4,5,6,7)][rs_excel[ , -c(4,5,6,7)] == 'NULL'] = NA   # renaming NULL to NA (to better indicate the value as missing)
    
    #als er geen info over behandelbeperking beschikbaar is, ga dan van "geen behandelbeperking" uit
    rs_excel$BehandelBeperking[is.na(rs_excel$BehandelBeperking)] = 'Geen behandelbeperking'
    
    # determining the last (valid) datetime of a mutation as the prediction date
    rs_excel$Bestemming[rs_excel$einddatumtijd > Sys.time()] = NA
    rs_excel$einddatumtijd[rs_excel$einddatumtijd > Sys.time()] = NA   # setting enddates far away (e.g. 2999) to na   
    datetime_pred = max(c(rs_excel$einddatumtijd[!is.na(rs_excel$Bestemming)], rs_excel$startdatumtijd))
    
    # obtaining LOS rounded up to days
    rs_excel = rs_excel %>% mutate(einddatumtijd = as.POSIXct(einddatumtijd, tz = 'UTC'), startdatumtijd = as.POSIXct(startdatumtijd, tz = 'UTC'))
    rs_excel$Bestemming[is.na(rs_excel$Bestemming) & rs_excel$einddatumtijd < datetime_pred] = 'Onbekend'
    rs_excel$einddatumtijd[rs_excel$einddatumtijd > datetime_pred] = NA   # setting enddates far away (e.g. 2999) to na   
    rs_excel = rs_excel %>% mutate(startdatum = as.Date(startdatumtijd), einddatum = as.Date(einddatumtijd))
    rs_excel = rs_excel %>% mutate(ligduur = sapply(difftime(rs_excel$einddatum , rs_excel$startdatum, unit = 'days') + 1, as.numeric))  
    
    #when patient is still there, the LOS is set to the LOS up to now
    rs_excel$ligduur[is.na(rs_excel$ligduur)] = sapply(difftime(as.Date(datetime_pred) , rs_excel$startdatum[is.na(rs_excel$ligduur)], unit = 'days') + 1, as.integer)
    
    #verkrijg bestemmingen
    mogelijke_bestemmingen = unique(rs_excel$Bestemming)
    bestemmingen_overleden = mogelijke_bestemmingen[grepl('Overleden', mogelijke_bestemmingen)]
    bestemmingen_naar_huis = mogelijke_bestemmingen[grepl('woonomgeving', mogelijke_bestemmingen)]
    bestemmingen_instelling = mogelijke_bestemmingen[grepl('instelling', mogelijke_bestemmingen)|
                                                       grepl('verzorgingshuis', mogelijke_bestemmingen) | 
                                                       grepl('Hospice', mogelijke_bestemmingen)]
    
    
    #defining IC set as the set where IC rows have entries
    ICset = rs_excel %>% filter( !(is.na(ICBeginDatumTijd)|is.na(ICEindDatumTijd) |is.na(BestemmingIC)|is.na(HerkomstIC))) %>%
      select(-ICBeginDatumTijd, -ICEindDatumTijd, -BestemmingIC, -HerkomstIC)
    
    ICset$Bestemming[is.na(ICset$Bestemming)] = 'nog op IC' # defining destination when patient is still in IC
    
    #determining the departure path for each IC patient
    ICset = ICset %>% mutate(vertrek_type = NA)
    ICset$vertrek_type[((ICset$Bestemming == 'Kliniek') )] = 'IC - vertrek naar Kliniek'
    ICset$vertrek_type[((ICset$Bestemming == 'nog op IC') )] = 'IC - Nog opgenomen'
    ICset$vertrek_type[((ICset$Bestemming == 'Ander ziekenhuis') )] = 'IC - ander ziekenhuis'
    
    ICset$vertrek_type[sapply(ICset$Bestemming, function(bst){ any(bestemmingen_overleden == bst)})] = 'IC - overleden'
    ICset$vertrek_type[sapply(ICset$Bestemming, function(bst){ any(bestemmingen_naar_huis == bst)})] = 'IC - naar huis'
    ICset$vertrek_type[sapply(ICset$Bestemming, function(bst){ any(bestemmingen_instelling == bst)})] = 'IC - overige instelling'
    ICset$vertrek_type[is.na(ICset$vertrek_type)] = 'IC - anders'
    
    #determining event variable for survival analysis (Kaplan-Meijer estimate)
    ICset$event = ifelse(ICset$vertrek_type == "IC - Ander ziekenhuis", 0,
                         ifelse(ICset$vertrek_type == "IC - Nog opgenomen", 0, 1))
    
    df_ic = ICset %>% mutate(opname_type = ifelse(Herkomst != 'Kliniek'&
                                                    Herkomst != "Uit eigen ziekenhuis"&
                                                    Herkomst != "Ander ziekenhuis", 'direct', 'indirect'))
    
    #determining arrivals directly to IC (not from other hospitals)
    Aankomst_IC = ICset %>% filter(Herkomst != 'Kliniek'&
                                     Herkomst != "Uit eigen ziekenhuis"&
                                     Herkomst != "Ander ziekenhuis") %>%
      select(startdatum) %>% 
      mutate(aantal = 1) %>%
      group_by(startdatum) %>% 
      summarise(aantal_ic = sum(aantal), .groups = 'drop') %>%
      ungroup()
    
    
    #defining kliniek set as the set where the IC rows do not have entries (they are missing/NA)
    Kliniekset = rs_excel %>% filter( (is.na(ICBeginDatumTijd)|is.na(ICEindDatumTijd) |is.na(BestemmingIC)|is.na(HerkomstIC)))%>%
      select(-ICBeginDatumTijd, -ICEindDatumTijd, -BestemmingIC, -HerkomstIC)
    
    Kliniekset$Bestemming[is.na(Kliniekset$Bestemming)] = 'nog op kliniek'# defining destination when patient is still in clinic
    
    #determining the departure path for each clinic patient
    Kliniekset = Kliniekset %>% mutate(vertrek_type = NA)
    Kliniekset$vertrek_type[((Kliniekset$Bestemming == 'IC') )] = 'Kliniek - vertrek naar IC'
    Kliniekset$vertrek_type[((Kliniekset$Bestemming == 'nog op kliniek') )] = 'Kliniek - Nog opgenomen'
    Kliniekset$vertrek_type[((Kliniekset$Bestemming == 'Ander ziekenhuis') )] = 'Kliniek - ander ziekenhuis'
    
    Kliniekset$vertrek_type[sapply(Kliniekset$Bestemming, function(bst){ any(bestemmingen_overleden == bst)})] = 'Kliniek - overleden'
    Kliniekset$vertrek_type[sapply(Kliniekset$Bestemming, function(bst){ any(bestemmingen_naar_huis == bst)})] = 'Kliniek - naar huis'
    Kliniekset$vertrek_type[sapply(Kliniekset$Bestemming, function(bst){ any(bestemmingen_instelling == bst)})] = 'Kliniek - overige instelling'
    Kliniekset$vertrek_type[is.na(Kliniekset$vertrek_type)] = 'Kliniek - anders'
    
    #determining event variable for survival analysis
    Kliniekset$event <- ifelse(Kliniekset$vertrek_type == "Kliniek - Ander ziekenhuis", 0,
                               ifelse(Kliniekset$vertrek_type == "Kliniek - Nog opgenomen", 0, 1))
    
    
    df_kl = Kliniekset %>% mutate(opname_type = ifelse(Herkomst != 'IC'&
                                                         Herkomst != "Uit eigen ziekenhuis"&
                                                         Herkomst != "Ander ziekenhuis", 'direct', 'indirect'))
    
    #determining arrivals directly to clinic (not from other hospitals, depts)
    Aankomst_kliniek = Kliniekset %>% filter(Herkomst != 'IC'&
                                               Herkomst != "Uit eigen ziekenhuis"&
                                               Herkomst != "Ander ziekenhuis") %>%
      select(startdatum) %>%
      mutate(aantal = 1) %>%
      group_by(startdatum) %>% 
      summarise(aantal_kliniek = sum(aantal), .groups = 'drop') %>%
      ungroup()
    
    
    #merging the arrival information
    aankomst_totaal = tibble(startdatum = seq(min(c(Aankomst_IC$startdatum, Aankomst_kliniek$startdatum)), as.Date(datetime_pred), by = 'days'))
    aankomst_totaal = left_join(aankomst_totaal, Aankomst_IC, by = 'startdatum') %>% left_join(Aankomst_kliniek, by = 'startdatum')%>%
      replace_na(replace = list(aantal_ic = 0, aantal_kliniek = 0)) %>% rename(datum = startdatum)
    
    # #merging the arrival information
    # aankomst_totaal = as_tibble(merge(Aankomst_IC, Aankomst_kliniek, by= 'startdatum', all= TRUE))%>% 
    #   replace_na(replace = list(aantal_ic = 0, aantal_kliniek = 0)) %>% rename(datum = startdatum)
    
    aankomst_totaal = aankomst_totaal %>%
      add_column(aantal_cumulatief_ic = cumsum(aankomst_totaal$aantal_ic), .before = 3)
    
    aankomst_totaal = aankomst_totaal %>%
      add_column(aantal_cumulatief_kliniek = cumsum(aankomst_totaal$aantal_kliniek), .after  = 4)
    
    df_nieuwe_opnames = aankomst_totaal %>%
      add_column(aantal = aankomst_totaal$aantal_ic + aankomst_totaal$aantal_kliniek,
                 aantal_cumulatief = aankomst_totaal$aantal_cumulatief_ic +aankomst_totaal$aantal_cumulatief_kliniek, 
                 .before = 2)%>% 
      mutate(
        datum = date(datum),
        num = 1:nrow(.)
      )
  }
  if(ziekenhuis %in% c('LUMC','ETZ','Haga', 'GHZ', 'ZGT')){ # algemene data, staat in "standaard format".
    
    bestand_opnamemutaties <- list.files(paste0("data/",ziekenhuis,"/"),
                                         full.names = TRUE
    )%>% tail(n=1)
    print(paste("Bestand opnamemutaties:", bestand_opnamemutaties))
    
    #read data
    tbl_opnames = read_delim(bestand_opnamemutaties, delim = ';', na = c("","NA","NULL")) 
    
    if(ziekenhuis == 'GHZ'){
      colnames(tbl_opnames) = c("Anonieme opnamecode","Herkomst","Bestemming" ,"Startdatumtijdmin" ,"Einddatumtijdmax","leeftijdID","geslacht","BMI","BehandelBeperking","IC opname"    )}
    
    #soms heet kolom Startdatumtijd en soms Startdatumtijdmin. We maken er in beide gevallen startdatumtijd (en einddatumtijd) van
    if ("Startdatumtijd" %in% colnames(tbl_opnames)){
      tbl_opnames = tbl_opnames  %>% mutate(startdatumtijd = rewrite_date(Startdatumtijd), einddatumtijd = rewrite_date(Einddatumtijd)) %>%
        select(-Startdatumtijd, -Einddatumtijd)
    } else if ("Startdatumtijdmin" %in% colnames(tbl_opnames)){
      tbl_opnames = tbl_opnames  %>% mutate(startdatumtijd = rewrite_date(Startdatumtijdmin), einddatumtijd = rewrite_date(Einddatumtijdmax))  %>% 
        select(-Startdatumtijdmin, -Einddatumtijdmax)
    }
    
  
    #als er geen info over behandelbeperking beschikbaar is, ga dan van "geen behandelbeperking" uit
    tbl_opnames$BehandelBeperking[is.na(tbl_opnames$BehandelBeperking)] = 'Geen behandelbeperking'
    
    # determining the last (valid) datetime of a mutation as the prediction date
    tbl_opnames$Bestemming[tbl_opnames$einddatumtijd >= Sys.time()] = NA   # setting enddates far away (e.g. 2999) to na   
    tbl_opnames$einddatumtijd[tbl_opnames$einddatumtijd >= Sys.time()] = NA   # setting enddates far away (e.g. 2999) to na   
    datetime_pred = max(c(tbl_opnames$einddatumtijd[!is.na(tbl_opnames$Bestemming)], tbl_opnames$startdatumtijd))
    tbl_opnames = tbl_opnames %>% mutate(einddatumtijd = as.POSIXct(einddatumtijd, tz = 'UTC'), startdatumtijd = as.POSIXct(startdatumtijd, tz = 'UTC'))
    tbl_opnames$Bestemming[is.na(tbl_opnames$Bestemming) & tbl_opnames$einddatumtijd < datetime_pred] = 'Onbekend'
    tbl_opnames$Bestemming[tbl_opnames$einddatumtijd >= as.POSIXct(datetime_pred)] = NA   # bestemming wordt NA (later gecorrigeerd tot  "op IC")
    tbl_opnames$einddatumtijd[tbl_opnames$einddatumtijd >= as.POSIXct(datetime_pred)] = NA   # setting enddates far away (e.g. 2999) to na   
    tbl_opnames = tbl_opnames %>% mutate(startdatum = as.Date(startdatumtijd), einddatum = as.Date(einddatumtijd))
    tbl_opnames = tbl_opnames %>% mutate(ligduur = sapply(difftime(tbl_opnames$einddatum , tbl_opnames$startdatum, unit = 'days') + 1, as.numeric))  # obtaining LOS
    
    #when patient is still there, the LOS is set to the LOS up to now
    tbl_opnames$ligduur[is.na(tbl_opnames$ligduur)] = sapply(difftime(as.Date(datetime_pred) , tbl_opnames$startdatum[is.na(tbl_opnames$ligduur)], unit = 'days') + 1, as.integer)
    
    
    mogelijke_bestemmingen = unique(tbl_opnames$Bestemming)
    bestemmingen_overleden = mogelijke_bestemmingen[grepl('Overleden', mogelijke_bestemmingen) | grepl('overleden', mogelijke_bestemmingen)]
    bestemmingen_naar_huis = c(mogelijke_bestemmingen[grepl('woonomgeving', mogelijke_bestemmingen) ], 'naar huis', 'Naar huis')
    bestemmingen_instelling = mogelijke_bestemmingen[grepl('instelling', mogelijke_bestemmingen)|
                                                       grepl('verzorgingshuis', mogelijke_bestemmingen) | 
                                                       grepl('Hospice', mogelijke_bestemmingen) | grepl('Instelling', mogelijke_bestemmingen)]
    
    
    #defining IC set as the set where IC data is filled in
    ICset = tbl_opnames %>% filter(`IC opname` %in% c('ja', 'Ja'))
    
    ICset$Bestemming[is.na(ICset$Bestemming)] = 'nog op IC' # defining destination when patient is still on IC
    
    #determining the departure path for each IC patient
    ICset = ICset %>% mutate(vertrek_type = NA)
    
    #bepaal vertrektype. 
    ICset$vertrek_type[((ICset$Bestemming == 'Naar Kliniek') | (ICset$Bestemming == 'Covid-19 Kliniek'))] = 'IC - vertrek naar Kliniek'
    ICset$vertrek_type[((ICset$Bestemming == 'nog op IC'))] = 'IC - Nog opgenomen'
    ICset$vertrek_type[ICset$Bestemming %in% c('Ander ziekenhuis','Covid-19 IC ziekenhuis X',
                                               "Algemeen Ziekenhuis"   ,  "Academisch Ziekenhuis")] = 'IC - ander ziekenhuis'
    
    ICset$vertrek_type[sapply(ICset$Bestemming, function(bst){ any(bestemmingen_overleden == bst)})] = 'IC - overleden'
    ICset$vertrek_type[sapply(ICset$Bestemming, function(bst){ any(bestemmingen_naar_huis == bst)})] = 'IC - naar huis'
    ICset$vertrek_type[sapply(ICset$Bestemming, function(bst){ any(bestemmingen_instelling == bst)})] = 'IC - overige instelling'
    ICset$vertrek_type[is.na(ICset$vertrek_type)] = 'IC - anders'
    
    #determining event variable for survival analysis
    ICset$event = ifelse(ICset$vertrek_type == "IC - ander ziekenhuis", 0,
                         ifelse(ICset$vertrek_type == "IC - Nog opgenomen", 0, 1))
    
    # df_ic = ICset %>% mutate(opname_type = ifelse(Herkomst != 'Van Kliniek'&
    #                                                 Herkomst != "Uit eigen ziekenhuis"&
    #                                                 Herkomst != "Ander ziekenhuis",  'indirect' , 'direct'))
    
    #determining arrivals directly to IC (not from other hospitals)
    Aankomst_IC = ICset %>% filter(Herkomst != 'Van Kliniek'&
                                     Herkomst != "Uit eigen ziekenhuis"&
                                     Herkomst != "Ander ziekenhuis") %>%
      select(startdatum) %>% 
      mutate(aantal = 1) %>%
      group_by(startdatum) %>% 
      summarise(aantal_ic = sum(aantal), .groups = 'drop') %>%
      ungroup()
    
    
    #defining kliniek set as the set where no IC data is specified
    Kliniekset = tbl_opnames %>% filter(`IC opname` %in% c('nee', 'Nee'))
    
    Kliniekset$Bestemming[is.na(Kliniekset$Bestemming)] = 'nog op kliniek'# defining destination when patient is still in clinic
    
    #determining the departure path for each clinic patient
    Kliniekset = Kliniekset %>% mutate(vertrek_type = NA)
    Kliniekset$vertrek_type[((Kliniekset$Bestemming %in% c('Naar IC', 'Covid-19 IC')) )] = 'Kliniek - vertrek naar IC'
    Kliniekset$vertrek_type[((Kliniekset$Bestemming == 'nog op kliniek') )] = 'Kliniek - Nog opgenomen'
    Kliniekset$vertrek_type[Kliniekset$Bestemming %in% c('Ander ziekenhuis', 'Covid-19 Kliniek ziekenhuis X',
                                                         "Algemeen Ziekenhuis"   ,  "Academisch Ziekenhuis") ] = 'Kliniek - ander ziekenhuis'
    
    Kliniekset$vertrek_type[sapply(Kliniekset$Bestemming, function(bst){ any(bestemmingen_overleden == bst)})] = 'Kliniek - overleden'
    Kliniekset$vertrek_type[sapply(Kliniekset$Bestemming, function(bst){ any(bestemmingen_naar_huis == bst)})] = 'Kliniek - naar huis'
    Kliniekset$vertrek_type[sapply(Kliniekset$Bestemming, function(bst){ any(bestemmingen_instelling == bst)})] = 'Kliniek - overige instelling'
    Kliniekset$vertrek_type[is.na(Kliniekset$vertrek_type)] = 'Kliniek - anders'
    
    #determining event variable for survival analysis
    Kliniekset$event <- ifelse(Kliniekset$vertrek_type == "Kliniek - ander ziekenhuis", 0,
                               ifelse(Kliniekset$vertrek_type == "Kliniek - Nog opgenomen", 0, 1))
    
    
    # df_kl = Kliniekset %>% mutate(opname_type = ifelse(Herkomst != 'Van IC'&
    #                                                      Herkomst != "Uit eigen ziekenhuis"&
    #                                                      Herkomst != "Ander ziekenhuis", 'direct', 'indirect'))
    # 
    #determining arrivals directly to clinic (not from other hospitals)
    Aankomst_kliniek = Kliniekset %>% filter(Herkomst != 'Van IC'&
                                               Herkomst != "Uit eigen ziekenhuis"&
                                               Herkomst != "Ander ziekenhuis") %>%
      select(startdatum) %>% 
      mutate(aantal = 1) %>%
      group_by(startdatum) %>% 
      summarise(aantal_kliniek = sum(aantal), .groups = 'drop') %>%
      ungroup()
    
    #merging the arrival information
    aankomst_totaal = tibble(startdatum = seq(min(c(Aankomst_IC$startdatum, Aankomst_kliniek$startdatum)), as.Date(datetime_pred), by = 'days'))
    aankomst_totaal = left_join(aankomst_totaal, Aankomst_IC, by = 'startdatum') %>% left_join(Aankomst_kliniek, by = 'startdatum')%>%
      replace_na(replace = list(aantal_ic = 0, aantal_kliniek = 0)) %>% rename(datum = startdatum)
    
    
    #haal extreem vroege aankomsten weg
    aankomst_totaal = aankomst_totaal[aankomst_totaal$datum >= as.Date('2020-03-03'),]
    
    aankomst_totaal = aankomst_totaal %>%
      add_column(aantal_cumulatief_ic = cumsum(aankomst_totaal$aantal_ic), .before = 3)
    
    aankomst_totaal = aankomst_totaal %>%
      add_column(aantal_cumulatief_kliniek = cumsum(aankomst_totaal$aantal_kliniek), .after  = 4)
    
    df_nieuwe_opnames = aankomst_totaal %>%
      add_column(aantal = aankomst_totaal$aantal_ic + aankomst_totaal$aantal_kliniek,
                 aantal_cumulatief = aankomst_totaal$aantal_cumulatief_ic +aankomst_totaal$aantal_cumulatief_kliniek, 
                 .before = 2)%>% 
      mutate(
        datum = date(datum),
        num = 1:nrow(.)
      )
    
  }

  bestemmingen_ic = c('IC', 'Naar IC', 'Covid-19 IC')
  bestemmingen_kl = c('Kliniek','Naar Kliniek','Covid-19 Kliniek')
  
  #sanity checks
  if(any(sapply(c(bestemmingen_naar_huis), function(str){str %in%ICset$Bestemming}))){cat(blue('mogelijke fout in data: patiënten gaan vanaf IC naar huis')) }
  assertthat::assert_that(!any(sapply(c(bestemmingen_ic), function(str){str %in%ICset$Bestemming})),msg =  'patiënten gaan vanaf IC naar IC' )
  assertthat::assert_that(!any(sapply(c(bestemmingen_kl), function(str){str %in%Kliniekset$Bestemming})),msg =  'patiënten gaan vanaf Kliniek naar Kliniek' )

  
  return(list(ICset = ICset,
              Kliniekset = Kliniekset,
              df_nieuwe_opnames = df_nieuwe_opnames, datetime_pred = datetime_pred))
  
}

