
voorspel_lgc <- function(mod, df, aantal_dagen = 7, level_pi = .80) {
  # Haal de naam van de afhankelijke variabele op
  dep_naam <- all.vars(summary(mod)$formula)[1]
  
  # Maak een nieuw data frame
  num <- 1:(nrow(df) + aantal_dagen)
  datum_nieuw <- map_dfr(list(x = 1:aantal_dagen), ~ max(df$datum) + days(.))
  datum <- c(df$datum, datum_nieuw)
  df_pred <- tibble(datum, num)
  
  # Voeg de afhankelijke variabele toe
  dep <- c(pull(df, dep_naam), rep(NA, aantal_dagen))
  df_pred[dep_naam] <- dep
  
  # Maak voorspellingen en transformeer ze terug
  # N.B.: Ik heb hier voor predictie-intervallen gekozen, dat lijkt me in dat geval geschikter dan
  # betrouwbaarheidsintervallen. Eens?
  pred <- predFit(mod, df_pred, interval = "prediction", level = level_pi) %>%
    as_tibble()
  pred$datum <- df_pred$datum
  pred$obs <- pull(df_pred, dep_naam)
  
  return(pred)
}


muteer_df_nieuwe_opnames <- function(df_nieuwe_opnames) {
  df <- df_nieuwe_opnames %>%
    mutate(num = 1:nrow(.))

  df$weekend <- ifelse(weekdays(df$datum) %in% c("zaterdag", "zondag"), 1, 0)

  return(df)
}

voorspel_instroom = function(df_nieuwe_opnames, aantal_dagen, scenarios, voorspeldatum, print_ = T, exclude_until = as.Date('0000-01-01'), data_besmettingen = NULL){
  #aantal dagen dat instroom vooruit voorspelt moet worden > aantal dagen dat de bezetting vooruit voorspelt moet worden
  aantal_dagen = max(0, as.integer(as.Date(voorspeldatum) - max(df_nieuwe_opnames$datum))) + aantal_dagen

  df_nieuwe_opnames = muteer_df_nieuwe_opnames(df_nieuwe_opnames)
  datum_nieuw <- map_dfr(list(x = 1:aantal_dagen), ~ max(df_nieuwe_opnames$datum) + days(.))
  nieuwe_opnames = tibble(datum = c(df_nieuwe_opnames$datum, datum_nieuw))

  # Richards curve ------------------------------------------------------------------------------
  if('log' %in% scenarios | 'log_upr' %in% scenarios | 'log_lwr' %in% scenarios){
    df_nieuwe_opnames_RC = df_nieuwe_opnames %>% filter(datum >= exclude_until)
    nieuwe_opnames = fit_curve_krijg_voorspellingen(df = df_nieuwe_opnames_RC, aantal_dagen, max_peaks = 2, var = 'aantal')
  }


  if('besmettingen' %in% scenarios){
    # # verkrijg besmettingsdata
    # df_besmettingen_tot = read_delim('https://data.rivm.nl/covid-19/COVID-19_aantallen_gemeente_per_dag.csv', delim = ';')
    #
    # #df van besmettingen en opnames voor de roaz regio
    # data_besmettingen = df_besmettingen_tot%>% filter(ROAZ_region == roaz_regio(ziekenhuis))%>%
    #   select( datum = Date_of_publication, Besmettingen = Total_reported)%>%
    #   group_by(datum)%>% summarise(Besmettingen = sum(Besmettingen), .groups = 'drop_last') %>% ungroup()
    # write.csv2(data_besmettingen, 'predictions/besmettingen_naz_west.csv')

    #haal besmettingen op
    data_besmettingen = data_besmettingen %>% select(datum = datum, Besmettingen ) %>%
                        mutate(datum = as.Date(datum))%>% filter(datum <= voorspeldatum & datum >= '2020-07-01')%>% add_column(num = seq(nrow(.)))
                        
     
    data_besmettingen = tail(data_besmettingen %>% mutate(Besmettingen = ma(Besmettingen, 7)), -7) # neem wekelijks gem
    # data_besmettingen = data_besmettingen %>% mutate(Besmettingen = ewma(Besmettingen,0.2)) # neem ewma
    
    # #fit een optimale vertraging en uitfiltering van de besmettingen op de bekende aankomstdata voor het ziekenhuis
    daadwerkelijke_instroom = df_nieuwe_opnames %>% select(datum, opnames = aantal)
    data_besmettingen2      = data_besmettingen %>% filter(as.Date(datum) %in% as.Date(daadwerkelijke_instroom$datum))
    daadwerkelijke_instroom = daadwerkelijke_instroom %>% filter(as.Date(datum) %in% as.Date(data_besmettingen$datum))
    
    lag_opt  = 7
    opnames  = tail(daadwerkelijke_instroom$opnames, -lag_opt)
    besm     = head(data_besmettingen2$Besmettingen, -lag_opt)
    
    weights = weights_exp(length(opnames), 1.2)
    coef_opt = as.numeric(t(besm)%*%diag(weights)%*%opnames/t(besm)%*%diag(weights)%*%besm)#OLS om uitfiltering te bepalen
    weights_old = rep(1, length(opnames))
    coef_opt_old = as.numeric(t(besm)%*%diag(weights_old)%*%opnames/t(besm)%*%diag(weights_old)%*%besm)#OLS om uitfiltering te bepalen

    #voeg voorspelling toe aan df
    voorsp = head(data_besmettingen$Besmettingen*coef_opt, -lag_opt)
    voorsp_old = head(data_besmettingen$Besmettingen*coef_opt_old, -lag_opt)
    dates  = tail(data_besmettingen$datum + days(lag_opt), -lag_opt)
    tbl_voorsp = tibble(datum = dates, besmettingen = voorsp, besmettingen_old = voorsp_old)
    nieuwe_opnames = suppressMessages(quiet(left_join(nieuwe_opnames, tbl_voorsp)))

  }

  if('vlak' %in% scenarios){
  # Vlak gemiddelde -----------------------------------------------------------------------------

  # Gemiddelde instroom laatste zeven dagen?
  gem_aantal_opnames <- nieuwe_opnames %>%
    drop_na(nieuwe_opn) %>%
    tail(7) %>%
    pull(nieuwe_opn) %>%
    mean()

  pred_vlak <- rep(gem_aantal_opnames, aantal_dagen)

  nieuwe_opnames$vlak <- c(rep(NA, nrow(nieuwe_opnames) - aantal_dagen), pred_vlak)}

  if('piek' %in% scenarios){
  # Nieuwe Richards curve -----------------------------------------------------------

  laatste_pred_nieuwe_opnames <- nieuwe_opnames %>%
    filter(datum == max(df_nieuwe_opnames$datum)) %>%
    pull(log)

  startpunt <- which(nieuwe_opnames$log > laatste_pred_nieuwe_opnames)[1]

  # Maak nieuwe voorspellingen door dezelfde Richards curve te fitten vanaf X dagen terug
  # (X = startpunt)
  pred_piek <- voorspel_lgc(lgc,  tail(df_nieuwe_opnames, startpunt), aantal_dagen) %>%
    voorspel_nieuwe_opnames() %>%
    pull(log) %>%
    tail(aantal_dagen + 1)

  nieuwe_opnames$piek <- c(
    rep(NA, nrow(nieuwe_opnames) - aantal_dagen - 1),
    pred_piek)}

  if('exp' %in% scenarios){
    # Exponentieel --------------------------------------------------------------------------------

    pred_exp <- map_dbl(1:aantal_dagen, ~ gem_aantal_opnames * (2^(1 / 9))^.x)

    nieuwe_opnames$exp <- c(rep(NA, nrow(nieuwe_opnames) - aantal_dagen), pred_exp)
  }
  voorspelde_opnames = nieuwe_opnames
  return(voorspelde_opnames)


}

