bepaal_verdeelsleutel = function(naam_strategie, simuleerdata_ziekenhuizen, simuleerdata_regio, huidig_tijdstip, theta_init, schaling_reg_aank, horizon = 3){
  if(naam_strategie == 'individuele ziekenhuizen'){
      # verdeelsleutel blijft hetzelfde
      verdeelsleutel             = theta_init
      return(list(verdeelsleutel = verdeelsleutel, theta_init = theta_init))
  }else{
    if(naam_strategie == 'regionaal ziekenhuis'){
      #alles gaat naar het regio ziekenhuis
      verdeelsleutel = ones(1,1)
      colnames(verdeelsleutel) = names(simuleerdata_ziekenhuizen)
      return(list(verdeelsleutel = verdeelsleutel, theta_init = theta_init))
    }else{
      if(naam_strategie == 'optimale verdeelsleutel'){
        # bij optimale verdeelsleutel verdelen we de aankomstintensiteit adhv theta. 
        # resultaat is een matrix met gelijke rijen, met elke kolom een vector met theta_i er in.
        set.seed(as.numeric(Sys.time()))
        
        # verkrijg besmettingsdata tot aan voorspelmoment en voorspel aankomsten vanaf begin voorspeldatum
        data_aankomsten = quiet(df_aankomsten %>% add_column(datum = as.Date(df_aankomsten[,2]), Opnames = 1) %>% select(c(datum, Opnames)) %>% group_by(datum) %>% summarise(datum = first(datum), 
                                                                                                                                                                              Opnames = sum(Opnames), .groups = 'drop_last') %>% ungroup())
        data_besmettingen = quiet(simuleerdata_regio$data_besmettingen %>% left_join(data_aankomsten, by = 'datum'))
        data_besmettingen$Opnames.y[is.na(data_besmettingen$Opnames.y)] = data_besmettingen$Opnames.x[is.na(data_besmettingen$Opnames.y)]
        data_besmettingen = data_besmettingen %>% select(-Opnames.x) %>% rename(Opnames = Opnames.y) %>% filter(datum <= as.Date(huidig_tijdstip))
        
        source("R/01_voorspel_instroom.R")
        voorspelde_regionale_instroom = quiet(voorspel_instroom(df_nieuwe_opnames = data_besmettingen %>% select(datum, aantal = Opnames) %>% mutate(datum = as.Date(datum)), aantal_dagen = 3, voorspeldatum = huidig_tijdstip, scenarios = 'besmettingen', data_besmettingen = data_besmettingen))%>% filter(datum >= max(data_besmettingen$datum))
        # voorspelde_regionale_instroom              = quiet(fit_besmettingen_opnames(data_besmettingen , horizon = min(7, horizon + 1 )) %>% filter(datum >= max(data_besmettingen$datum))) 
        voorspelde_regionale_instroom$besmettingen = voorspelde_regionale_instroom$besmettingen*schaling_reg_aank
        
        ##init
        thetas_zkh                    = zeros(1, length(simuleerdata_ziekenhuizen))
        colnames(thetas_zkh)          = names(simuleerdata_ziekenhuizen)
        
        if(all(voorspelde_regionale_instroom$besmettingen == 0)){
          #als er geen instroom voorspelt wordt kan het altijd, theta = 1 overal
          thetas_zkh = thetas_zkh + 1
        }else{
          #vind theta voor alle zkh
          for(ziekenhuis in names(simuleerdata_ziekenhuizen)){
            #verkrijg data ziekenhuis
            simuleerdata = simuleerdata_ziekenhuizen[[ziekenhuis]]
            
            #kijk of er uberhaupt plekken vrij zijn in zkh
            capaciteit_zkh = unlist(simuleerdata$capaciteit)
            bezetting_zkh = unlist(simuleerdata$bezetting)
            bedden_vrij_zkh = all(capaciteit_zkh > bezetting_zkh)
            
            if(bedden_vrij_zkh){
              #split in aankomsten IC en aankomsten kliniek, schaal op met zelfde constante als de echte aankomstintensiteit
              # tic()
              thetas_zkh[,ziekenhuis] = bereken_theta(simuleerdata, voorspelde_regionale_instroom, horizon, huidig_tijdstip, theta_init = theta_init[,ziekenhuis], print_ = F)
              # print(thetas_zkh[,ziekenhuis])
              # toc()
              # tic()
              # print(bereken_theta(simuleerdata, voorspelde_regionale_instroom, horizon, huidig_tijdstip, theta_init = theta_init[,ziekenhuis]))
              # toc()
            }else{
              thetas_zkh[,ziekenhuis] = 0
            }
          }
        }
        
        #als alle theta's 0 zijn, gebruiken we de vaste verdeelsleutel
        if(sum(thetas_zkh) == 0){
          verdeelsleutel = t(as.matrix(simuleerdata_regio$historische_verdeelsleutel ))
        }else{
          #normaliseer theta
          verdeelsleutel = thetas_zkh/sum(thetas_zkh)
        }
        return(list(verdeelsleutel = verdeelsleutel, theta_init = thetas_zkh))
      }
    }
  }
}