test_cvg_df_GR = function(df){
  #turn the df into a mcmc object list for assessing the convergence
  list_mcmc_obj = list()
  for(c in seq(ncol(df))){
    list_mcmc_obj = append(list_mcmc_obj, list(as.mcmc(df[,c])))
  }
  #convergence holds when the Gelman diagnostic is below 1.1
  flag = gelman.diag(as.mcmc.list(list_mcmc_obj))$psrf[1] < 1.1
  return(flag)
}


bereken_theta2 = function(simuleerdata, voorspelde_regionale_instroom, horizon, huidig_tijdstip, theta_init, print_ = F){
  # vind theta obv stochastische approximatie. Om niet-negativiteit te modelleren gebruiken we log van theta als parameter voor
  # het Robbins-Monro algoritme

  kans_naar_ic     = simuleerdata$kans_naar_ic
  capaciteit       = simuleerdata$capaciteit
  alpha_levels     = simuleerdata$alpha_levels

  #verkrijg arrival intensities en sla op
  preds_ic         = voorspelde_regionale_instroom %>% select(datum, besmettingen) %>% mutate(besmettingen = besmettingen*kans_naar_ic)
  preds_kl         = voorspelde_regionale_instroom %>% select(datum, besmettingen) %>% mutate(besmettingen = besmettingen*(1-kans_naar_ic))

  besm_ic = preds_ic$besmettingen
  besm_kl = preds_kl$besmettingen

  inner_M    = 2 # aantal simulaties dat per iteratie van stochastische approximatie uitgevoerd wordt
  nchains    = 2 #aantal Markov ketens
  log_thetas = list()
  for(chain in seq(nchains)){
    log_thetas = append(log_thetas, list(list(kliniek = log(max(1e-5, theta_init)),
                                              IC      = log(max(1e-5, theta_init)))))
    }

  vals_kl        = zeros(1,nchains)                  # huidige waarde residual kliniek (per keten)
  vals_ic        = zeros(1,nchains)                  # huidige waarde residual ic (per keten)
  log_thetas_ic  = zeros(1,nchains)                  # huidige waarde theta (per keten)
  log_thetas_kl  = zeros(1,nchains)                  # huidige waarde theta (per keten)

  num_test_cvg     = 250 # sample size om te testen op convergentie
  burn_in          = 0
  convg            = F   # convergentie vlag
  epsilon          = 1/4
  df_log_thetas_ic = c() # matrix met theta per iteratie (rijen) en keten (kolommen)
  df_log_thetas_kl = c() # matrix met theta per iteratie (rijen) en keten (kolommen)
  df_vals_kl       = c() # matrix met residuals per iteratie (rijen) en keten (kolommen)
  df_vals_ic       = c() # matrix met residuals per iteratie (rijen) en keten (kolommen)

  source('R/DES.R')
  k = 1
  while(!(convg | k > 1e4)){
    for(i in seq(nchains)){
      ## stochastische approximatie om chi_hW voor de ward (kliniek) te bepalen
      #genereer eerst een sample van theta_kl zdd prob(genoeg bedden kl onder theta_kl) >= alpha_kl
      theta_kl = exp(log_thetas[[i]]$kliniek)

      preds_ic$besmettingen = theta_kl*besm_ic
      preds_kl$besmettingen = theta_kl*besm_kl

      #verkrijg intensiteitsfuncties Poisson proces
      list_scenario_arrival_rates = get_arrival_rates(preds_ic = preds_ic, preds_kl = preds_kl, departure_info = simuleerdata$departure_info)

      #aankomstintensiteit bepalen voor evaluatie, Poisson proces voorspellingen uit besmettingen
      patienttype_info                          = simuleerdata$patienttype_info
      patienttype_info$pat_KL_IC$arrival_info   = list_scenario_arrival_rates$besmettingen$KL_IC
      patienttype_info$pat_KL_rest$arrival_info = list_scenario_arrival_rates$besmettingen$KL_rest
      patienttype_info$pat_IC_KL$arrival_info   = list_scenario_arrival_rates$besmettingen$IC_KL
      patienttype_info$pat_IC_rest$arrival_info = list_scenario_arrival_rates$besmettingen$IC_rest

      # draai DES, nu in for-loop aangezien we maar klein aantal iteraties hebben (dus hoeft niet parallel)
      simresults = list()
      for(m in seq(inner_M)){
        simresults = append(simresults, list(perform_DES(horizon, department_names =  c('Kliniek', 'IC'),
                                                         patienttype_info = patienttype_info,
                                                         current_patient_info = simuleerdata$current_patient_info, by_hour = F,
                                                         start_datetime = huidig_tijdstip)))
      }

      #krijg indicatoren genoeg bedden gegeven capaciteit die er is
      indc_genoeg_kl = sapply(simresults, function(tbl){all(tbl$Kliniek <= capaciteit$Kliniek)})

      #bepalen residual voor kliniek
      res_kl = mean(indc_genoeg_kl)- alpha_levels$Kliniek

      #bepalen nieuwe waarde van theta obv Robbins-Monro algoritme met 1/(k^(1/2 + epsilon)) als stapgrootte
      log_thetas[[i]]$kliniek     = log_thetas[[i]]$kliniek + k^(-(1/2+epsilon))*res_kl
      log_thetas_kl[,i]           = log_thetas[[i]]$kliniek
      vals_kl[,i]                 = res_kl

      ## stochastische approximatie om chi_{hI} voor de IC te bepalen
      #genereer een sample van theta_ic zdd prob(genoeg bedden ic onder theta_ic) >= alpha_ic
      theta_ic = exp(log_thetas[[i]]$IC)

      preds_ic$besmettingen = theta_ic*besm_ic
      preds_kl$besmettingen = theta_ic*besm_kl

      #verkrijg intensiteitsfuncties Poisson proces
      list_scenario_arrival_rates = get_arrival_rates(preds_ic = preds_ic, preds_kl = preds_kl, departure_info = simuleerdata$departure_info)

      #aankomstintensiteit bepalen voor evaluatie, Poisson proces voorspellingen uit besmettingen
      patienttype_info                          = simuleerdata$patienttype_info
      patienttype_info$pat_KL_IC$arrival_info   = list_scenario_arrival_rates$besmettingen$KL_IC
      patienttype_info$pat_KL_rest$arrival_info = list_scenario_arrival_rates$besmettingen$KL_rest
      patienttype_info$pat_IC_KL$arrival_info   = list_scenario_arrival_rates$besmettingen$IC_KL
      patienttype_info$pat_IC_rest$arrival_info = list_scenario_arrival_rates$besmettingen$IC_rest

      # draai DES, nu in for-loop aangezien we maar klein aantal iteraties hebben (dus hoeft niet parallel)
      simresults = list()
      for(m in seq(inner_M)){
        simresults = append(simresults, list(perform_DES(horizon, department_names =  c('Kliniek', 'IC'),
                                                         patienttype_info = patienttype_info,
                                                         current_patient_info = simuleerdata$current_patient_info, by_hour = F,
                                                         start_datetime = huidig_tijdstip)))
      }

      #krijg indicatoren genoeg bedden gegeven capaciteit die er is
      indc_genoeg_ic = sapply(simresults, function(tbl){all(tbl$IC <= capaciteit$IC)})

      #bepalen residual voor IC
      res_ic = mean(indc_genoeg_ic) - alpha_levels$IC

      #bepalen nieuwe waarde van theta obv Robbins-Monro algoritme met k^(-(1/2+epsilon)) als stapgrootte
      log_thetas[[i]]$IC     = log_thetas[[i]]$IC + k^(-(1/2+epsilon))*res_ic
      log_thetas_ic[,i]      = log_thetas[[i]]$IC
      vals_ic[,i]            = res_ic
    }
    
    df_log_thetas_kl  = rbind(df_log_thetas_kl, log_thetas_kl)
    df_log_thetas_ic  = rbind(df_log_thetas_ic, log_thetas_ic)
    df_vals_kl        = rbind(df_vals_kl, vals_kl)
    df_vals_ic        = rbind(df_vals_ic, vals_ic)

    #print (optional)
    if(print_){
      print(paste0('it = ',k))
      print(paste0('waarden theta kl: ',exp(log_thetas_kl)))
      print(paste0('waarden theta ic: ',exp(log_thetas_ic)))
    }
    
    #use convergence diagnostic on the residuals to determine whether the chains have converged to same stationary distribution
    if( nrow(df_vals_kl) > burn_in & mod(nrow(df_vals_kl), num_test_cvg) == 0){
      if(mean(tail(exp(df_log_thetas_kl), num_test_cvg)) < 1e-5 & mean(tail(exp(df_log_thetas_ic), num_test_cvg)) < 1e-5){
        convg = T #theta = 0
      }else{
        if(sd(tail(df_vals_kl)) > 1e-5 & sd(tail(df_vals_ic)) > 1e-5){
           convg = test_cvg_df_GR(tail(df_vals_kl, num_test_cvg)) &  test_cvg_df_GR(tail(df_vals_kl, num_test_cvg))  
           }else{
             convg = F
           }
        if(is.na(convg)){
          convg = F
        }}
    }
    k = k+1
  }
  #maak schatting voor theta
  theta = min( exp(mean(tail(df_log_thetas_ic, num_test_cvg))),  exp(mean(tail(df_log_thetas_kl), num_test_cvg)))
  return(theta)
}




bereken_theta = function(simuleerdata, voorspelde_regionale_instroom, horizon, huidig_tijdstip, theta_init, print_ = F){
  # vind theta obv stochastische approximatie. Om niet-negativiteit te modelleren gebruiken we log van theta als parameter voor
  # het Robbins-Monro algoritme

  kans_naar_ic     = simuleerdata$kans_naar_ic
  capaciteit       = simuleerdata$capaciteit
  alpha_levels     = simuleerdata$alpha_levels

  #verkrijg arrival intensities en sla op
  preds_ic         = voorspelde_regionale_instroom %>% select(datum, besmettingen) %>% mutate(besmettingen = besmettingen*kans_naar_ic)
  preds_kl         = voorspelde_regionale_instroom %>% select(datum, besmettingen) %>% mutate(besmettingen = besmettingen*(1-kans_naar_ic))

  besm_ic = preds_ic$besmettingen
  besm_kl = preds_kl$besmettingen

  inner_M    = 5 # aantal simulaties dat per iteratie van stochastische approximatie uitgevoerd wordt
  nchains    = 1 #aantal Markov ketens
  log_thetas = ones(1,nchains)*log(max(1e-5, theta_init))   # huidige waarde log theta's (per keten)
  vals       = zeros(1,nchains)                  # huidige waarde residual (per keten)

  num_test_cvg = 300 # sample size om te testen op convergentie
  convg        = F   # convergentie vlag
  df_thetas    = c() # matrix met theta per iteratie (rijen) en keten (kolommen)
  df_vals      = c() # matrix met residuals per iteratie (rijen) en keten (kolommen)

  source('R/DES.R')
  k = 1
  while(!(convg | k > 1e4)){
    for(i in seq(length(log_thetas))){
      theta                 = exp(log_thetas[i])
      preds_ic$besmettingen = theta*besm_ic
      preds_kl$besmettingen = theta*besm_kl

      #verkrijg intensiteitsfuncties Poisson proces
      list_scenario_arrival_rates = get_arrival_rates(preds_ic = preds_ic, preds_kl = preds_kl, departure_info = simuleerdata$departure_info)

      #aankomstintensiteit bepalen voor evaluatie, Poisson proces voorspellingen uit besmettingen
      patienttype_info                          = simuleerdata$patienttype_info
      patienttype_info$pat_KL_IC$arrival_info   = list_scenario_arrival_rates$besmettingen$KL_IC
      patienttype_info$pat_KL_rest$arrival_info = list_scenario_arrival_rates$besmettingen$KL_rest
      patienttype_info$pat_IC_KL$arrival_info   = list_scenario_arrival_rates$besmettingen$IC_KL
      patienttype_info$pat_IC_rest$arrival_info = list_scenario_arrival_rates$besmettingen$IC_rest

      
      current_patient_info = lapply(simuleerdata$current_patient_info, function(L){ 
        L$LOS = NULL
        L$seed = NULL
        L$departure_time = NULL
        return(L)})
      
      # draai DES, nu in for-loop aangezien we maar klein aantal iteraties hebben (dus hoeft niet parallel)
      simresults = list()
      for(m in seq(inner_M)){
        simresults = append(simresults, list(perform_DES(horizon, department_names =  c('Kliniek', 'IC'),
                                                    patienttype_info = patienttype_info,
                                                    current_patient_info = current_patient_info, by_hour = F,
                                                    start_datetime = huidig_tijdstip)))
      }

      #krijg indicatoren genoeg bedden gegeven capaciteit die er is
      indc_genoeg_ic = sapply(simresults, function(tbl){all(tbl$IC <= capaciteit$IC)})
      indc_genoeg_kl = sapply(simresults, function(tbl){all(tbl$Kliniek <= capaciteit$Kliniek)})

      #bepalen residual
      val = min(mean(indc_genoeg_ic) - alpha_levels$IC, mean(indc_genoeg_kl)- alpha_levels$Kliniek)
      vals[i] = val

      #bepalen nieuwe waarde van theta obv geprojecteerd (niet) Robbins-Monro algoritme met 1/sqrt(k) als stapgrootte
      log_theta     = log_thetas[i] + k^(-(1/2+0.01))*val
      log_thetas[i] = log_theta
    }

    if(print_){
      print(paste0('it = ',k))
      print(paste0('waarden theta: ',exp(log_thetas)))
      print(paste0('residuen: ',vals))
      print(paste0('mean resids: ', abs(mean(tail(df_vals, num_test_cvg))) ))
    }
    df_thetas = rbind(df_thetas, exp(log_thetas))
    df_vals = rbind(df_vals, vals)

    #use convergence diagnostic on the residuals to determine whether the two chains have converged to same stationary distribution
    if(dim(df_vals)[[1]]>2 & mod(dim(df_vals)[[1]], num_test_cvg) == 0){
      if(mean(tail(df_thetas, num_test_cvg)) < 1e-7){
        convg = T #theta = 0
      }else{
      df_mcmc = tail(df_vals, num_test_cvg)

      convg = abs(mean(tail(df_vals, 500))) < 0.01
      # list_mcmc_obj = list()
      # for(c in seq(dim(df_vals)[[2]])){
      #   list_mcmc_obj = append(list_mcmc_obj, list(as.mcmc(df_mcmc[,c])))
      # }
      # convg = gelman.diag(as.mcmc.list(list_mcmc_obj))$psrf[1] < 1.1
      if(is.na(convg)){
        convg = F
      }}
    }
    k = k+1
  }
  #maak schatting voor theta
  theta = mean(tail(df_thetas, num_test_cvg))
  return(theta)
}

if(sys.nframe() == 0){
  theta =  0.169390528383719

  M = 1000
  tol = 0.01
  kans_naar_ic     = simuleerdata$kans_naar_ic
  num_ziekenhuizen = length(simuleerdata_ziekenhuizen)
  capaciteit       = simuleerdata$capaciteit

  M = max(M, round(2/(tol^2)))

  #verkrijg arrival intensities en sla op
  preds_ic         = voorspelde_regionale_instroom %>% select(datum, besmettingen) %>% mutate(besmettingen = theta*besmettingen*kans_naar_ic)
  preds_kl         = voorspelde_regionale_instroom %>% select(datum, besmettingen) %>% mutate(besmettingen = theta*besmettingen*(1-kans_naar_ic))

  #verkrijg intensiteitsfuncties Poisson proces
  list_scenario_arrival_rates = get_arrival_rates(preds_ic = preds_ic, preds_kl = preds_kl, departure_info = simuleerdata$departure_info)

  #aankomstintensiteit bepalen voor evaluatie, Poisson proces voorspellingen uit besmettingen
  patienttype_info                          = simuleerdata$patienttype_info
  patienttype_info$pat_KL_IC$arrival_info   = list_scenario_arrival_rates$besmettingen$KL_IC
  patienttype_info$pat_KL_rest$arrival_info = list_scenario_arrival_rates$besmettingen$KL_rest
  patienttype_info$pat_IC_KL$arrival_info   = list_scenario_arrival_rates$besmettingen$IC_KL
  patienttype_info$pat_IC_rest$arrival_info = list_scenario_arrival_rates$besmettingen$IC_rest

  # draai DES
  source('R/DES.R')
  #maak voorspelling toekomstige bezetting
  cl <- makeCluster(cores)
  registerDoParallel(cl, cores=cores)
  simresults = foreach(icount(M), .combine = append, .export = c('perform_DES', 'rnhpois', 'sample_remaining_LOS',
                                                                 'first'), .packages = c('pracma','lubridate','dbplyr','tibble','tidyr','dplyr')) %dopar% {
                                                                   list(perform_DES(horizon, department_names =  c('Kliniek', 'IC'),
                                                                                    patienttype_info = patienttype_info,
                                                                                    current_patient_info = simuleerdata$current_patient_info,
                                                                                    start_datetime = huidig_tijdstip, by_hour = F))}
  stopCluster(cl)

  #krijg indicatoren genoeg bedden gegeven capaciteit die er is
  indc_genoeg_ic = sapply(simresults, function(tbl){all(tbl$IC <= capaciteit$IC)})
  indc_genoeg_kl = sapply(simresults, function(tbl){all(tbl$Kliniek <= capaciteit$Kliniek)})

  print(paste0('empirische kans genoeg bedden IC = ',mean(indc_genoeg_ic)))
  print(paste0('empirische kans genoeg bedden Kliniek = ',mean(indc_genoeg_kl)))

}
