rm(list=setdiff(ls(), c("df_besmettingen_tot", 'df_opnames_tot')))
# Laad packages en basisfuncties
source("R/utils.R") # basisfuncties + packages inladen
source("R/functies.R") # basisfuncties + packages inladen
source("R/functies_condprobs.R") #  basisfuncties + packages inladen

# verkrijg besmettingsdata en opnamedata
if(!exists('df_besmettingen_tot') | !exists('df_opnames_tot')){
  df_besmettingen_tot = read_delim('https://data.rivm.nl/covid-19/COVID-19_aantallen_gemeente_per_dag.csv', delim = ';')
  df_opnames_tot      = read_delim('https://data.rivm.nl/covid-19/COVID-19_ziekenhuisopnames.csv', delim = ';')
}

#init params
start_eval = as.Date('2020-09-01')
horizon = 3 #days
scenarios = c('log', 'besmettingen')
date = start_eval
overplaatsingen_er_uit = T

start_2e_golf = as.Date('2020-07-01')
ziekenhuis = 'Haga'
save_plots = T
plot       = T


# laad functie om data in te lezen, evalueer op ziekenhuis, LET OP: laadt RIVM data in dus zorg dat je internet hebt
source("R/00_maak_datasets.R", encoding = "UTF-8")
list2env(maak_datasets(ziekenhuis), envir = .GlobalEnv)
if(overplaatsingen_er_uit){
  res = haal_paden_in_uitplaatsingen_er_uit(Kliniekset, ICset)
  Kliniekset = res[[1]]
  ICset = res[[2]]
  arrival_dates = Kliniekset %>% filter(Herkomst %in% c("Eigen woonomgeving"))
  
  Aankomst_kliniek = Kliniekset %>% filter(Herkomst != 'Van IC'&
                                             Herkomst != "Uit eigen ziekenhuis"&
                                             Herkomst != "Ander ziekenhuis") %>%
    select(startdatum) %>% 
    mutate(aantal = 1) %>%
    group_by(startdatum) %>% 
    summarise(aantal_kliniek = sum(aantal), .groups = 'drop') %>%
    ungroup()
  
  Aankomst_IC = ICset %>% filter(Herkomst != 'Van Kliniek'&
                                   Herkomst != "Uit eigen ziekenhuis"&
                                   Herkomst != "Ander ziekenhuis") %>%
    select(startdatum) %>% 
    mutate(aantal = 1) %>%
    group_by(startdatum) %>% 
    summarise(aantal_ic = sum(aantal), .groups = 'drop') %>%
    ungroup()
  
  #merging the arrival information
  aankomst_totaal = tibble(startdatum = seq(min(c(Aankomst_IC$startdatum, Aankomst_kliniek$startdatum)), as.Date(datetime_pred), by = 'days'))
  aankomst_totaal = left_join(aankomst_totaal, Aankomst_IC, by = 'startdatum') %>% left_join(Aankomst_kliniek, by = 'startdatum')%>%
    replace_na(replace = list(aantal_ic = 0, aantal_kliniek = 0)) %>% rename(datum = startdatum)
  
  #haal extreem vroege aankomsten weg
  aankomst_totaal = aankomst_totaal[aankomst_totaal$datum >= as.Date('2020-03-03'),]
  
  aankomst_totaal = aankomst_totaal %>%
    add_column(aantal_cumulatief_ic = cumsum(aankomst_totaal$aantal_ic), .before = 3)
  
  aankomst_totaal = aankomst_totaal %>%
    add_column(aantal_cumulatief_kliniek = cumsum(aankomst_totaal$aantal_kliniek), .after  = 4)
  
  df_nieuwe_opnames = aankomst_totaal %>%
    add_column(aantal = aankomst_totaal$aantal_ic + aankomst_totaal$aantal_kliniek,
               aantal_cumulatief = aankomst_totaal$aantal_cumulatief_ic +aankomst_totaal$aantal_cumulatief_kliniek, 
               .before = 2)%>% 
    mutate(
      datum = date(datum),
      num = 1:nrow(.)
    )
}


# laad functies om regionale aankomsten te voorspellen op basis besmettingen, evalueer op nieuwe opnames
source("R/01_voorspel_instroom.R", encoding = "UTF-8")
source("R/code regionaal/voorspel_regionale_instroom.R", encoding = "UTF-8")
# df van besmettingen en opnames voor de roaz regio

#maak dfs met besmettingen en opnames om vanuit te voorspellen
df_nieuwe_opnames = df_nieuwe_opnames %>% mutate(EWMA_opn = ewma(aantal, 0.1)) %>% filter(datum >= start_2e_golf)
data_besmettingen = df_besmettingen_tot %>% filter(ROAZ_region == roaz_regio(ziekenhuis))%>%
  select( datum = Date_of_publication, Besmettingen = Total_reported, Opnames = Hospital_admission)%>%
  group_by(datum)%>% summarise(Besmettingen = sum(Besmettingen), Opnames = sum(Opnames), .groups = 'drop_last') %>% ungroup()%>% 
  filter(as.Date(datum) %in% df_nieuwe_opnames$datum) %>% add_column(num = 1:nrow(.)) %>% filter(datum >= start_2e_golf)

#haal NICE data opnames op
data_opnames     = df_opnames_tot %>% filter(Security_region_name %in% veiligheidsreg_per_roaz_regio[[roaz_regio(ziekenhuis)]])%>%
                                      rename(datum = Date_of_statistics, Opnames = Hospital_admission) %>%
                                      group_by(datum) %>% summarise(Opnames = sum(Opnames), .groups = 'drop_last') %>% ungroup()%>%
                                      filter(as.Date(datum) %in% df_nieuwe_opnames$datum) %>% filter(datum >= start_2e_golf)

data_besmettingen = data_besmettingen %>% select(-Opnames) %>% left_join(data_opnames)

df_nieuwe_opnames_regionaal = data_besmettingen %>% select(datum, aantal = Opnames, num) %>% add_column(aantal_cumulatief = cumsum(data_besmettingen$Opnames)) %>%
                                                    mutate(EWMA_opn = ewma(aantal, 0.1)) %>% filter(datum >= start_2e_golf)

#vind beste vertraging voor besmettingen vanuit regio
days_avg = 7
data_besmettingen =  tail(data_besmettingen %>% mutate(gem_besm = ma(Besmettingen, days_avg)) %>% mutate(gem_opn = ewma(Opnames, 0.1)),-days_avg) #neem gemiddelde afgelopen 7 dagen

MSE_min = Inf
lag_opt = 0
coef_opt = 1
for(lag in seq(2,14)){
  opnames = tail(data_besmettingen$gem_opn, -lag)
  besm = head(data_besmettingen$gem_besm, -lag)
  OLS_scale = as.numeric(t(besm)%*%opnames/t(besm)%*%besm)
  MSE = mean((opnames - besm*OLS_scale)^2)
  if(MSE < MSE_min){
    MSE_min = MSE
    lag_opt = lag
    coef_opt = OLS_scale
  }
}
lag = lag_opt
opnames = tail(data_besmettingen$Opnames, -lag)
besm = head(data_besmettingen$gem_besm, -lag)
OLS_scale = as.numeric(t(besm)%*%opnames/t(besm)%*%besm)
df_fit_aankomsten = tibble(datum = tail(data_besmettingen$datum, -lag_opt), opnames = tail(data_besmettingen$Opnames, -lag_opt),
                           besmettingen = head(data_besmettingen$gem_besm, -lag_opt)*coef_opt, gem_opn = ewma(opnames, 0.1))


#plot regionale aankomsten vs besmettingen
plot_besm_regio  = ggplot(data_besmettingen %>% filter(as.Date(datum) >= as.Date('2020-09-04') & as.Date(datum) <= as.Date('2021-01-31')), aes(x = datum)) + 
  geom_line(aes(y = gem_besm), col = 'red', size = 1.2) + geom_point(aes(y = Besmettingen), size = 1.2)+
  theme(legend.position = 'none')+ ylab('Regional infections') + xlab('date')+  theme_bw()+
  scale_y_continuous(breaks = integer_breaks())+ylim(c(0, max(as.data.frame(data_besmettingen %>% select(-datum )))+2))

plot_fit_aankomsten = ggplot(df_fit_aankomsten %>% filter(as.Date(datum) >=  as.Date('2020-09-04') & as.Date(datum) <= as.Date('2021-01-31')), aes(x  = datum)) +
  geom_line(aes(y = gem_opn), col = 'red', size = 1.2) + 
  geom_line(aes(y = besmettingen), col= 'purple', size = 0.6)+
  geom_point(aes(y = opnames), size = 1.2)+
  theme(legend.position = 'none')+ ylab('Regional hospitalisations') + xlab('date')+  theme_bw()+
  scale_y_continuous(breaks = integer_breaks())+ylim(c(0, max(as.data.frame(df_fit_aankomsten %>% select(-datum )))+2))

if(save_plots){
  ggsave(paste0('R/data analyse/data/',roaz_regio(ziekenhuis),'_infections.pdf'),plot_besm_regio, device = cairo_pdf, width = 125*3/4, height = 125*3/4, units = 'mm')
  ggsave(paste0('R/data analyse/data/',roaz_regio(ziekenhuis),'_hospitalizations_fit_tot.pdf'),plot_fit_aankomsten, device = cairo_pdf, width = 125*3/4, height = 125*3/4, units = 'mm')
}



end_date = min(as.Date('2021-01-31') , max(df_nieuwe_opnames$datum))
#voorspel aankomsten bevestigd positief COVID-19 patienten expanding window 
dates = c()
voorspellingen_regio = c()
voorspellingen_ziekenhuis = c()
while(date <= end_date - days(horizon)){
  print(date)
  dates = c(dates, toString(date + days(horizon)))#datums voorspellen
  #expanding window voor regio
  voorspellingen_regio = rbind(voorspellingen_regio, tail(voorspel_instroom(df_nieuwe_opnames = df_nieuwe_opnames_regionaal %>% filter(datum <= date),
                                                                            aantal_dagen = horizon, scenarios, voorspeldatum = date,
                                                                            exclude_until = start_2e_golf), 1) %>% select(log, besmettingen))
  
  #expanding window voor zkh
  voorspellingen_ziekenhuis = rbind(voorspellingen_ziekenhuis, tail(voorspel_instroom(df_nieuwe_opnames = df_nieuwe_opnames %>% filter(datum <= date),
                                                                                      aantal_dagen = horizon, scenarios, voorspeldatum = date, exclude_until = start_2e_golf), 1) %>% select(log, besmettingen))
  
  #update s
  date = date + days(1)
}

df_voorspellingen_regio = as_tibble(voorspellingen_regio) %>% add_column(datum = as.Date(dates), .before = 1) %>%
                        add_column( realisatie = (df_nieuwe_opnames_regionaal %>% filter(datum %in% as.Date(dates)))$aantal)%>%
                          add_column(ewma_opn = (df_nieuwe_opnames_regionaal %>% filter(datum %in% as.Date(dates)))$EWMA_opn) %>% rename(voorspellingen_RC = log, voorspellingen_besmettingen = besmettingen)
  
#plot regionale aankomsten vs besmettingen
plot_regional = ggplot(df_voorspellingen_regio, aes(x = datum)) + geom_line(aes(y = ewma_opn), col = 'red', size = 1.2) + geom_point(aes(y = realisatie), size = 1.2)+
  geom_line(aes(y = voorspellingen_RC), col = 'orange', size = 1.2)+
               geom_line(aes(y = voorspellingen_besmettingen), col = 'purple', size = 0.6) +  theme_bw()+
  theme(legend.position = 'none')+ ylab('Regional hospitalisations') + xlab('date')+
  scale_y_continuous(breaks = integer_breaks())+ylim(c(0, max(as.data.frame(df_voorspellingen_regio %>% select(-datum )))+2))

if(save_plots){
 ggsave(paste0('R/data analyse/data/',roaz_regio(ziekenhuis),'_arrivals_and_pred_infection.pdf'),plot_regional, device = cairo_pdf, width = 125*3/4, height = 125*3/4, units = 'mm')
}
if(plot){
  ggplotly(plot_regional)
}

df_voorspellingen_zkh = as_tibble(voorspellingen_ziekenhuis) %>% add_column(datum = as.Date(dates), .before = 1) %>% add_column( realisatie = (df_nieuwe_opnames %>% filter(datum %in% as.Date(dates)))$aantal)%>%
  add_column(ewma_opn = (df_nieuwe_opnames %>% filter(datum %in% as.Date(dates)))$EWMA_opn) %>% rename(voorspellingen_RC = log, voorspellingen_besmettingen = besmettingen)


#plot regionale aankomsten vs besmettingen
plot_hospital  = ggplot(df_voorspellingen_zkh, aes(x = datum)) + geom_line(aes(y = ewma_opn), col = 'red', size = 1.2) + geom_point(aes(y = realisatie), size = 1.2)+
               geom_line(aes(y = voorspellingen_RC), col = 'orange',  size = 1.2) +
             geom_line(aes(y = voorspellingen_besmettingen), col = 'purple', size = 0.6)+
             theme_bw()+
           theme(legend.position = 'none')+ ylab('Arrivals to hospital') + xlab('date')+
           scale_y_continuous(breaks = integer_breaks())+ylim(c(0, max(as.data.frame(df_voorspellingen_zkh %>% select(-datum, -realisatie )))+2))

if(save_plots){ggsave(paste0('R/data analyse/data/',ziekenhuis,'_arrivals_and_pred_infection.pdf'),plot_hospital, device = cairo_pdf, width = 125*3/4, height = 125*3/4, units = 'mm')}
if(plot){ggplotly(plot_hospital)}


