source('R/utils.R')
require(reshape2)
require(tibbletime)
library(xtable)
library(scales)
function_overbedden = function(v){mean(sapply(v,function(x){max(x,0)}))}

library(tables)
Sys.setlocale("LC_ALL", "Dutch")
genereer_namen_verplaatsingen_tussen_cl = function(vec_regios_cluster){
  vec_namen_verplaatsingen_tussen_cl = c()
  for(naam_regio_1 in vec_regios_cluster){
    for(naam_regio_2 in vec_regios_cluster[vec_regios_cluster != naam_regio_1]){
      vec_namen_verplaatsingen_tussen_cl = c(vec_namen_verplaatsingen_tussen_cl,
                                             paste0('van ', naam_regio_1, ' naar ', naam_regio_2))
    }
  }
  return(vec_namen_verplaatsingen_tussen_cl)
}

path = './regionale_info/resultaten spreiding regionaal/simulaties/resultaten stoch prog/'
files <- list.files(path = path ,pattern = '.rds')
namen_strategien = c('plek', 'CHOIR')
namen_engels = c('plek' = 'a', 'beddenaantal'='Num. Beds', 'CHOIR' = 'b')
namen_nl = c('plek' = 'Vrij bed', 'beddenaantal'='Aantal vrije bedden', 'CHOIR' = 'Optimalisatie')
kpis_tot = list()
save_results = T
upper_bound_sims = 250
level = 0.9
#params
clusters = list(c('Acute Zorgregio Oost', "Traumazorgnetwerk Midden-Nederland"),
                  c('Netwerk Acute Zorg West', 'Netwerk Acute Zorg Brabant'))
geen_clusters = list(c('Acute Zorgregio Oost',  'Netwerk Acute Zorg West'),
                     c('Acute Zorgregio Oost',  'Netwerk Acute Zorg Brabant'),
                     c("Traumazorgnetwerk Midden-Nederland", 'Netwerk Acute Zorg West'),
                     c("Traumazorgnetwerk Midden-Nederland", 'Netwerk Acute Zorg Brabant'))

kolomnamen_verpl_binnen_cl = as.vector(sapply(clusters, function(v){genereer_namen_verplaatsingen_tussen_cl(v)}))
kolomnamen_verpl_tussen_cl = as.vector(sapply(geen_clusters, function(v){genereer_namen_verplaatsingen_tussen_cl(v)}))

for(filename in files){
  loadres = load_obj(paste0(path, filename))
  for(naam_strategie in namen_strategien){
    kpis_strat = c()
    res  =loadres$lijst_info[[naam_strategie]]
    for(lijst_info_strat in res ){
      verplaatsingen_ic_binnen_clusters =  sum(lijst_info_strat$overplaatsingen_ic %>% select(kolomnamen_verpl_binnen_cl))
      verplaatsingen_kl_binnen_clusters =  sum(lijst_info_strat$overplaatsingen_kl %>% select(kolomnamen_verpl_binnen_cl))
      
      verplaatsingen_ic_tussen_clusters =  sum(lijst_info_strat$overplaatsingen_ic %>% select(kolomnamen_verpl_tussen_cl))
      verplaatsingen_kl_tussen_clusters =  sum(lijst_info_strat$overplaatsingen_kl %>% select(kolomnamen_verpl_tussen_cl))
      
      verplaatsingen_ic_outer =  sum(lijst_info_strat$overplaatsingen_ic %>% select(contains('Buitenland')))
      verplaatsingen_kl_outer = sum(lijst_info_strat$overplaatsingen_kl %>% select(contains('Buitenland')))
      
      verplaatsingen_ic_tot =  sum(lijst_info_strat$overplaatsingen_ic %>% select(-datum))
      verplaatsingen_kl_tot = sum(lijst_info_strat$overplaatsingen_kl %>% select(-datum))
      
      kpis_strat = rbind(kpis_strat, c(verplaatsingen_ic_tot, verplaatsingen_ic_outer, verplaatsingen_ic_binnen_clusters,verplaatsingen_ic_tussen_clusters,
                                       verplaatsingen_kl_tot, verplaatsingen_kl_outer, verplaatsingen_kl_binnen_clusters,verplaatsingen_kl_tussen_clusters
                                       ))
    }
    
    kpis_tot[[naam_strategie]] = rbind(kpis_tot[[naam_strategie]], kpis_strat)
    if(nrow(kpis_tot[[naam_strategie]]) > upper_bound_sims){
      kpis_tot[[naam_strategie]] = head(kpis_tot[[naam_strategie]], upper_bound_sims)
    }
  }  
}

tbl_ic = tibble()
tbl_kl = tibble()
for(naam_strategie in namen_strategien){
  num_rows = dim(kpis_tot[[naam_strategie]])[[1]]
  longtbl = as_tibble(as.numeric(kpis_tot[[naam_strategie]] )) %>% rename(Overplaatsingen = value) %>% add_column(afdeling = rep(c('IC', 'Kliniek'), each = 4*num_rows),
                                                                              strategie = namen_engels[naam_strategie], KPI = rep(rep(c('Total',  'External Region', 
                                                                                                                                'Inside clusters', 'Between clusters'), each = num_rows),2))
  
  longtbl_ic = longtbl %>% filter(afdeling == 'IC') %>% select(-afdeling)
  longtbl_kl = longtbl %>% filter(afdeling == 'Kliniek') %>% select(-afdeling)
  tbl_ic = bind_rows(tbl_ic, longtbl_ic)
  tbl_kl = bind_rows(tbl_kl, longtbl_kl)
}

library(ggplot2)
vgf = 1.3
plot_kl= ggplot(tbl_kl , aes(x = factor(KPI, levels = c('Total',  'External Region', 
                                                         'Inside clusters', 'Between clusters')), y = Overplaatsingen, color = strategie)) + geom_boxplot(size =  1) +
  theme_bw()+theme(legend.position = 'none') + ylab('Ward Reallocations')+xlab('')
ggsave(paste0('R/data analyse/data/boxplot_res_spreiding_regios_kl.png'),plot_kl, width = 6, height = 3, units = 'in', dpi= 300)

plot_ic = ggplot(tbl_ic , aes(x =  factor(KPI, levels = c('Total',  'External Region', 
                                                          'Inside clusters', 'Between clusters')), y = Overplaatsingen, color = strategie)) + geom_boxplot(size =  1) +
  theme_bw()+theme(legend.position = 'none') + ylab('ICU Reallocations')+xlab('')
ggsave(paste0('R/data analyse/data/boxplot_res_spreiding_regios_ic.png'),plot_ic, width = 6, height = 3, units = 'in', dpi = 300)



path = './regionale_info/resultaten spreiding ziekenhuizen/simulaties original/'
files <- list.files(path = path ,pattern = '.rds')
namen_strategien = c('individuele ziekenhuizen', 'optimale verdeelsleutel')
namen_engels = c('individuele ziekenhuizen' = 'a', 'optimale verdeelsleutel'='b')
kpis_tot = list()
save_results = T
upper_bound_sims = 250


for(filename in files){
  loadres = load_obj(paste0(path, filename))
  for(naam_strategie in namen_strategien){
    kpis_strat = c()
    res_tot  =loadres$simulatie_resultaten[[naam_strategie]]
    for(res in res_tot ){
      dagen_overbed_kl_detail = apply(res$df_bezetting %>% select(contains('Kliniek')) > res$df_capaciteit%>% select(contains('Kliniek')), 2, sum)
      dagen_overbedden_kl_tot = sum(apply(res$df_bezetting %>% select(contains('Kliniek')) > res$df_capaciteit%>% select(contains('Kliniek')), 1, any))
      
      overbedden_kl_detail = apply(res$df_bezetting %>% select(contains('Kliniek')) - res$df_capaciteit%>% select(contains('Kliniek')), 2, function_overbedden)
      overbedden_kl_tot    = sum(overbedden_kl_detail)
      
      
      dagen_overbed_ic_detail = apply(res$df_bezetting %>% select(contains('IC')) > res$df_capaciteit%>% select(contains('IC')), 2, sum)
      dagen_overbedden_ic_tot = sum(apply(res$df_bezetting %>% select(contains('IC')) > res$df_capaciteit%>% select(contains('IC')), 1, any))
      
      overbedden_ic_detail = apply(res$df_bezetting %>% select(contains('IC')) - res$df_capaciteit%>% select(contains('IC')), 2, function_overbedden)
      overbedden_ic_tot    = sum(overbedden_kl_detail)
      kpis_strat_kl = rbind(kpis_strat, c(overbedden_kl_detail, overbedden_kl_tot))
      kpis_strat_ic = rbind(kpis_strat, c( overbedden_ic_detail, overbedden_ic_tot))
    }
    
    kpis_tot[[naam_strategie]][['Kliniek']] = rbind(kpis_tot[[naam_strategie]][['Kliniek']], kpis_strat_kl)
    kpis_tot[[naam_strategie]][['IC']] = rbind(kpis_tot[[naam_strategie]][['IC']], kpis_strat_ic)
  }  
}

colnames(kpis_tot$`individuele ziekenhuizen`$Kliniek) = c('LUMC', 'Haga','GHZ','Region')
colnames(kpis_tot$`individuele ziekenhuizen`$IC) = c('LUMC', 'Haga','GHZ','Region')
colnames(kpis_tot$`optimale verdeelsleutel`$Kliniek) = c('LUMC', 'Haga','GHZ','Region')
colnames(kpis_tot$`optimale verdeelsleutel`$IC) = c('LUMC', 'Haga','GHZ','Region')

tbl_indiv = as_tibble(kpis_tot$`individuele ziekenhuizen`$Kliniek) %>% add_column(count = seq(nrow(.))) %>% pivot_longer(-count) %>% add_column(strategie = namen_engels[1]) %>% rename(KPI = name)
tbl_LB = as_tibble(kpis_tot$`optimale verdeelsleutel`$Kliniek) %>% add_column(count = seq(nrow(.))) %>% pivot_longer(-count) %>% add_column(strategie = namen_engels[2])%>% rename(KPI = name)
tbl_kl = rbind(tbl_indiv, tbl_LB)

plot_kl = ggplot(tbl_kl , aes(x = factor(KPI, levels = c('LUMC', 'Haga','GHZ','Region')), y = value, color = strategie)) + geom_boxplot(size =  1) +
  theme_bw()+theme(legend.position = 'none') + ylab('Ward Overbeds per Day')+xlab('') 
ggsave(paste0('R/data analyse/data/boxplot_res_spreiding_ziekenhuizen_kl.png'),plot_kl, width = 6, height = 3, units = 'in', dpi= 300)


tbl_indiv = as_tibble(kpis_tot$`individuele ziekenhuizen`$IC) %>% add_column(count = seq(nrow(.))) %>% pivot_longer(-count) %>% add_column(strategie = namen_engels[1]) %>% rename(KPI = name)
tbl_LB = as_tibble(kpis_tot$`optimale verdeelsleutel`$IC) %>% add_column(count = seq(nrow(.))) %>% pivot_longer(-count) %>% add_column(strategie = namen_engels[2])%>% rename(KPI = name)
tbl_ic = rbind(tbl_indiv, tbl_LB)

plot_ic = ggplot(tbl_ic , aes(x = factor(KPI, levels = c('LUMC', 'Haga','GHZ','Region')), y = value, color = strategie)) + geom_boxplot(size =  1) +
  theme_bw()+theme(legend.position = 'none') + ylab('ICU Overbeds per Day')+xlab('') 
ggsave(paste0('R/data analyse/data/boxplot_res_spreiding_ziekenhuizen_ic.png'),plot_ic, width = 6, height = 3, units = 'in', dpi= 300)
