rm(list=ls())
# source('R/code regionaal/verkrijg_info_spreiding_ziekenhuizen.R')   [warning: this script loads/overrides data~!]
source('R/utils.R')
source('R/DES.R')
source('R/get_system_information.R')
source('R/code_regionaal/voorspel_regionale_instroom.R')
source('R/code_regionaal/bepaal_verdeelsleutel.R')
source('R/code_regionaal/bereken_theta.R')

doe_simulatie = function(huidig_tijdstip, end_evaluatie, strategie_verdeelsleutel, m, simuleerdata_ziekenhuizen, simuleerdata_regio, theta_init,data_start_zkh,
                         schaling_reg_aank, timenum, df_aankomsten, ziekenhuizen_regio, lijst_zkh, df_capaciteit, df_bezetting, dates, df_verdeelsleutel, simuleerdata_init, 
                         randomseed, simulatie_resultaten, horizon, toevg_1,store = F){
  while(huidig_tijdstip <= end_evaluatie+ hours(10)){
    
    file_progress = paste0('progress_spreiding_ziekenhuizen_alpha_LUMC_', toString(data_start_zkh$LUMC$alpha_levels$Kliniek),
                    '_alpha_Haga_', toString(data_start_zkh$Haga$alpha_levels$Kliniek), "_alpha_GHZ_", toString(data_start_zkh$GHZ$alpha_levels$Kliniek),
                    "_hor=",toString(horizon),'.txt')
    write(paste0('sim = ', m,', strategie: ',strategie_verdeelsleutel, ', datum = ',huidig_tijdstip ), paste0('regionale_info/simulaties spreiding regionaal/',file_progress))
    
    ##bepaal verdeling aankomstintensiteit, res_verdeelsl is een matrix met kolommen het aantal beschouwde ziekenhuizen in de simulatie
    # de rijen horen bij ziekenhuizen in de regio, voor simulatie van aankomsten bij elk beschouwd ziekenhuis wordt de bijbehorende kolom genomen
    # daarin staan dan gewichten die een gewogen som bepalen van aankomstintensiteit bij ziekenhuizen in de regio, die de aankomstintensiteit bij beschouwd ziekenhuis bepalen
    temp_randomseed = .Random.seed
    print(huidig_tijdstip)
    res_verdeelsl   = bepaal_verdeelsleutel(strategie_verdeelsleutel, simuleerdata_ziekenhuizen, 
                                            simuleerdata_regio, huidig_tijdstip, theta_init, schaling_reg_aank, horizon = horizon)
    #zet seed weer terug
    set.seed(timenum)
    .Random.seed = temp_randomseed
    
    #verdeel aankomsten over ziekenhuizen
    verdeelsleutel              = res_verdeelsl$verdeelsleutel
    aankomsten_tussen_nu_morgen = df_aankomsten[(df_aankomsten[,2] > huidig_tijdstip) & (df_aankomsten[,2] <= huidig_tijdstip + days(1)), ]
    toewijzing_zkh              = t(rmultinom(nrow(aankomsten_tussen_nu_morgen), 1, as.numeric(verdeelsleutel)))
    colnames(toewijzing_zkh)    = colnames(verdeelsleutel)
    
    # initiele theta voor volgende sim is de theta van deze sim
    theta_init        = res_verdeelsl$theta_init 
    df_verdeelsleutel = rbind(df_verdeelsleutel, theta_init)
    
    #check dat de verdeelsleutel de totale aankomstintensiteit verdeeld
    assertthat::assert_that(norm(apply(res_verdeelsl$verdeelsleutel,1,sum) - ones(length(ziekenhuizen_regio),1))<1e-8)
    for(ziekenhuis in lijst_zkh[[strategie_verdeelsleutel]]){
      simuleerdata_zkh                = simuleerdata_ziekenhuizen[[ziekenhuis]]
      
      #bepaal pat die naar dit zkh gaan
      aankomsten_tussen_nu_morgen_zkh = aankomsten_tussen_nu_morgen[toewijzing_zkh[,ziekenhuis] == 1,]
      
      #stel voor elke pat een vaste aankomsttijd in
      patienttype_info                          = simuleerdata_zkh$patienttype_info
      if(nrow(aankomsten_tussen_nu_morgen_zkh)>0){
        for(ptype in names(patienttype_info)){
          aankomsten_ptype = sort(as.numeric(difftime(aankomsten_tussen_nu_morgen_zkh[sapply(aankomsten_tussen_nu_morgen_zkh[,1], function(name){grepl(name, ptype)}),2], as.Date(huidig_tijdstip), unit = 'day')))
          patienttype_info[[ptype]]$arrival_info = list(cuml_intensity_NHPP = get_function_next(aankomsten_ptype),
                                                        icuml_intensity_NHPP = Inf)
        }}else{
          for(ptype in names(patienttype_info)){
            patienttype_info[[ptype]]$arrival_info = zero_intensity_info()
          }
        }
      
      #simuleer 1 dag vooruit 
      simresult = perform_DES(horizon=1, department_names = c('Kliniek', 'IC'), patienttype_info = patienttype_info, 
                              current_patient_info = simuleerdata_zkh$current_patient_info, 
                              start_datetime = huidig_tijdstip, return_list_pat_sys = T, by_hour = F)
      ##volgende dag
      # nieuwe gedetailleerde bezetting bepalen.
      # voor alle patiënten is de departure time relatief tot vorig simuleermoment uitgerekend, daarom trekken we er voor de volgende dag (nu dus) 1 dag vanaf
      current_patient_info = list()
      for(patient in simresult$remaining_patients){
        patient$departure_time = patient$departure_time-1 
        patient$LOS_init      = patient$LOS_init+1
        current_patient_info = append(current_patient_info , list(patient))
      }
      
      #bereken nieuwe bezetting
      current_patient_info_kl   = current_patient_info[lapply(current_patient_info, function(list2){list2$department}) == 'Kliniek']
      bezetting_kl_zkh          = length(current_patient_info_kl)
      
      current_patient_info_ic   = current_patient_info[lapply(current_patient_info, function(list2){list2$department}) == 'IC']
      bezetting_ic_zkh          = length(current_patient_info_ic)
      
      current_patient_info      = append(current_patient_info_ic, current_patient_info_kl)
      
      #update simuleerdata
      simuleerdata_ziekenhuizen[[ziekenhuis]]$bezetting            = list(IC = length(current_patient_info_ic),
                                                                          Kliniek = length(current_patient_info_kl)) 
      
      simuleerdata_ziekenhuizen[[ziekenhuis]]$current_patient_info = current_patient_info #update data huidige pat
    }
    
    #update tijdstip
    huidig_tijdstip = huidig_tijdstip + days(1)
    
    #sla bezetting over tijd op in df
    bezetting_dag = c() 
    for(ziekenhuis in lijst_zkh[[strategie_verdeelsleutel]]){
      #bereken huidige bezetting in regio
      bezetting_dag = c(bezetting_dag, c( simuleerdata_ziekenhuizen[[ziekenhuis]]$bezetting$IC, 
                                          simuleerdata_ziekenhuizen[[ziekenhuis]]$bezetting$Kliniek))
    }
    df_bezetting   = rbind(df_bezetting, bezetting_dag)
    df_capaciteit  = rbind(df_capaciteit, as_tibble((t(unlist(data_start_zkh)))) %>% select(contains('.capaciteit.') &
                                                                                            contains(names(simuleerdata_init))))
    
    #hou overplaatsingen van herkomst naar bestemming bij
    dates                 = c(dates, toString(huidig_tijdstip))
    
    if(store){
      list_state = mget(c('huidig_tijdstip', 'end_evaluatie', 'strategie_verdeelsleutel', 'm', 'simuleerdata_ziekenhuizen', 'simuleerdata_regio', 'theta_init',
                         'schaling_reg_aank', 'timenum', 'df_aankomsten', 'ziekenhuizen_regio', 'lijst_zkh', 'df_capaciteit', 'df_bezetting', 'dates', 
                         'df_verdeelsleutel', 'data_start_zkh','simuleerdata_init', 'randomseed','simulatie_resultaten'))
      save(list_state, file = paste0('regionale_info/simulaties spreiding regionaal/temp_',strategie_verdeelsleutel,'_',schaling_reg_aank,toevg_1,'.rds'))
    }
  }
  return(mget(c('df_verdeelsleutel', 'dates', 'df_bezetting' , 'df_capaciteit', 'timenum')))
}

Mstart = 1
Mtot = 250#aantal scenarios

#params
regio                  = 'Netwerk Acute Zorg West'
ziekenhuizen_regio     = c('LUMC','Haga', 'GHZ')  # welke ziekenhuizen zitten in de regio? 
lijst_zkh              = list('individuele ziekenhuizen' = ziekenhuizen_regio,
                              'optimale verdeelsleutel' = ziekenhuizen_regio,
                              'regionaal ziekenhuis' = c('Netwerk Acute Zorg West als ziekenhuis' )) #onder welk scenario bekijken we welke ziekenhuizen?
strategien_verdeelsleutel = c('optimale verdeelsleutel' )#'optimale verdeelsleutel')#

evaluatie_horizon      = 5        #dagen 
end_evaluatie          = as.Date('2021-01-07')
start_evaluatie        = end_evaluatie - days(evaluatie_horizon) + hours(10)
tussentijds_opslaan    = T
store_1sim             = T

#lijst met huidige bezetting en capaciteit
data_start_zkh                 = list(LUMC   = list(bezetting    = list(IC = 7,   Kliniek = 22),
                                                    capaciteit   = list(IC = 11,  Kliniek = 29),
                                                    alpha_levels = list(IC = 0.9, Kliniek = 0.9)),
                                      Haga   = list(bezetting    = list(IC = 13,  Kliniek = 30),
                                                    capaciteit   = list(IC = 17,  Kliniek = 41),
                                                    alpha_levels = list(IC = 0.9, Kliniek = 0.9)),
                                      GHZ    = list(bezetting    = list(IC = 8,   Kliniek = 25),
                                                    capaciteit   = list(IC = 13,  Kliniek = 31),
                                                    alpha_levels = list(IC = 0.9, Kliniek = 0.9)))


data_start_zkh[['Netwerk Acute Zorg West als ziekenhuis']] = list(bezetting = list(IC       = sum(sapply(data_start_zkh, function(l){l$bezetting$IC})),
                                                                                   Kliniek  = sum(sapply(data_start_zkh, function(l){l$bezetting$Kliniek}))),
                                                                  capaciteit = list(IC      = sum(sapply(data_start_zkh, function(l){l$capaciteit$IC})),
                                                                                    Kliniek = sum(sapply(data_start_zkh, function(l){l$capaciteit$Kliniek}))),
                                                                  alpha_levels = list(IC    = 0.9, Kliniek = 0.9))

cores <- detectCores(logical = FALSE)
horizon = 3
if(tussentijds_opslaan | store_1sim){
  #onder welke naam sla ik data op? toevg_1 geeft globaal de run aan, toevg_2 geeft een slice aan van 10 simulaties uit die run
  pad      = 'regionale_info/simulaties spreiding regionaal/'
  files    = list.files(pad,pattern = '.rds')
  start    = paste0('simulaties_spreiding_ziekenhuizen_L', toString(data_start_zkh$LUMC$alpha_levels$Kliniek),
                    '_H', toString(data_start_zkh$Haga$alpha_levels$Kliniek), "_G", toString(data_start_zkh$GHZ$alpha_levels$Kliniek),
                    "_hor=",toString(horizon))
  toevg_1 = '_1'
  i=1
  while(any(grepl(paste0(start, toevg_1), files))){
    i=i+1
    toevg_1 = paste0('_',i)
  }
  toevg_2 = "_1"
  res = list("init")
  save(res, file = paste0(pad,start,toevg_1, toevg_2,'.rds'))
}

#verkrijg oude files
path = './regionale_info/resultaten spreiding ziekenhuizen/simulaties original/'
files <- list.files(path = path ,pattern = '.rds')
seed_info_old = list()
aankomsten_old = list()
for(file in files){
  results_old = load_obj(paste0(path, file))[[1]][[1]]
  for(result in results_old){
    seed_info_old = append(seed_info_old, list(result$seed_info))
    aankomsten_old = append(aankomsten_old, list(result$df_aankomsten))
  }
}

#verkrijg data over besmettingen en aankomsten in de regio (voor bepalen theta's )
simuleerdata_regio = load_obj(paste0('regionale_info/', regio,' als ziekenhuis.rds')) # als dit niet bestaat, run dan "verkrijg_info_ziekenhuis.R" met nieuwe data
simulatie_resultaten = list()
schaling_reg_aank      = simuleerdata_regio$schaling_reg_aank

for(m in seq(Mstart, Mtot)){
  seed_info = seed_info_old[[m]]
  timenum = seed_info$seed#as.numeric(Sys.time())
  randomseed = seed_info$randomseed
  
  #sample aankomsten voor elk patiënt-type vooraf, resultaat is een reeks ptypen en als rij-namen de relatieve aankomsttijden tov start_datumtijd
  arrival_rates      = get_arrival_rates(preds_ic = simuleerdata_regio$preds_opnames_ic %>% mutate(ewma_opn = schaling_reg_aank*ewma_opn),
                                         preds_kl = simuleerdata_regio$preds_opnames_kl %>% mutate(ewma_opn = schaling_reg_aank*ewma_opn), 
                                         departure_info = simuleerdata_regio$departure_info)
  df_aankomsten = c()
  for(ptype in names(arrival_rates$ewma_opn)){
    ar_ptype           = arrival_rates$ewma_opn[[ptype]]
    path               = sample_path_nhpois(evaluatie_horizon + 1 + 10/24, ar_ptype$cuml_intensity_NHPP, ar_ptype$icuml_intensity_NHPP, cur_time = 10/24) 
    path[,1]           = rep(ptype, nrow(path))
    df_aankomsten      = rbind(df_aankomsten, path)
  }
  df_aankomsten = aankomsten_old[[m]]#cbind(df_aankomsten, as.Date(start_evaluatie) + ddays(as.numeric(row.names(df_aankomsten))))
  
  for(strategie_verdeelsleutel in strategien_verdeelsleutel){
    set.seed(timenum)
    .Random.seed = randomseed
    
    #bepaal initiele theta
    theta_init      = t(as.matrix(simuleerdata_regio$historische_verdeelsleutel ))
    df_verdeelsleutel    = c()
    
    #maak initiele state van het systeem
    simuleerdata_ziekenhuizen = list()
    for(ziekenhuis in lijst_zkh[[strategie_verdeelsleutel]]){
      simuleerdata_zkh = simuleerdata_regio
      
      #kans overplaatsing bepalen
      kans_kl_ic  = max(simuleerdata_zkh$condprobs_kl_ic_gbhbp$est$ligduur_01)  #aanname dat pat geen bhbp heeft
      kans_ic_kl  = max(simuleerdata_zkh$condprobs_ic_kl$est$ligduur_01)
      
      #info huidige patienten bepalen, data_nieuwe_pat_kliniek is de lijst aan data die je zou verwachten als een patiënt die dag is binnengekomen op de kliniek, 
      #  nodige data om pad te sim
      huidige_bezetting_zkh   = data_start_zkh[[ziekenhuis]]$bezetting
      data_nieuwe_pat_kliniek = list(list(LOS_init = 1, department = 'Kliniek', prob_types = list(pat_KL_IC = kans_kl_ic, pat_KL_rest = 1 - kans_kl_ic)))
      data_nieuwe_pat_ic      = list(list(LOS_init = 1, department = 'IC',      prob_types = list(pat_IC_KL = kans_ic_kl, pat_IC_rest = 1 - kans_ic_kl)))
      
      #init data is aantal nieuwe pat bij kliniek en ic
      huidig_pat_ic                   = data_nieuwe_pat_ic
      huidig_pat_ic[[1]]$arrival_time = start_evaluatie
      huidig_pat_kl                   = data_nieuwe_pat_kliniek
      huidig_pat_kl[[1]]$arrival_time = start_evaluatie
      
      current_patient_info = append(rep(huidig_pat_kl, huidige_bezetting_zkh$Kliniek),
                                    rep(huidig_pat_ic, huidige_bezetting_zkh$IC))
      
      #combineer in totale lijst, vraag: moet simuleerdata_zkh hier nog in zitten?
      simuleerdata_ziekenhuizen[[ziekenhuis]] = append(simuleerdata_zkh, list(current_patient_info  = current_patient_info, 
                                                                              bezetting             = data_start_zkh[[ziekenhuis]]$bezetting,
                                                                              capaciteit            = data_start_zkh[[ziekenhuis]]$capaciteit,
                                                                              data_nieuwe_pat       = list(Kliniek = data_nieuwe_pat_kliniek,
                                                                                                           IC      = data_nieuwe_pat_ic),
                                                                              kans_naar_ic          = simuleerdata_zkh$perc_ic,
                                                                              alpha_levels          = data_start_zkh[[ziekenhuis]]$alpha_levels))
      
    }
    simuleerdata_init = simuleerdata_ziekenhuizen
  
    #initialiseer dataframes om bezetting en capaciteit op te slaan
    df_bezetting   = as_tibble(t(unlist(data_start_zkh[lijst_zkh[[strategie_verdeelsleutel]]]))) %>% select(contains('bezetting')) 
    df_capaciteit  = as_tibble(t(unlist(data_start_zkh))) %>% select(contains('.capaciteit.') & contains(names(simuleerdata_init)))
    
    huidig_tijdstip = start_evaluatie
    dates           = c()
    
    #simuleer
    list2env(doe_simulatie(huidig_tijdstip, end_evaluatie, strategie_verdeelsleutel, m, simuleerdata_ziekenhuizen, simuleerdata_regio, theta_init,data_start_zkh,
                           schaling_reg_aank, timenum, df_aankomsten, ziekenhuizen_regio, lijst_zkh, df_capaciteit, df_bezetting, dates, df_verdeelsleutel, simuleerdata_init, 
                           randomseed, simulatie_resultaten, toevg_1, horizon = horizon, store = store_1sim), envir = environment())
    
    #sla verloop van theta over tijd op
    df_verdeelsleutel = as_tibble(df_verdeelsleutel) %>% add_column(datum = c(start_evaluatie, as.POSIXct(head(dates,-1), tz = 'UTC')), .before = 1)
    colnames(df_verdeelsleutel) = c('datum', sapply(colnames(df_verdeelsleutel)[-1], function(name){paste0('verdeelsleutel.',name)}))
    
    dates_incl_start = c(toString(start_evaluatie), dates)
    df_bezetting  = as_tibble(df_bezetting) %>% add_column(datum = as.POSIXct(dates_incl_start, tz = 'UTC'), .before = 1)
    df_capaciteit = as_tibble(df_capaciteit) %>% add_column(datum = as.POSIXct(dates_incl_start, tz = 'UTC'), .before = 1)
    
    seed_info                                        = list(seed =timenum, randomseed = randomseed)
    simulatie_resultaten[[strategie_verdeelsleutel]] = append(simulatie_resultaten[[strategie_verdeelsleutel]], list(mget(c( 'df_bezetting', 'df_aankomsten', 'seed_info',
                                                                                                                             'data_start_zkh', 'simuleerdata_init', 'df_verdeelsleutel',
                                                                                                                             'df_capaciteit'))))
  }   
  if(tussentijds_opslaan){
    res = mget(c('simulatie_resultaten' ))
    save(res, file = paste0(pad,start,toevg_1, toevg_2,'.rds'))
    if(m > 2 & mod(m,10)==0){#maak nieuwe file aan
      toevg_2 = '_1'
      i=1
      while(file.exists(paste0(pad,start, toevg_1, toevg_2,'.rds'))){
        i = i+1
        toevg_2 = paste0('_',i)
      }
      simulatie_resultaten = list()
    }
  }
}


