source('R/utils.R')


beleid_nav_plek = function(huidig_tijdstip, aankomsten, simuleerdata_ziekenhuizen, simuleerdata_regio, bedden_vrijgehouden = 2){
  set.seed(as.numeric(Sys.time()))
  #verplaats patiënten naar zkh waar plek is.
  #bepaal regios met  kliniek bedden vrij
  capaciteit_kl_zkh        = sapply(simuleerdata_ziekenhuizen, function(List){List$capaciteit$Kliniek })
  bezetting_kl_zkh         = sapply(simuleerdata_ziekenhuizen, function(List){List$bezetting$Kliniek })
  vrije_plekken_kl_zkh     = sapply(capaciteit_kl_zkh - bezetting_kl_zkh, function(x){max(x,0)})  # max(0, capa - bez)
  
  #lijst om bestemming uit te samplen, regio staat er in als er plek is
  lijst_vrije_plekken_kl = Reduce(append, sapply(names(vrije_plekken_kl_zkh), function(name){rep(name, ifelse(vrije_plekken_kl_zkh[name]>0,1,0))})) 
  nodige_opnames_kl      = aankomsten$aantal_kl
  
  #maak lege vector aan met bestemmingen voor aangekomen pat
  namen_ziekenhuizen          = c(names(simuleerdata_ziekenhuizen), 'Buiten regio') 
  vector_opnames_kl           = zeros(1,length(namen_ziekenhuizen))
  colnames(vector_opnames_kl) = namen_ziekenhuizen
  
  copy_nodige_opnames_kl = nodige_opnames_kl
  while(copy_nodige_opnames_kl > 0){
    if(length(lijst_vrije_plekken_kl) == 0){
      # als er geen vrije plekken meer zijn, stuur alles naar buiten
      vector_opnames_kl[,'Buiten regio'] = copy_nodige_opnames_kl
      copy_nodige_opnames_kl     = 0
    }else{
      #sample random bestemming, verwerk in vector met bestemmingen
      zkh_bestemming                     = sample(lijst_vrije_plekken_kl,1)
      vector_opnames_kl[,zkh_bestemming] = vector_opnames_kl[,zkh_bestemming] + 1
      
      #verwerk in sample lijsten
      vrije_plekken_kl_zkh[zkh_bestemming]     = vrije_plekken_kl_zkh[zkh_bestemming] - 1
      if(vrije_plekken_kl_zkh[zkh_bestemming] <= bedden_vrijgehouden){
        lijst_vrije_plekken_kl = lijst_vrije_plekken_kl[-which(lijst_vrije_plekken_kl == zkh_bestemming)]
      }
      copy_nodige_opnames_kl                   = copy_nodige_opnames_kl - 1
    }
  }
  
  ## zelfde weer voor ic
  #bepaal regios met  kliniek bedden vrij
  capaciteit_ic_zkh        = sapply(simuleerdata_ziekenhuizen, function(List){List$capaciteit$IC })
  bezetting_ic_zkh         = sapply(simuleerdata_ziekenhuizen, function(List){List$bezetting$IC })
  vrije_plekken_ic_zkh     = sapply(capaciteit_ic_zkh - bezetting_ic_zkh, function(x){max(x,0)})  # max(0, capa - bez)
  
  #lijst om bestemming uit te samplen, regio staat er in als er plek is
  lijst_vrije_plekken_ic = Reduce(append, sapply(names(vrije_plekken_ic_zkh), function(name){rep(name, ifelse(vrije_plekken_ic_zkh[name]>0,1,0))})) 
  nodige_opnames_ic      = aankomsten$aantal_ic
  
  #maak lege vector aan met bestemmingen voor aangekomen pat
  namen_ziekenhuizen          = c(names(simuleerdata_ziekenhuizen), 'Buiten regio') 
  vector_opnames_ic           = zeros(1,length(namen_ziekenhuizen))
  colnames(vector_opnames_ic) = namen_ziekenhuizen
  
  copy_nodige_opnames_ic = nodige_opnames_ic
  while(copy_nodige_opnames_ic > 0){
    if(length(lijst_vrije_plekken_ic) == 0){
      # als er geen vrije plekken meer zijn, stuur alles naar buiten
      vector_opnames_ic[,'Buiten regio'] = copy_nodige_opnames_ic
      copy_nodige_opnames_ic     = 0
    }else{
      #sample random bestemming, verwerk in vector met bestemmingen
      zkh_bestemming                     = sample(lijst_vrije_plekken_ic,1)
      vector_opnames_ic[,zkh_bestemming] = vector_opnames_ic[,zkh_bestemming] + 1
      
      #verwerk in sample lijsten
      vrije_plekken_ic_zkh[zkh_bestemming]     = vrije_plekken_ic_zkh[zkh_bestemming] - 1
      if(vrije_plekken_ic_zkh[zkh_bestemming] <= bedden_vrijgehouden){
        lijst_vrije_plekken_ic = lijst_vrije_plekken_ic[-which(lijst_vrije_plekken_ic == zkh_bestemming)]
      }
      copy_nodige_opnames_ic                   = copy_nodige_opnames_ic - 1
    }
  }
  return(list(vector_opnames_kl = vector_opnames_kl, vector_opnames_ic = vector_opnames_ic, theta_init = theta_init))
}

beleid_nav_beddenaantal = function(huidig_tijdstip, aankomsten, simuleerdata_ziekenhuizen, simuleerdata_regio, theta_init, bedden_vrijgehouden = 2){
  set.seed(as.numeric(Sys.time()))
  #verplaats patiënten naar zkh nav beddenaantal
  #bepaal regios met  kliniek bedden vrij
  capaciteit_kl_zkh        = sapply(simuleerdata_ziekenhuizen, function(List){List$capaciteit$Kliniek })
  bezetting_kl_zkh         = sapply(simuleerdata_ziekenhuizen, function(List){List$bezetting$Kliniek })
  vrije_plekken_kl_zkh     = sapply(capaciteit_kl_zkh - bezetting_kl_zkh, function(x){max(x,0)})  # max(0, capa - bez)
  
  #lijst om bestemming uit te samplen, regio staat er in als er plek is
  lijst_vrije_plekken_kl = Reduce(append, sapply(names(vrije_plekken_kl_zkh), function(name){rep(name, vrije_plekken_kl_zkh[name])})) 
  nodige_opnames_kl      = aankomsten$aantal_kl
  
  #maak lege vector aan met bestemmingen voor aangekomen pat
  namen_ziekenhuizen          = c(names(simuleerdata_ziekenhuizen), 'Buiten regio') 
  vector_opnames_kl           = zeros(1,length(namen_ziekenhuizen))
  colnames(vector_opnames_kl) = namen_ziekenhuizen
  
  copy_nodige_opnames_kl = nodige_opnames_kl
  while(copy_nodige_opnames_kl > 0){
    if(length(lijst_vrije_plekken_kl) == 0){
      # als er geen vrije plekken meer zijn, stuur alles naar buiten
      vector_opnames_kl[,'Buiten regio'] = copy_nodige_opnames_kl
      copy_nodige_opnames_kl     = 0
    }else{
      #sample random bestemming, verwerk in vector met bestemmingen
      zkh_bestemming                     = sample(lijst_vrije_plekken_kl,1)
      vector_opnames_kl[,zkh_bestemming] = vector_opnames_kl[,zkh_bestemming] + 1
      
      #verwerk in sample lijsten
      vrije_plekken_kl_zkh[zkh_bestemming]     = vrije_plekken_kl_zkh[zkh_bestemming] - 1
      if(vrije_plekken_kl_zkh[zkh_bestemming] <= bedden_vrijgehouden){
        lijst_vrije_plekken_kl = lijst_vrije_plekken_kl[-first(which(lijst_vrije_plekken_kl == zkh_bestemming))]
      }
      copy_nodige_opnames_kl                   = copy_nodige_opnames_kl - 1
    }
  }
  
  ## zelfde weer voor ic
  ## zelfde weer voor ic
  #bepaal regios met  kliniek bedden vrij
  capaciteit_ic_zkh        = sapply(simuleerdata_ziekenhuizen, function(List){List$capaciteit$IC })
  bezetting_ic_zkh         = sapply(simuleerdata_ziekenhuizen, function(List){List$bezetting$IC })
  vrije_plekken_ic_zkh     = sapply(capaciteit_ic_zkh - bezetting_ic_zkh, function(x){max(x,0)})  # max(0, capa - bez)
  
  #lijst om bestemming uit te samplen, regio staat er in als er plek is
  lijst_vrije_plekken_ic = Reduce(append, sapply(names(vrije_plekken_ic_zkh), function(name){rep(name, vrije_plekken_ic_zkh[name])})) 
  nodige_opnames_ic      = aankomsten$aantal_ic
  
  #maak lege vector aan met bestemmingen voor aangekomen pat
  namen_ziekenhuizen          = c(names(simuleerdata_ziekenhuizen), 'Buiten regio') 
  vector_opnames_ic           = zeros(1,length(namen_ziekenhuizen))
  colnames(vector_opnames_ic) = namen_ziekenhuizen
  
  copy_nodige_opnames_ic = nodige_opnames_ic
  while(copy_nodige_opnames_ic > 0){
    if(length(lijst_vrije_plekken_ic) == 0){
      # als er geen vrije plekken meer zijn, stuur alles naar buiten
      vector_opnames_ic[,'Buiten regio'] = copy_nodige_opnames_ic
      copy_nodige_opnames_ic     = 0
    }else{
      #sample random bestemming, verwerk in vector met bestemmingen
      zkh_bestemming                     = sample(lijst_vrije_plekken_ic,1)
      vector_opnames_ic[,zkh_bestemming] = vector_opnames_ic[,zkh_bestemming] + 1
      
      #verwerk in sample lijsten
      vrije_plekken_ic_zkh[zkh_bestemming]     = vrije_plekken_ic_zkh[zkh_bestemming] - 1
      if(vrije_plekken_ic_zkh[zkh_bestemming] <= bedden_vrijgehouden){
        lijst_vrije_plekken_ic = lijst_vrije_plekken_ic[-which(lijst_vrije_plekken_ic == zkh_bestemming)]
      }
      copy_nodige_opnames_ic                   = copy_nodige_opnames_ic - 1
    }
  }
  return(list(vector_opnames_kl = vector_opnames_kl, vector_opnames_ic = vector_opnames_ic, theta_init = theta_init))
}


beleid_CHOIR = function(huidig_tijdstip, aankomsten, simuleerdata_ziekenhuizen, simuleerdata_regio, theta_init, horizon  = 3, M = 1000){
  set.seed(as.numeric(Sys.time()))
  # verkrijg besmettingsdata tot aan voorspelmoment en voorspel aankomsten
  data_besmettingen             = simuleerdata_regio$data_besmettingen %>% filter(datum <= as.Date(huidig_tijdstip))
  voorspelde_regionale_instroom = quiet(fit_besmettingen_opnames(data_besmettingen , horizon = horizon +1 ) %>% filter(datum > max(data_besmettingen$datum))) 
  
  thetas_zkh           = zeros(1, length(simuleerdata_ziekenhuizen))
  colnames(thetas_zkh) = names(simuleerdata_ziekenhuizen)
  
  if(all(voorspelde_regionale_instroom$besmettingen == 0)){
    #als er geen instroom voorspelt wordt kan het altijd
    thetas_zkh = thetas_zkh + 1
  }else{
  
  #vind theta voor alle zkh
  source('R/code regionaal/bereken_theta.R')
  for(ziekenhuis in names(simuleerdata_ziekenhuizen)){
    print(ziekenhuis)
    #verkrijg data regio
    simuleerdata = simuleerdata_ziekenhuizen[[ziekenhuis]]
    
    #kijk of er uberhaupt plekken vrij zijn in zkh
    capaciteit_zkh = unlist(simuleerdata$capaciteit)
    bezetting_zkh = unlist(simuleerdata$bezetting)
    bedden_vrij_zkh = all(capaciteit_zkh >= bezetting_zkh)
    
    if(bedden_vrij_zkh){
      #split in aankomsten IC en aankomsten kliniek, schaal op met zelfde constante als de echte aankomstintensiteit
      thetas_zkh[,ziekenhuis] = bereken_theta(simuleerdata, voorspelde_regionale_instroom, horizon, huidig_tijdstip, theta_init = theta_init[,ziekenhuis], print_ = T)
    }else{
      thetas_zkh[,ziekenhuis] = 0
    }
  }}
  
  #normaliseer kansen
  kansen_opnames = thetas_zkh/sum(thetas_zkh)
  
  #bepaal regios met  kliniek bedden vrij
  capaciteit_kl_zkh        = sapply(simuleerdata_ziekenhuizen, function(List){List$capaciteit$Kliniek })
  bezetting_kl_zkh         = sapply(simuleerdata_ziekenhuizen, function(List){List$bezetting$Kliniek })
  vrije_plekken_kl_zkh     = sapply(capaciteit_kl_zkh - bezetting_kl_zkh, function(x){max(x,0)})  # max(0, capa - bez)
  
  #lijst om bestemming uit te samplen, regio staat er in als er plek is
  lijst_vrije_plekken_kl = names(simuleerdata_ziekenhuizen)
  nodige_opnames_kl      = aankomsten$aantal_kl
  
  #maak lege vector aan met bestemmingen voor aangekomen pat
  namen_ziekenhuizen          = c(names(simuleerdata_ziekenhuizen), 'Buiten regio') 
  vector_opnames_kl           = zeros(1,length(namen_ziekenhuizen))
  colnames(vector_opnames_kl) = namen_ziekenhuizen
  
  copy_nodige_opnames_kl = nodige_opnames_kl
  while(copy_nodige_opnames_kl > 0){
    if(length(lijst_vrije_plekken_kl) == 0){
      # als er geen vrije plekken meer zijn, stuur alles naar buiten
      vector_opnames_kl[,'Buiten regio'] = copy_nodige_opnames_kl
      copy_nodige_opnames_kl     = 0
    }else{
      #sample random bestemming met kans op basis thetas, verwerk in vector met bestemmingen
      zkh_bestemming                     = sample(lijst_vrije_plekken_kl,1, prob = kansen_opnames)
      vector_opnames_kl[,zkh_bestemming] = vector_opnames_kl[,zkh_bestemming] + 1
      
      #verwerk in sample lijsten
      vrije_plekken_kl_zkh[zkh_bestemming]     = vrije_plekken_kl_zkh[zkh_bestemming] - 1
      if(vrije_plekken_kl_zkh[zkh_bestemming] <= 0 ){
        lijst_vrije_plekken_kl = lijst_vrije_plekken_kl[-which(lijst_vrije_plekken_kl == zkh_bestemming)]
        kansen_opnames[, zkh_bestemming] = 0
        kansen_opnames  = kansen_opnames/sum(kansen_opnames)
      }
      copy_nodige_opnames_kl                   = copy_nodige_opnames_kl - 1
    }
  }
  
  ## zelfde weer voor ic
  #bepaal regios met  kliniek bedden vrij
  capaciteit_ic_zkh        = sapply(simuleerdata_ziekenhuizen, function(List){List$capaciteit$IC })
  bezetting_ic_zkh         = sapply(simuleerdata_ziekenhuizen, function(List){List$bezetting$IC })
  vrije_plekken_ic_zkh     = sapply(capaciteit_ic_zkh - bezetting_ic_zkh, function(x){max(x,0)})  # max(0, capa - bez)
  
  #lijst om bestemming uit te samplen, regio staat er in als er plek is
  lijst_vrije_plekken_ic = names(simuleerdata_ziekenhuizen)
  nodige_opnames_ic      = aankomsten$aantal_ic
  
  #maak lege vector aan met bestemmingen voor aangekomen pat
  namen_ziekenhuizen          = c(names(simuleerdata_ziekenhuizen), 'Buiten regio') 
  vector_opnames_ic           = zeros(1,length(namen_ziekenhuizen))
  colnames(vector_opnames_ic) = namen_ziekenhuizen
  
  copy_nodige_opnames_ic = nodige_opnames_ic
  while(copy_nodige_opnames_ic > 0){
    if(length(lijst_vrije_plekken_ic) == 0){
      # als er geen vrije plekken meer zijn, stuur alles naar buiten
      vector_opnames_ic[,'Buiten regio'] = copy_nodige_opnames_ic
      copy_nodige_opnames_ic     = 0
    }else{
      #sample random bestemming, verwerk in vector met bestemmingen
      zkh_bestemming                     = sample(lijst_vrije_plekken_ic,1, prob = kansen_opnames)
      vector_opnames_ic[,zkh_bestemming] = vector_opnames_ic[,zkh_bestemming] + 1
      
      #verwerk in sample lijsten
      vrije_plekken_ic_zkh[zkh_bestemming]     = vrije_plekken_ic_zkh[zkh_bestemming] - 1
      if(vrije_plekken_ic_zkh[zkh_bestemming] <= 0){
        lijst_vrije_plekken_ic = lijst_vrije_plekken_ic[-which(lijst_vrije_plekken_ic == zkh_bestemming)]
        kansen_opnames[, zkh_bestemming] = 0
        kansen_opnames = kansen_opnames/sum(kansen_opnames) #normaliseer kansen opnieuw
      }
      copy_nodige_opnames_ic                   = copy_nodige_opnames_ic - 1
    }
  }
  return(list(vector_opnames_kl = vector_opnames_kl, vector_opnames_ic = vector_opnames_ic, theta_init = thetas_zkh))
  
  
  
  
}

