rm(list=ls())
source('R/utils.R')
require(reshape2)
require(tibbletime)
library(xtable)
library(scales)
library(tables)

genereer_namen_verplaatsingen_tussen_cl = function(vec_regios_cluster){
  vec_namen_verplaatsingen_tussen_cl = c()
  for(naam_regio_1 in vec_regios_cluster){
    for(naam_regio_2 in vec_regios_cluster[vec_regios_cluster != naam_regio_1]){
      vec_namen_verplaatsingen_tussen_cl = c(vec_namen_verplaatsingen_tussen_cl,
                                             paste0('van ', naam_regio_1, ' naar ', naam_regio_2))
    }
  }
  return(vec_namen_verplaatsingen_tussen_cl)
}

#params
regios = c('Acute Zorgregio Oost', 'Netwerk Acute Zorg Brabant',
           'Netwerk Acute Zorg West', "Traumazorgnetwerk Midden-Nederland")

#definieer  transportkosten
kosten_transport  = matrix(c(10, 50, 50,10, 100, 
                          50, 10, 10,50, 100, 
                          50, 10, 10,50, 100, 
                          10, 50, 50,10, 100), ncol = 1)


paths = list("Integer Program" = './regionale_info/resultaten spreiding regionaal/simulaties/resultaten artikel//',
             "Stochastic Program" = './regionale_info/resultaten spreiding regionaal/simulaties/resultaten stoch prog/')
filename_store = 'regionale_info/resultaten spreiding regionaal/tabel_spreiding_regios_vergelijking_orig_stoch.tex'
# paths = list("horizon $3$" = './regionale_info/resultaten spreiding regionaal/simulaties/resultaten artikel//',
#              "horizon $5$" = './regionale_info/resultaten spreiding regionaal/simulaties/resultaten revisie hor = 5//')
# filename_store = 'regionale_info/resultaten spreiding regionaal/tabel_spreiding_regios_vergelijking_horizons.tex'
include_totcost = T
namen_engels = c( 'CHOIR' = 'Optimisation')

save_results = T
upper_bound_sims = 250
level = 0.9
#params
clusters = list(c('Acute Zorgregio Oost', "Traumazorgnetwerk Midden-Nederland"),
                c('Netwerk Acute Zorg West', 'Netwerk Acute Zorg Brabant'))
geen_clusters = list(c('Acute Zorgregio Oost',  'Netwerk Acute Zorg West'),
                     c('Acute Zorgregio Oost',  'Netwerk Acute Zorg Brabant'),
                     c("Traumazorgnetwerk Midden-Nederland", 'Netwerk Acute Zorg West'),
                     c("Traumazorgnetwerk Midden-Nederland", 'Netwerk Acute Zorg Brabant'))

kolomnamen_verpl_binnen_cl = as.vector(sapply(clusters, function(v){genereer_namen_verplaatsingen_tussen_cl(v)}))
kolomnamen_verpl_tussen_cl = as.vector(sapply(geen_clusters, function(v){genereer_namen_verplaatsingen_tussen_cl(v)}))

namen_strategien = c('CHOIR')
nsim = c()
df_kpis_long_methods = list()
for(method in names(paths)){
  kpis_tot = list()
  files <- list.files(path = paths[[method]] ,pattern = '.rds')
  for(filename in files){
    loadres = load_obj(paste0(paths[[method]], filename))
    for(naam_strategie in namen_strategien){
      kpis_strat = c()
      res  =loadres$lijst_info[[naam_strategie]]
      for(lijst_info_strat in res ){
        verplaatsingen_ic_binnen_clusters =  sum(lijst_info_strat$overplaatsingen_ic %>% select(kolomnamen_verpl_binnen_cl))
        verplaatsingen_kl_binnen_clusters =  sum(lijst_info_strat$overplaatsingen_kl %>% select(kolomnamen_verpl_binnen_cl))
        
        verplaatsingen_ic_tussen_clusters =  sum(lijst_info_strat$overplaatsingen_ic %>% select(kolomnamen_verpl_tussen_cl))
        verplaatsingen_kl_tussen_clusters =  sum(lijst_info_strat$overplaatsingen_kl %>% select(kolomnamen_verpl_tussen_cl))
        
        verplaatsingen_ic_outer =  sum(lijst_info_strat$overplaatsingen_ic %>% select(contains('Buitenland')))
        verplaatsingen_kl_outer = sum(lijst_info_strat$overplaatsingen_kl %>% select(contains('Buitenland')))
        
        verplaatsingen_ic_tot =  sum(lijst_info_strat$overplaatsingen_ic %>% select(-datum))
        verplaatsingen_kl_tot = sum(lijst_info_strat$overplaatsingen_kl %>% select(-datum))
        
        kosten_verpl_ic = sum(as.matrix(as.data.frame(lijst_info_strat$overplaatsingen_ic)[,-1])%*%kosten_transport)/verplaatsingen_ic_tot
        kosten_verpl_kl = sum(as.matrix(as.data.frame(lijst_info_strat$overplaatsingen_kl)[,-1])%*%kosten_transport)/verplaatsingen_kl_tot
        
        if(include_totcost){
          kpis_strat = rbind(kpis_strat, c(verplaatsingen_ic_tot, verplaatsingen_ic_outer, verplaatsingen_ic_binnen_clusters,verplaatsingen_ic_tussen_clusters,kosten_verpl_ic,
                                           verplaatsingen_kl_tot, verplaatsingen_kl_outer, verplaatsingen_kl_binnen_clusters,verplaatsingen_kl_tussen_clusters,kosten_verpl_kl
          ))
        }else{
          kpis_strat = rbind(kpis_strat, c(verplaatsingen_ic_tot, verplaatsingen_ic_outer, verplaatsingen_ic_binnen_clusters,verplaatsingen_ic_tussen_clusters,
                                           verplaatsingen_kl_tot, verplaatsingen_kl_outer, verplaatsingen_kl_binnen_clusters,verplaatsingen_kl_tussen_clusters
          ))
        }
     
      }
      
      kpis_tot[[naam_strategie]] = rbind(kpis_tot[[naam_strategie]], kpis_strat)
      if(nrow(kpis_tot[[naam_strategie]]) > upper_bound_sims){
        kpis_tot[[naam_strategie]] = head(kpis_tot[[naam_strategie]], upper_bound_sims)
      }
    }  
  }
  nsim = c(nsim, dim(kpis_tot[[naam_strategie]])[1])
  df_means = c()
  df_radii = c()
  for(naam_strategie in namen_strategien){
    df_means = rbind(df_means, apply(kpis_tot[[naam_strategie]], 2, function(v){mean(as.numeric(v), na.rm = T)} ))
    df_radii = rbind(df_radii, apply(kpis_tot[[naam_strategie]], 2, function(v){qt(0.975, length(v)-1)*sd(as.numeric(v), na.rm = T)/sqrt(length(v[!is.na(v)]))}))
  }
  df_means_long = as_tibble(df_means) %>% add_column(policy = namen_engels[namen_strategien], .before = 1) %>% pivot_longer(-policy) %>% rename(mean = value)
  df_radii_long = as_tibble(df_radii) %>% add_column(policy = namen_engels[namen_strategien], .before = 1) %>% pivot_longer(-policy) %>% rename(radius = value)
  
  df_kpis_long                                 = df_means_long %>% left_join(df_radii_long, c('policy', 'name'))
  means_formatted                              = gsub(' ', '\\\\phantom{0}', format(round(df_kpis_long$mean,2), nsmall = 2))
  radii_formatted                              = gsub(' ', '\\\\phantom{0}', format(round(df_kpis_long$radius,2), nsmall = 2))
  df_kpis_long$value                           = sapply(1:length(means_formatted), function(i){paste0('$\\text{',means_formatted[i],' $\\pm$ ', radii_formatted[i], '}$')})
  df_kpis_long$value[is.na(df_kpis_long$mean)] = '-'
  
  if(include_totcost){
    df_kpis_long                                 = df_kpis_long %>% select(-name) %>% add_column(dept = rep(rep(c('ICU', 'Ward'), each = 5), length(namen_strategien))) %>%
      add_column(kpi = rep(c('Total',  'External', 'In Clusters', 'Btw. Clusters', 'Average Cost'),2*length(namen_strategien)))
  }else{
    df_kpis_long                                 = df_kpis_long %>% select(-name) %>% add_column(dept = rep(rep(c('ICU', 'Ward'), each = 4), length(namen_strategien))) %>%
      add_column(kpi = rep(c('Total',  'External', 'In Clusters', 'Btw. Clusters'),2*length(namen_strategien)))
  }
 
  
  df_kpis_long$policy = method
  if(strcmp(method, names(paths)[1])){
    df_kpis_long_tot = df_kpis_long
    }else{
      df_kpis_long_tot = df_kpis_long_tot %>% full_join(df_kpis_long )
    }
  }

print(nsim)
results_table <- tabular(RowFactor(kpi, name = 'KPI')*RowFactor(dept, name = 'Dept.') ~
                           Heading()*Factor(policy)*Heading()*(value)*Heading()*(toString)*
                           DropEmpty(empty=""),
                         data = df_kpis_long_tot )
if(include_totcost){
  results_table = results_table[c(8,7,4,3,6,5,2,1,10,9),]
}else{
  results_table = results_table[c(8,7,4,3,6,5,2,1),]
}

results_table
if(save_results){
  write(tables::latex.tabular(results_table)[[1]], file = filename_store)
}





