rm(list=ls())
function_overbedden = function(v){mean(sapply(v,function(x){max(x,0)}))}

source('R/utils.R')
require(reshape2)
require(tibbletime)
library(xtable)
library(scales)
library(tables)
library(cowplot)
#sample file randomly to investigate
path = './regionale_info/resultaten spreiding ziekenhuizen/simulaties revision lumc 0.99/'
files <- list.files(path = path ,pattern = '.rds')
file = sample(files,1)
loadres = load_obj(paste0(path, file))
naam_strategie = 'optimale verdeelsleutel' #'individuele ziekenhuizen'
simres = sample(loadres$simulatie_resultaten[[naam_strategie]], 1)[[1]]

#bepaal verdeelsleutel
historische_verdeelsleutel = as.numeric(loadres$simulatie_resultaten[['individuele ziekenhuizen']][[1]]$df_verdeelsleutel[1,-1])[c(2,3,1)]
df_verdeelsleutel = simres$df_verdeelsleutel
df_verdeelsleutel[,-1] = simres$df_verdeelsleutel[,-1]/(as.matrix(apply(simres$df_verdeelsleutel[,-1], 1, sum))%*%ones(1,3))
indc_all_0 = apply(df_verdeelsleutel[,-1], 1, function(v){any(is.na(v))})
df_verdeelsleutel[indc_all_0,-1] = ones(sum(indc_all_0),1)%*%historische_verdeelsleutel
df_verdeelsleutel[df_verdeelsleutel==Inf] = 1/3

#bepaal historische capaciteit & bezetting
df_capaciteit =simres$df_capaciteit
df_bezetting = simres$df_bezetting


df_tot =  df_bezetting %>% left_join(df_verdeelsleutel, by ='datum') %>% left_join(df_capaciteit, by ='datum')

panes = c('Ward' = 'Kliniek', 'ICU'='IC', 'Allocation'='verdeelsleutel')
df_tot_long = pivot_longer(df_tot, c(-datum))
df_tot_long = df_tot_long %>% add_column(pane = factor(sapply(df_tot_long$name, function(name1){names(panes)[sapply(panes, function(name2){grepl(name2, name1)})]}),
                                                       levels = c('Ward', 'ICU', 'Allocation')))


colors = c( '#2b83ba','#3ded97', '#fdae80')
linetypes = c('solid', 'solid', 'solid')
linewidths = c(1.1*5/24, 1.1*5/12,1.1)
unq_names = unique(df_tot_long$name)
colorspec = c()
linespec  = c()
widthspec = c()
names_colorspec = c()
simuleerdata_init = simres$simuleerdata_init
for(i in 1:length(names(simuleerdata_init))){
  zkh = names(simuleerdata_init)[i]
  color = colors[i]
  linetype  = linetypes[i]
  linewidth = linewidths[i]
  names_zkh = unq_names[sapply(unq_names, function(name){grepl(zkh,name)})]
  names_colorspec = c(names_colorspec, names_zkh)
  colorspec = c(colorspec, rep(color, length(names_zkh)))
  linespec  = c(linespec , rep(linetype, length(names_zkh)))
  widthspec = c(widthspec, rep(linewidth, length(names_zkh)))
}
names(colorspec) = names_colorspec
names(linespec)  = names_colorspec
names(widthspec) = names_colorspec
colorspec = rev(colorspec)
labels = c('Ward' = 'Occupancy Wards', 'ICU' = 'Occupancy ICU', 'Allocation' = "Allocation probability") 



df_plot_bez_kl = df_bezetting %>% select('datum' | contains('Kliniek'))  %>%
  pivot_longer(-datum)
df_plot_cap_kl = df_capaciteit %>% select('datum' | contains('Kliniek'))  %>%
  pivot_longer(-datum)

plot_kl = ggplot() +  geom_line(data = df_plot_cap_kl, aes(x = datum, y = value, color = name, size = name), linetype = 'dashed')+ 
  geom_step(data = df_plot_bez_kl, aes(x = datum, y = value, color = name, linetype = name, size = name))+
  theme_bw()+ scale_colour_manual(values = colorspec)+
  scale_size_manual(values = widthspec)+
  scale_linetype_manual(values = linespec) +
  theme(legend.position = 'none')+xlab('Date') + ylab('Occupancy Ward') +
  scale_y_continuous(breaks = integer_breaks())+ylim(c(0, 60))


df_plot_bez_ic = df_bezetting %>% select('datum' | contains('IC')) %>% 
  pivot_longer(-datum)
df_plot_cap_ic =  df_capaciteit %>% select('datum' | contains('IC')) %>% 
  pivot_longer(-datum)
plot_ic = ggplot() +geom_line(data = df_plot_cap_ic, aes(x = datum, y = value, color = name, size = name), linetype = 'dashed')+
  geom_step(data = df_plot_bez_ic, aes(x = datum, y = value, color = name, linetype = name, size = name))+ theme_bw()+
  scale_colour_manual(values = colorspec)+
  scale_linetype_manual(values = linespec) +
  scale_size_manual(values = widthspec)+
  theme(legend.position = 'none')+xlab('Date')+ylab('Occupancy ICU')+
  scale_y_continuous(breaks = integer_breaks())+ylim(c(0, 27))

df_plot_verdeel = df_verdeelsleutel  %>% pivot_longer(-datum)
plot_verdeel = ggplot(df_plot_verdeel, aes(x=datum)) + geom_step(aes(y = value, color = name, linetype = name, size = name))+ theme_bw()+
  scale_colour_manual(values = colorspec)+
  scale_linetype_manual(values = linespec) +
  scale_size_manual(values = widthspec)+
  theme(legend.position = 'none')+xlab('Date')+ylab('Allocation Probability')+
  scale_y_continuous(labels = scales::percent, limits = c(0,1))


#verkrijg oude files
path = './regionale_info/resultaten spreiding ziekenhuizen/simulaties original/'
files <- list.files(path = path ,pattern = '.rds')
app = 1
i_sim = 1
loadres_2 = load_obj(paste0(path, files[app]))
simres_2 = loadres_2$simulatie_resultaten[[naam_strategie]][[i_sim]]
while(!(simres_2$seed_info$seed == simres$seed_info$seed)){
  # print(app)
  # print(i_sim)
  i_sim = i_sim + 1
  if(i_sim > length(loadres_2$simulatie_resultaten[[naam_strategie]])){
    i_sim = 1
    app = app + 1
    loadres_2 = load_obj(paste0(path, files[app]))
    simres_2 = loadres_2$simulatie_resultaten[[naam_strategie]][[i_sim]]
  }else{
    simres_2 = loadres_2$simulatie_resultaten[[naam_strategie]][[i_sim]]
  }
}
assertthat::are_equal(simres_2$df_aankomsten, simres$df_aankomsten)
assertthat::are_equal(simres_2$seed_info$randomseed, simres$seed_info$randomseed)


#bepaal verdeelsleutel
df_verdeelsleutel = simres_2$df_verdeelsleutel
df_verdeelsleutel[,-1] = simres_2$df_verdeelsleutel[,-1]/(as.matrix(apply(simres_2$df_verdeelsleutel[,-1], 1, sum))%*%ones(1,3))
indc_all_0 = apply(df_verdeelsleutel[,-1], 1, function(v){any(is.na(v))})
df_verdeelsleutel[indc_all_0,-1] = ones(sum(indc_all_0),1)%*%historische_verdeelsleutel
df_verdeelsleutel[df_verdeelsleutel==Inf] = 1/3

#bepaal historische capaciteit & bezetting
df_capaciteit =simres_2$df_capaciteit
df_bezetting = simres_2$df_bezetting

df_plot_bez_kl = df_bezetting %>% select('datum' | contains('Kliniek'))  %>%
  pivot_longer(-datum)
df_plot_cap_kl = df_capaciteit %>% select('datum' | contains('Kliniek'))  %>%
  pivot_longer(-datum)

plot_kl_vast = ggplot() +  geom_line(data = df_plot_cap_kl, aes(x = datum, y = value, color = name, size = name), linetype = 'dashed')+ 
  geom_step(data = df_plot_bez_kl, aes(x = datum, y = value, color = name, linetype = name, size = name))+
  theme_bw()+ scale_colour_manual(values = colorspec)+
  scale_size_manual(values = widthspec)+
  scale_linetype_manual(values = linespec) +
  theme(legend.position = 'none')+xlab('Date') + ylab('Occupancy Ward') +
  scale_y_continuous(breaks = integer_breaks())+ylim(c(0, 60))


df_plot_bez_ic = df_bezetting %>% select('datum' | contains('IC')) %>% 
  pivot_longer(-datum)
df_plot_cap_ic =  df_capaciteit %>% select('datum' | contains('IC')) %>% 
  pivot_longer(-datum)
plot_ic_vast = ggplot() +geom_line(data = df_plot_cap_ic, aes(x = datum, y = value, color = name, size = name), linetype = 'dashed')+
  geom_step(data = df_plot_bez_ic, aes(x = datum, y = value, color = name, linetype = name, size = name))+ theme_bw()+
  scale_colour_manual(values = colorspec)+
  scale_linetype_manual(values = linespec) +
  scale_size_manual(values = widthspec)+
  theme(legend.position = 'none')+xlab('Date')+ylab('Occupancy ICU')+
  scale_y_continuous(breaks = integer_breaks())+ylim(c(0, 27))

df_plot_verdeel = df_verdeelsleutel  %>% pivot_longer(-datum)
plot_verdeel_vast = ggplot(df_plot_verdeel, aes(x=datum)) + geom_step(aes(y = value, color = name, linetype = name, size = name))+ theme_bw()+
  scale_colour_manual(values = colorspec)+
  scale_linetype_manual(values = linespec) +
  scale_size_manual(values = widthspec)+
  theme(legend.position = 'none')+xlab('Date')+ylab('Allocation Probability')+
  scale_y_continuous(labels = scales::percent, limits = c(0,1))

# title <- ggdraw() + 
#   draw_label(
#     "blabla",
#     fontface = 'bold',
#     x = 0,
#     hjust = 0
#   )+
#   theme(
#     # add margin on the left of the drawing canvas,
#     # so title is aligned with left edge of first plot
#     plot.margin = margin(0, 0, 0, 7)
  # )
plot_verloop_vdslt = plot_grid( plot_kl, plot_ic, plot_verdeel, plot_kl_vast, plot_ic_vast, plot_verdeel_vast, ncol = 2, align = "v", byrow = F) 
plot_verloop_vdslt



dagen_overbed_kl_detail = apply(simres$df_bezetting %>% select(contains('Kliniek')) > simres$df_capaciteit%>% select(contains('Kliniek')), 2, sum)
dagen_overbedden_kl_tot = sum(dagen_overbed_kl_detail)
dagen_overbed_ic_detail = apply(simres$df_bezetting %>% select(contains('IC')) > simres$df_capaciteit%>% select(contains('IC')), 2, sum)
dagen_overbedden_ic_tot = sum(dagen_overbed_ic_detail)

overbedden_kl_detail = apply(simres$df_bezetting %>% select(contains('Kliniek')) - simres$df_capaciteit%>% select(contains('Kliniek')), 2, function_overbedden)
overbedden_kl_tot    = sum(overbedden_kl_detail)
overbedden_ic_detail = apply(simres$df_bezetting %>% select(contains('IC')) - simres$df_capaciteit%>% select(contains('IC')), 2, function_overbedden)
overbedden_ic_tot    = sum(overbedden_ic_detail)

dagen_overbed_kl_detail2 = apply(simres_2$df_bezetting %>% select(contains('Kliniek')) > simres_2$df_capaciteit%>% select(contains('Kliniek')), 2, sum)
dagen_overbedden_kl_tot2 = sum(dagen_overbed_kl_detail2)
dagen_overbed_ic_detail2 = apply(simres_2$df_bezetting %>% select(contains('IC')) > simres_2$df_capaciteit%>% select(contains('IC')), 2, sum)
dagen_overbedden_ic_tot2 = sum(dagen_overbed_ic_detail2)

overbedden_kl_detail2 = apply(simres_2$df_bezetting %>% select(contains('Kliniek')) - simres_2$df_capaciteit%>% select(contains('Kliniek')), 2, function_overbedden)
overbedden_kl_tot2    = sum(overbedden_kl_detail2)
overbedden_ic_detail2 = apply(simres_2$df_bezetting %>% select(contains('IC')) - simres_2$df_capaciteit%>% select(contains('IC')), 2, function_overbedden)
overbedden_ic_tot2    = sum(overbedden_ic_detail2)
print(overbedden_kl_detail)
print(overbedden_kl_detail2)
print(overbedden_ic_detail)
print(overbedden_ic_detail2)
print(dagen_overbed_kl_detail)
print(dagen_overbed_kl_detail2)
print(dagen_overbed_ic_detail)
print(dagen_overbed_ic_detail2)
print(apply(simres$df_bezetting %>% select(contains('Kliniek')), 2, mean))
print(apply(simres_2$df_bezetting %>% select(contains('Kliniek')), 2, mean))
print(apply(simres$df_verdeelsleutel[,-1], 2, mean))
print(apply(simres_2$df_verdeelsleutel[,-1], 2, mean))


# ggsave(paste0('regionale_info/resultaten spreiding ziekenhuizen/plot_verloop_verdeelsleutel.pdf'),plot_verloop_vdslt, device = cairo_pdf, width = 175*9/8, height = 175*5/4, units = 'mm')







