rm(list=setdiff(ls(), "df_besmettingen_tot"))
# Laad packages en basisfuncties
source("R/utils.R")              #  basisfuncties + packages inladen
source("R/functies.R")           #  basisfuncties + packages inladen
source("R/functies_condprobs.R") #  basisfuncties + packages inladen

# verkrijg besmettingsdata RIVM, LET OP: laadt RIVM data in dus zorg dat je internet hebt
if(!exists('df_besmettingen_tot')){
  df_besmettingen_tot = read_delim('https://data.rivm.nl/covid-19/COVID-19_aantallen_gemeente_per_dag.csv', delim = ';')
}

evaluatie_horizon = 30*2 #dagen (oftewel 2 maanden)
start_evaluatie = as.Date('2021-01-07') - days(evaluatie_horizon)
store = T

#data aankomsten brabant https://www.nazb.nl/covid-19/dashboard-covid-19-regio-brabant
# ik heb geen data van regio oost
regios = c("Netwerk Acute Zorg Brabant" ,"Acute Zorgregio Oost" ,  'Netwerk Acute Zorg West' ,  "Traumazorgnetwerk Midden-Nederland")
ziekenhuis_regio = c("Acute Zorgregio Oost" = 'rs', "Netwerk Acute Zorg Brabant" = 'ETZ', 'Netwerk Acute Zorg West' = 'Haga', 
                     'Netwerk Acute Zorg regio Zwolle' ='Haga',"Traumazorgnetwerk Midden-Nederland" = 'Haga')

shifts = c("Acute Zorgregio Oost" = 0, "Netwerk Acute Zorg Brabant" = 0, 'Netwerk Acute Zorg West' = 0, 'Netwerk Acute Zorg regio Zwolle' = 0,  "Traumazorgnetwerk Midden-Nederland" = 0)
opschaling_kl = c("Acute Zorgregio Oost" = 8, "Netwerk Acute Zorg Brabant" = 5, 'Netwerk Acute Zorg West' = 2.5, 'Netwerk Acute Zorg regio Zwolle'= 4, "Traumazorgnetwerk Midden-Nederland" = 2.5)
opschaling_ic = c("Acute Zorgregio Oost" = 20, "Netwerk Acute Zorg Brabant" = 2, 'Netwerk Acute Zorg West' = 10,'Netwerk Acute Zorg regio Zwolle' = 9,   "Traumazorgnetwerk Midden-Nederland" = 10)

tibble_arrivals = tibble()

for(regio in regios){
  kliniek_opschaling = opschaling_kl[regio]
  ic_opschaling      = opschaling_ic[regio]
  shift              = shifts[regio]
  
  # laad functie om data in te lezen, evalueer op ziekenhuis
  source("R/00_maak_datasets.R", encoding = "UTF-8")
  list2env(maak_datasets(ziekenhuis_regio[regio]), envir = .GlobalEnv)
  laatste_datumtijd_opnamebestand = datetime_pred
  
  # laad functies om regionale aankomsten te voorspellen op basis besmettingen, evalueer op nieuwe opnames
  # df van besmettingen en opnames (incl EWMA opnames) voor de roaz regio
  data_besmettingen   = df_besmettingen_tot%>% filter(ROAZ_region == regio)%>%
    select( datum = Date_of_publication, Besmettingen = Total_reported, Opnames = Hospital_admission)%>%
    group_by(datum)%>% summarise(Besmettingen = sum(Besmettingen), Opnames = sum(Opnames), .groups = 'drop_last') %>% ungroup()%>%
    filter(as.Date(datum) >= as.Date('2020-07-01')) %>% add_column(num = 1:nrow(.))
  
  data_besmettingen$datum = data_besmettingen$datum + ddays(shifts[regio])
  
  dates                = seq(start_evaluatie, start_evaluatie + days(evaluatie_horizon+1), 'day')
  data_besmettingen    = data_besmettingen %>% add_column(ewma_opn = ewma(data_besmettingen$Opnames, 0.3)) 
  
  #maak dataset voorspelde opnames voor uitsplitsen kliniek en ic
  voorspelde_opnames   = data_besmettingen %>% filter(as.Date(datum) %in% dates)%>%select(-Besmettingen) %>% rename(nieuwe_opn = Opnames) %>%
    mutate(fit = NA, obs = NA, lwr = NA, upr = NA )
  
  # laad functies om aankomsten te splitsen per afdeling, evalueer op voorspelde en geobserveerde opnames
  source("R/02_voorspel_instroom_ic_kl.R", encoding = "UTF-8")
  list2env(voorspel_instroom_ic_kl(voorspelde_opnames, df_nieuwe_opnames, ziekenhuis_regio[regio], voorspeldatum = as.Date('0000-01-01')), envir = .GlobalEnv)
  
  # laad functies survival analyse op ligduren, evalueer op ICset en Kliniekset
  source("R/03_survival_analyse.R", encoding = "UTF-8")
  list2env( survival_analyse(ziekenhuis_regio[regio], ICset, Kliniekset), envir= .GlobalEnv)
  
  #laad functie om systeeminfo op te halen voor DES, evalueer
  source('R/get_system_information.R', encoding = "UTF-8")
  list2env(get_system_information(Kliniekset, ICset, preds_opnames_ic, preds_opnames_kl, 
                                  condprobs_ic, condprobs_kl, condprobs_ic_kl, condprobs_kl_ic, 
                                  condprobs_kl_ic_gbhbp, laatste_datumtijd_opnamebestand ), envir = .GlobalEnv)
  
  if(!exists('p2')){
    tibble_arrivals_kl = tibble(date = voorspelde_opnames$datum, !!paste0('Arrivals ', regio) :=  voorspelde_opnames$ewma_opn*(1-perc_ic)*opschaling_kl[regio])
    tibble_arrivals_ic = tibble(date = voorspelde_opnames$datum, !!paste0('Arrivals ', regio) :=  voorspelde_opnames$ewma_opn*(perc_ic)*opschaling_ic[regio])
    # plot(voorspelde_opnames$datum, voorspelde_opnames$ewma_opn, col = colors[ziekenhuis], type = 'l',ylim = c(0, max(voorspelde_opnames$ewma_opn)), xlab = 'date', ylab = 'Regional Hospitalisations', lwd = 1.2)
    p2 = 1
  }else{
    tibble_arrivals_kl = tibble_arrivals_kl %>% inner_join(tibble(date = voorspelde_opnames$datum, !!paste0('Arrivals ', regio) :=  voorspelde_opnames$ewma_opn*(1-perc_ic)*opschaling_kl[regio]))
    tibble_arrivals_ic = tibble_arrivals_ic %>% inner_join(tibble(date = voorspelde_opnames$datum, !!paste0('Arrivals ', regio) :=  voorspelde_opnames$ewma_opn*perc_ic*opschaling_ic[regio]))
    # lines(voorspelde_opnames$datum, voorspelde_opnames$ewma_opn, col = colors[ziekenhuis], lwd = 1.2)
  }
  
  #sla resultaten op voor regio
  results = mget(c('departure_info', 'preds_opnames_ic','preds_opnames_kl', 'patienttype_info','condprobs_ic_kl','condprobs_kl_ic_gbhbp',
                   'data_besmettingen', 'perc_ic', 'shifts', 'ic_opschaling', 'kliniek_opschaling', 'shift'))
  if(store){
    save(results,  file = paste0('regionale_info/', regio,'.rds'))
  }
}


plot_ar_ward= ggplot(tibble_arrivals_kl, aes(x = date)) +
  geom_line(aes(y = `Arrivals Acute Zorgregio Oost`), size = 1.2, col = 'black')+
  geom_line(aes(y = `Arrivals Netwerk Acute Zorg West`), size = 1.2, col = 'blue')+
  geom_line(aes(y = `Arrivals Netwerk Acute Zorg Brabant`), size = 1.2, col = 'dark green')+
  geom_line(aes_string(y = paste0("`Arrivals ",tail(regios,1),"`")), size = 1.2, col = '#dd1c77')+ylab('Regional Ward Arrival Rate')+
  theme(legend.position = 'none')+  theme_bw()+
  scale_y_continuous(breaks = integer_breaks())+ylim(c(0, max(as.data.frame(tibble_arrivals_kl %>% select(-date )))+2))

plot_ar_icu= ggplot(tibble_arrivals_ic, aes(x = date)) +
  geom_line(aes(y = `Arrivals Acute Zorgregio Oost`), size = 1.2, col = 'black')+
  geom_line(aes(y = `Arrivals Netwerk Acute Zorg West`), size = 1.2, col = 'blue')+
  geom_line(aes(y = `Arrivals Netwerk Acute Zorg Brabant`), size = 1.2, col = 'dark green')+
  geom_line(aes_string(y = paste0("`Arrivals ",tail(regios,1),"`")), size = 1.2, col = '#dd1c77')+
  ylab('Regional ICU Arrival Rate')+
  theme(legend.position = 'none')+  theme_bw()+
  scale_y_continuous(breaks = integer_breaks())+ylim(c(0, max(as.data.frame(tibble_arrivals_ic %>% select(-date )))+2))

print(plot_ar_icu)
print(plot_ar_ward)

if(store){
  ggsave(paste0('regionale_info/resultaten spreiding regionaal/plot_arrival_rates_ward_simstudy.pdf'),plot_ar_ward, device = cairo_pdf, width = 125*3/4, height = 125*3/4, units = 'mm')
  ggsave(paste0('regionale_info/resultaten spreiding regionaal/plot_arrival_rates_icu_simstudy.pdf'),plot_ar_icu, device = cairo_pdf, width = 125*3/4, height = 125*3/4, units = 'mm')
}
