rm(list=setdiff(ls(), c("df_besmettingen_tot")))
# Laad packages en basisfuncties
source("R/utils.R")              #  basisfuncties + packages inladen
source("R/functies.R")           #  basisfuncties + packages inladen
source("R/functies_condprobs.R") #  basisfuncties + packages inladen

# verkrijg besmettingsdata RIVM, LET OP: laadt RIVM data in dus zorg dat je internet hebt
if(!exists('df_besmettingen_tot')){
  df_besmettingen_tot = read_delim('https://data.rivm.nl/covid-19/COVID-19_aantallen_gemeente_per_dag.csv', delim = ';')
}

#params
evaluatie_horizon = 30*2 #dagen (oftewel 2 maanden)
start_evaluatie   = as.Date('2021-01-07') - days(evaluatie_horizon)
dates             = seq(start_evaluatie, start_evaluatie + days(evaluatie_horizon+5), 'day')
store             = T
lijst_zkh         = c('GHZ', 'LUMC','Haga')
regio             = 'Netwerk Acute Zorg West'
schaling_reg_aank = 19/20

##init
Klinieksets = list()
ICsets = list()
list_preds_opnames_ic = list()
list_preds_opnames_kl = list()
list_df_nieuwe_opnames= list()

## verkrijg data per ziekenhuis apart
for(ziekenhuis in lijst_zkh){
  # laad functie om data in te lezen, evalueer op ziekenhuis
  source("R/00_maak_datasets.R", encoding = "UTF-8")
  list2env(maak_datasets(ziekenhuis), envir = .GlobalEnv)
  laatste_datumtijd_opnamebestand = datetime_pred
  
  #df_nieuwe_opnames wordt opgeslagen in lijst om later voor regio IC alles op te kunnen sommen (daarom moeten de beschouwde datums ook hetzelfde zijn)
  list_df_nieuwe_opnames[[ziekenhuis]] = df_nieuwe_opnames %>% filter(as.Date(datum) %in% seq(min(dates)-days(30), max(dates), by = 'day'))
  
  #muteer kliniekset/ICset per regio zodat je ze later onder elkaar kan plakken
  if(!exists('colnames_sets')){colnames_sets = colnames(Kliniekset)}
  Kliniekset                = Kliniekset %>% select(-contains("Ziekenhuis Verplaatsingen")) #soms zit de kolom "Ziekenhuis verplaatsingen" er in.. deze moet er uit om de sets te concateneren
  ICset                     = ICset %>% select(-contains("Ziekenhuis Verplaatsingen"))
  colnames(Kliniekset)      = colnames_sets #kolomnamen homogeniseren
  colnames(ICset)           = colnames_sets
  Klinieksets[[ziekenhuis]] = Kliniekset
  ICsets[[ziekenhuis]]      = ICset
}

# laad functies om regionale aankomsten te voorspellen op basis besmettingen, evalueer op nieuwe opnames
# df van besmettingen en opnames (incl EWMA opnames) voor de roaz regio
data_besmettingen   = df_besmettingen_tot%>% filter(ROAZ_region == regio)%>%
  select( datum = Date_of_publication, Besmettingen = Total_reported, Opnames = Hospital_admission)%>%
  group_by(datum)%>% summarise(Besmettingen = sum(Besmettingen), Opnames = sum(Opnames), .groups = 'drop_last') %>% ungroup()%>%
  filter(as.Date(datum) >= as.Date('2020-07-01')) %>% add_column(num = 1:nrow(.))%>% filter(as.Date(datum) %in% seq(min(dates)-days(30), max(dates), by = 'day'))

## verkrijg data voor het "regionale ziekenhuis"
Kliniekset_regio                = Reduce(rbind, Klinieksets) #plak Klinieksets aan elkaar
ICset_regio                     = Reduce(rbind, ICsets)

preds_opnames_ic          = list_df_nieuwe_opnames[[1]] %>% select(datum, aantal)
preds_opnames_ic$aantal   = apply(sapply(list_df_nieuwe_opnames, function(ls){ls$aantal_ic}),1,sum)
preds_opnames_ic$ewma_opn = ewma(preds_opnames_ic$aantal, 0.1)
preds_opnames_ic          = preds_opnames_ic %>% filter(datum >= as.Date(min(dates)))
# ggplot(preds_opnames_ic, aes(x = datum)) + geom_line(aes(y = ewma_opn)) + geom_point(aes(y = aantal))


preds_opnames_kl          = list_df_nieuwe_opnames[[1]] %>% select(datum, aantal)
preds_opnames_kl$aantal   = apply(sapply(list_df_nieuwe_opnames, function(ls){ls$aantal_kliniek}),1,sum)
preds_opnames_kl$ewma_opn = ewma(preds_opnames_kl$aantal, 0.3)
preds_opnames_kl          = preds_opnames_kl %>% filter(datum >= as.Date(min(dates)))
# ggplot(preds_opnames_kl, aes(x = datum)) + geom_line(aes(y = ewma_opn)) + geom_point(aes(y = aantal))

#opnames in regio is totaal van opnames bij de drie ziekenhuizen
data_besmettingen$Opnames = apply(sapply(list_df_nieuwe_opnames, function(obj){obj$aantal}),1,sum)

#bepaal percentage aankomsten dat direct naar IC gaat
perc_ic = mean(preds_opnames_ic$aantal/data_besmettingen %>% filter(datum >= as.Date(min(dates))) %>% pull(Opnames))

#bepaal vaste verdeelsleutel
historische_verdeelsleutel = sapply(list_df_nieuwe_opnames, function(ls){mean(ls$aantal/ data_besmettingen$Opnames)})

# laad functies survival analyse op ligduren, evalueer op ICset en Kliniekset
source("R/03_survival_analyse.R", encoding = "UTF-8")
list2env( survival_analyse(ziekenhuis, ICset_regio, Kliniekset_regio), envir= .GlobalEnv)

#laad functie om systeeminfo op te halen voor DES, evalueer
source('R/get_system_information.R', encoding = "UTF-8")
list2env(get_system_information(Kliniekset_regio, ICset_regio, preds_opnames_ic, preds_opnames_kl,
                                condprobs_ic, condprobs_kl, condprobs_ic_kl, condprobs_kl_ic,
                                condprobs_kl_ic_gbhbp, laatste_datumtijd_opnamebestand ), envir = .GlobalEnv)
#sla resultaten op voor zkh
results_regio_als_zkh = mget(c('departure_info', 'preds_opnames_ic','preds_opnames_kl', 'patienttype_info','condprobs_ic_kl','condprobs_kl_ic_gbhbp',
                               'data_besmettingen', 'perc_ic', 'historische_verdeelsleutel', 'schaling_reg_aank'))
if(store){
  save(results_regio_als_zkh,  file = paste0('regionale_info/',roaz_regio(ziekenhuis),' als ziekenhuis.rds'))
}



plot_ar_ward  = ggplot(preds_opnames_kl, aes(x = datum)) + geom_line(aes(y = ewma_opn*schaling_reg_aank), size = 1.2, color = 'red') + ylab('Regional Arrival Rate Ward') + xlab('Date')  +theme(legend.position = 'none')+  theme_bw()+
  scale_y_continuous(breaks = integer_breaks())+ylim(c(0, max(as.data.frame(preds_opnames_kl %>% select(-datum, -aantal )))+2))

plot_ar_icu = ggplot(preds_opnames_ic, aes(x = datum)) + geom_line(aes(y = ewma_opn*schaling_reg_aank), size = 1.2, color = 'red') + ylab('Regional Arrival Rate ICU') + xlab('Date')  +theme(legend.position = 'none')+  theme_bw()+
  scale_y_continuous(breaks = integer_breaks())+ylim(c(0, max(as.data.frame(preds_opnames_ic %>% select(-datum, -aantal )))+1))


if(store){
  ggsave(paste0('regionale_info/resultaten spreiding ziekenhuizen/plot_arrival_rate_ward_one_region_simstudy.pdf'),plot_ar_ward, device = cairo_pdf, width = 125*3/4, height = 125*3/4, units = 'mm')
  ggsave(paste0('regionale_info/resultaten spreiding ziekenhuizen/plot_arrival_rate_icu_one_region_simstudy.pdf'),plot_ar_icu, device = cairo_pdf, width = 125*3/4, height = 125*3/4, units = 'mm')
}



