source('R/utils.R')
require(reshape2)
require(tibbletime)
library(xtable)
library(scales)
library(tables)

genereer_namen_verplaatsingen_tussen_cl = function(vec_regios_cluster){
  vec_namen_verplaatsingen_tussen_cl = c()
  for(naam_regio_1 in vec_regios_cluster){
    for(naam_regio_2 in vec_regios_cluster[vec_regios_cluster != naam_regio_1]){
      vec_namen_verplaatsingen_tussen_cl = c(vec_namen_verplaatsingen_tussen_cl,
                                             paste0('van ', naam_regio_1, ' naar ', naam_regio_2))
    }
  }
  return(vec_namen_verplaatsingen_tussen_cl)
}

path = './regionale_info/resultaten spreiding regionaal/simulaties/resultaten artikel/'
files <- list.files(path = path ,pattern = '.rds')

namen_engels = c('plek' = 'Any Surplus', 'beddenaantal'='Num. Beds', 'CHOIR' = 'Optimisation',
                 'beddenaantal_0'='Num. Beds (k=0)', 'beddenaantal_1'='Num. Beds (k=1)')
kpis_tot = list()
save_results = T
include_totcost = T
upper_bound_sims = 250
level = 0.9
#params
clusters = list(c('Acute Zorgregio Oost', "Traumazorgnetwerk Midden-Nederland"),
                  c('Netwerk Acute Zorg West', 'Netwerk Acute Zorg Brabant'))
geen_clusters = list(c('Acute Zorgregio Oost',  'Netwerk Acute Zorg West'),
                     c('Acute Zorgregio Oost',  'Netwerk Acute Zorg Brabant'),
                     c("Traumazorgnetwerk Midden-Nederland", 'Netwerk Acute Zorg West'),
                     c("Traumazorgnetwerk Midden-Nederland", 'Netwerk Acute Zorg Brabant'))

kolomnamen_verpl_binnen_cl = as.vector(sapply(clusters, function(v){genereer_namen_verplaatsingen_tussen_cl(v)}))
kolomnamen_verpl_tussen_cl = as.vector(sapply(geen_clusters, function(v){genereer_namen_verplaatsingen_tussen_cl(v)}))

namen_strategien = names(load_obj(paste0(path, files[2]))$lijst_info)#c('plek', 'beddenaantal')#, 'CHOIR'
nsim = 0
for(filename in files){
  loadres = load_obj(paste0(path, filename))
  for(naam_strategie in namen_strategien){
    kpis_strat = c()
    res  =loadres$lijst_info[[naam_strategie]]
    if(strcmp(naam_strategie, namen_strategien[1])){
      nsim = nsim + length(res)
    }
    for(lijst_info_strat in res ){
      verplaatsingen_ic_binnen_clusters =  sum(lijst_info_strat$overplaatsingen_ic %>% select(kolomnamen_verpl_binnen_cl))
      verplaatsingen_kl_binnen_clusters =  sum(lijst_info_strat$overplaatsingen_kl %>% select(kolomnamen_verpl_binnen_cl))
      
      verplaatsingen_ic_tussen_clusters =  sum(lijst_info_strat$overplaatsingen_ic %>% select(kolomnamen_verpl_tussen_cl))
      verplaatsingen_kl_tussen_clusters =  sum(lijst_info_strat$overplaatsingen_kl %>% select(kolomnamen_verpl_tussen_cl))
      
      verplaatsingen_ic_outer =  sum(lijst_info_strat$overplaatsingen_ic %>% select(contains('Buitenland')))
      verplaatsingen_kl_outer = sum(lijst_info_strat$overplaatsingen_kl %>% select(contains('Buitenland')))
      
      verplaatsingen_ic_tot =  sum(lijst_info_strat$overplaatsingen_ic %>% select(-datum))
      verplaatsingen_kl_tot = sum(lijst_info_strat$overplaatsingen_kl %>% select(-datum))
      
      kosten_verpl_ic = sum(as.matrix(as.data.frame(lijst_info_strat$overplaatsingen_ic)[,-1])%*%kosten_transport)/verplaatsingen_ic_tot
      kosten_verpl_kl = sum(as.matrix(as.data.frame(lijst_info_strat$overplaatsingen_kl)[,-1])%*%kosten_transport)/verplaatsingen_kl_tot
      
      if(include_totcost){
        kpis_strat = rbind(kpis_strat, c(verplaatsingen_ic_tot, verplaatsingen_ic_outer, verplaatsingen_ic_binnen_clusters,verplaatsingen_ic_tussen_clusters,kosten_verpl_ic,
                                         verplaatsingen_kl_tot, verplaatsingen_kl_outer, verplaatsingen_kl_binnen_clusters,verplaatsingen_kl_tussen_clusters,kosten_verpl_kl
        ))
      }else{
        kpis_strat = rbind(kpis_strat, c(verplaatsingen_ic_tot, verplaatsingen_ic_outer, verplaatsingen_ic_binnen_clusters,verplaatsingen_ic_tussen_clusters,
                                         verplaatsingen_kl_tot, verplaatsingen_kl_outer, verplaatsingen_kl_binnen_clusters,verplaatsingen_kl_tussen_clusters
        ))
      }
      
      }
    
    kpis_tot[[naam_strategie]] = rbind(kpis_tot[[naam_strategie]], kpis_strat)
    if(nrow(kpis_tot[[naam_strategie]]) > upper_bound_sims){
      kpis_tot[[naam_strategie]] = head(kpis_tot[[naam_strategie]], upper_bound_sims)
    }
  }  
}


df_means = c()
df_radii = c()
for(naam_strategie in namen_strategien){
  df_means = rbind(df_means, apply(kpis_tot[[naam_strategie]], 2, function(v){mean(as.numeric(v), na.rm = T)} ))
  df_radii = rbind(df_radii, apply(kpis_tot[[naam_strategie]], 2, function(v){qt(0.975, length(v)-1)*sd(as.numeric(v), na.rm = T)/sqrt(length(v[!is.na(v)]))}))
}
df_means_long = as_tibble(df_means) %>% add_column(policy = namen_engels[namen_strategien], .before = 1) %>% pivot_longer(-policy) %>% rename(mean = value)
df_radii_long = as_tibble(df_radii) %>% add_column(policy = namen_engels[namen_strategien], .before = 1) %>% pivot_longer(-policy) %>% rename(radius = value)

df_kpis_long                                 = df_means_long %>% left_join(df_radii_long, c('policy', 'name'))
means_formatted                              = gsub(' ', '\\\\phantom{0}', format(round(df_kpis_long$mean,2), nsmall = 2))
radii_formatted                              = gsub(' ', '\\\\phantom{0}', format(round(df_kpis_long$radius,2), nsmall = 2))
df_kpis_long$value                           = sapply(1:length(means_formatted), function(i){paste0('$\\text{',means_formatted[i],' $\\pm$ ', radii_formatted[i], '}$')})
df_kpis_long$value[is.na(df_kpis_long$mean)] = '-'

if(include_totcost){
  df_kpis_long                                 = df_kpis_long %>% select(-name) %>% add_column(dept = rep(rep(c('ICU', 'Ward'), each = 5), length(namen_strategien))) %>%
    add_column(kpi = rep(c('Total',  'External', 'In Clusters', 'Btw. Clusters', 'Average Cost'),2*length(namen_strategien)))
}else{
  df_kpis_long                                 = df_kpis_long %>% select(-name) %>% add_column(dept = rep(rep(c('ICU', 'Ward'), each = 4), length(namen_strategien))) %>%
    add_column(kpi = rep(c('Total',  'External', 'In Clusters', 'Btw. Clusters'),2*length(namen_strategien)))
}



results_table <- tabular(RowFactor(kpi, name = 'KPI')*RowFactor(dept, name = 'Dept.') ~
                           Heading()*Factor(policy)*Heading()*(value)*Heading()*(toString)*
                           DropEmpty(empty=""),
                         data = df_kpis_long %>% filter(policy %in% c('Any Surplus', 'Num. Beds', 'Optimisation')))

if(include_totcost){
  results_table = results_table[c(10,9, 6,5, 8,7, 4,3,2,1),]
}else{
  results_table = results_table[c(10,9, 6,5, 8,7, 4,3)-2,]
}
results_table
if(save_results){
write(tables::latex.tabular(results_table)[[1]], file = 'regionale_info/resultaten spreiding regionaal/tabel_spreiding_regios.tex')
}

# if(strcmp(path ,'./regionale_info/resultaten spreiding regionaal/simulaties/resultaten artikel//')){
#   list_df_kpis_long = list(df_kpis_long %>% filter(policy %in% c('Num. Beds (k=0)')) %>% mutate(policy = rep('Num. Beds', 8)) %>% filter(kpi != 'Total'),
#                            df_kpis_long %>% filter(policy %in% c('Num. Beds (k=1)')) %>% mutate(policy = rep('Num. Beds', 8)) %>% filter(kpi != 'Total'),
#                            df_kpis_long %>% filter(policy %in% c('Num. Beds')) %>% filter(kpi != 'Total'))
#   
#   
#     df_different_k = list_df_kpis_long[[1]] %>% select(-mean, -radius) %>% add_column(k = '$k_{as} = 0$') %>%
#                      full_join(list_df_kpis_long[[2]] %>% select(-mean, -radius) %>% add_column(k = '$k_{as} = 1$')) %>%
#                      full_join(list_df_kpis_long[[3]] %>% select(-mean, -radius) %>% filter(policy == 'Num. Beds')%>% add_column(k= '$k_{as} = 2$'))%>%
#                      full_join(df_kpis_long %>% select(-mean, -radius) %>% filter(policy == 'Optimisation') %>% filter(kpi != 'Total')%>% add_column(k= '\\phantom{x}') )
#     
#     results_table_different_k <- tabular(RowFactor(kpi, name = 'KPI')*RowFactor(dept, name = 'Dept.') ~
#                                Heading()*Factor(k, name = '$k$')*Heading()*Factor(policy)*Heading()*(value)*Heading()*(toString)*
#                                DropEmpty(empty=""),
#                              data = df_different_k)
#   
#   results_table_different_k = results_table_different_k[c(8,7,6,5,4,3,2,1),]
#   results_table_different_k
#   if(save_results){
#     write(tables::latex.tabular(results_table_different_k)[[1]], file = 'regionale_info/resultaten spreiding regionaal/tabel_spreiding_regios_meerdere_k.tex')
#   }
# }else{
  list_df_kpis_long = list()
  ks = c(0,1)
  for(k in ks){
    path = paste0('./regionale_info/resultaten spreiding regionaal/simulaties/resultaten k = ',k,'/')
    files <- list.files(path = path ,pattern = '.rds')
    namen_strategien = c( 'beddenaantal')
    kpis_tot = list()
    save_results = F
    upper_bound_sims = 250
    level = 0.9
    
    #params
    clusters = list(c('Acute Zorgregio Oost', "Traumazorgnetwerk Midden-Nederland"),
                    c('Netwerk Acute Zorg West', 'Netwerk Acute Zorg Brabant'))
    geen_clusters = list(c('Acute Zorgregio Oost',  'Netwerk Acute Zorg West'),
                         c('Acute Zorgregio Oost',  'Netwerk Acute Zorg Brabant'),
                         c("Traumazorgnetwerk Midden-Nederland", 'Netwerk Acute Zorg West'),
                         c("Traumazorgnetwerk Midden-Nederland", 'Netwerk Acute Zorg Brabant'))
    
    kolomnamen_verpl_binnen_cl = as.vector(sapply(clusters, function(v){genereer_namen_verplaatsingen_tussen_cl(v)}))
    kolomnamen_verpl_tussen_cl = as.vector(sapply(geen_clusters, function(v){genereer_namen_verplaatsingen_tussen_cl(v)}))
    
    for(filename in files){
      loadres = load_obj(paste0(path, filename))
      for(naam_strategie in namen_strategien){
        kpis_strat = c()
        res  =loadres$lijst_info[[naam_strategie]]
        for(lijst_info_strat in res ){
          verplaatsingen_ic_binnen_clusters =  sum(lijst_info_strat$overplaatsingen_ic %>% select(kolomnamen_verpl_binnen_cl))
          verplaatsingen_kl_binnen_clusters =  sum(lijst_info_strat$overplaatsingen_kl %>% select(kolomnamen_verpl_binnen_cl))
          
          verplaatsingen_ic_tussen_clusters =  sum(lijst_info_strat$overplaatsingen_ic %>% select(kolomnamen_verpl_tussen_cl))
          verplaatsingen_kl_tussen_clusters =  sum(lijst_info_strat$overplaatsingen_kl %>% select(kolomnamen_verpl_tussen_cl))
          
          verplaatsingen_ic_outer =  sum(lijst_info_strat$overplaatsingen_ic %>% select(contains('Buitenland')))
          verplaatsingen_kl_outer = sum(lijst_info_strat$overplaatsingen_kl %>% select(contains('Buitenland')))
          
          verplaatsingen_ic_tot =  sum(lijst_info_strat$overplaatsingen_ic %>% select(-datum))
          verplaatsingen_kl_tot = sum(lijst_info_strat$overplaatsingen_kl %>% select(-datum))
          
          kosten_verpl_ic = sum(as.matrix(as.data.frame(lijst_info_strat$overplaatsingen_ic)[,-1])%*%kosten_transport)/verplaatsingen_ic_tot
          kosten_verpl_kl = sum(as.matrix(as.data.frame(lijst_info_strat$overplaatsingen_kl)[,-1])%*%kosten_transport)/verplaatsingen_kl_tot
          
          if(include_totcost){
            kpis_strat = rbind(kpis_strat, c(verplaatsingen_ic_tot, verplaatsingen_ic_outer, verplaatsingen_ic_binnen_clusters,verplaatsingen_ic_tussen_clusters,kosten_verpl_ic,
                                             verplaatsingen_kl_tot, verplaatsingen_kl_outer, verplaatsingen_kl_binnen_clusters,verplaatsingen_kl_tussen_clusters,kosten_verpl_kl
            ))
          }else{
            kpis_strat = rbind(kpis_strat, c(verplaatsingen_ic_tot, verplaatsingen_ic_outer, verplaatsingen_ic_binnen_clusters,verplaatsingen_ic_tussen_clusters,
                                             verplaatsingen_kl_tot, verplaatsingen_kl_outer, verplaatsingen_kl_binnen_clusters,verplaatsingen_kl_tussen_clusters
            ))
          }
          
        }
        
        kpis_tot[[naam_strategie]] = rbind(kpis_tot[[naam_strategie]], kpis_strat)
        if(nrow(kpis_tot[[naam_strategie]]) > upper_bound_sims){
          kpis_tot[[naam_strategie]] = head(kpis_tot[[naam_strategie]], upper_bound_sims)
        }
      }  
    }
    
    df_means = c()
    df_radii = c()
    for(naam_strategie in namen_strategien){
      df_means = rbind(df_means, apply(kpis_tot[[naam_strategie]], 2, function(v){mean(as.numeric(v), na.rm = T)} ))
      df_radii = rbind(df_radii, apply(kpis_tot[[naam_strategie]], 2, function(v){qt(0.975, length(v)-1)*sd(as.numeric(v), na.rm = T)/sqrt(length(v[!is.na(v)]))}))
    }
    df_means_long = as_tibble(df_means) %>% add_column(policy = namen_engels[namen_strategien], .before = 1) %>% pivot_longer(-policy) %>% rename(mean = value)
    df_radii_long = as_tibble(df_radii) %>% add_column(policy = namen_engels[namen_strategien], .before = 1) %>% pivot_longer(-policy) %>% rename(radius = value)
    
    df_kpis_long_k                                 = df_means_long %>% left_join(df_radii_long, c('policy', 'name'))
    means_formatted                              = gsub(' ', '\\\\phantom{0}', format(round(df_kpis_long_k$mean,2), nsmall = 2))
    radii_formatted                              = gsub(' ', '\\\\phantom{0}', format(round(df_kpis_long_k$radius,2), nsmall = 2))
    df_kpis_long_k$value                           = sapply(1:length(means_formatted), function(i){paste0('$\\text{',means_formatted[i],' $\\pm$ ', radii_formatted[i], '}$')})
    df_kpis_long_k$value[is.na(df_kpis_long_k$mean)] = '-'
    
    
    if(include_totcost){
      df_kpis_long_k                                 = df_kpis_long_k %>% select(-name) %>% add_column(dept = rep(rep(c('ICU', 'Ward'), each = 5), length(namen_strategien))) %>%
        add_column(kpi = rep(c('Total',  'External', 'In Clusters', 'Btw. Clusters', 'Avg. Realloc. Cost'),2*length(namen_strategien)))
    }else{
      df_kpis_long_k                                 = df_kpis_long_k %>% select(-name) %>% add_column(dept = rep(rep(c('ICU', 'Ward'), each = 4), length(namen_strategien))) %>%
        add_column(kpi = rep(c('Total',  'External', 'In Clusters', 'Btw. Clusters'),2*length(namen_strategien)))
    }
  
    list_df_kpis_long[[toString(k)]] = df_kpis_long_k
  }
  
  df_different_k = list_df_kpis_long[[1]] %>% select(-mean, -radius) %>% add_column(k = '$k_{as} = 0$') %>%
    full_join(list_df_kpis_long[[2]] %>% select(-mean, -radius) %>% add_column(k = '$k_{as} = 1$')) %>%
    full_join(df_kpis_long %>% select(-mean, -radius) %>% filter(policy == 'Num. Beds')%>% add_column(k= '$k_{as} = 2$')%>% filter(kpi != 'Total'))%>%
    full_join(df_kpis_long %>% select(-mean, -radius) %>% filter(policy == 'Optimisation')%>% add_column(k= '\\phantom{x}')%>% filter(kpi != 'Total') )
  
  results_table_different_k <- tabular(RowFactor(kpi, name = 'KPI')*RowFactor(dept, name = 'Dept.') ~
                                         Heading()*Factor(k, name = '$k$')*Heading()*Factor(policy)*Heading()*(value)*Heading()*(toString)*
                                         DropEmpty(empty=""),
                                       data = df_different_k)
  
  results_table_different_k = results_table_different_k[c( 6,5, 8,7, 4,3,2,1),]
  
  if(save_results){
    write(tables::latex.tabular(results_table_different_k)[[1]], file = 'regionale_info/resultaten spreiding regionaal/tabel_spreiding_regios_meerdere_k.tex')
  }
# }



