rm(list=ls())
source('R/utils.R')
require(reshape2)
require(tibbletime)
library(xtable)
library(scales)
library(tables)
library(cowplot)

path = 'regionale_info/resultaten spreiding ziekenhuizen/simulaties original 2/'
files <- list.files(path = path ,pattern = '.rds')
namen_engels = c("optimale verdeelsleutel" = 'Load Balancing' , "individuele ziekenhuizen" = 'Indiv. Hospitals', 'regionaal ziekenhuis' = 'Reg. Hospital')
kpis_tot = list()
save_results = F
upper_bound_sims = 250
namen_strategien = c( "individuele ziekenhuizen"  ,"regionaal ziekenhuis","optimale verdeelsleutel"    )
names_kpis = c('Over-bed Days Ward ','Over-beds Ward ', 'Over-bed Days ICU ', 'Over-beds ICU ')
# function_overbedden = function(v){sum(diff(sapply(v,function(x){max(x,0)}))[diff(sapply(v,function(x){max(x,0)}))>0])}
function_overbedden = function(v){mean(sapply(v,function(x){max(x,0)}))}

nsim = 0
for(filename in files){
  loadres = load_obj(paste0(path, filename))
  # gem_bezetting = list(0,0,0)
  # names(gem_bezetting) = namen_strategien
  # AD_bezetting = list(0,0,0)
  # names(AD_bezetting) = namen_strategien
  # k=list(1,1,1)
  # names(k) = namen_strategien
  for(naam_strategie in namen_strategien){
    simresults  = loadres$simulatie_resultaten[[naam_strategie]]
    if(strcmp(naam_strategie , namen_strategien[1])){
      nsim = nsim + length(simresults)
    }
    kpis_partial = c()
    for(res in simresults){
      row_results       = c()
      if(naam_strategie != 'regionaal ziekenhuis'){
        dagen_overbed_kl_detail = apply(res$df_bezetting %>% select(contains('Kliniek')) > res$df_capaciteit%>% select(contains('Kliniek')), 2, sum)
        dagen_overbedden_kl_tot = sum(apply(res$df_bezetting %>% select(contains('Kliniek')) > res$df_capaciteit%>% select(contains('Kliniek')), 1, any))
        dagen_overbed_ic_detail = apply(res$df_bezetting %>% select(contains('IC')) > res$df_capaciteit%>% select(contains('IC')), 2, sum)
        dagen_overbedden_ic_tot = sum(apply(res$df_bezetting %>% select(contains('IC')) > res$df_capaciteit%>% select(contains('IC')), 1, any))
        
        overbedden_kl_detail = apply(res$df_bezetting %>% select(contains('Kliniek')) - res$df_capaciteit%>% select(contains('Kliniek')), 2, function_overbedden)
        overbedden_kl_tot    = sum(overbedden_kl_detail)
        overbedden_ic_detail = apply(res$df_bezetting %>% select(contains('IC')) - res$df_capaciteit%>% select(contains('IC')), 2, function_overbedden)
        overbedden_ic_tot    = sum(overbedden_ic_detail)
        bezetting_kl_detail = apply(res$df_bezetting %>% select(contains('Kliniek')),2,mean)
        # bezetting_kl_tot = sum(bezetting_kl_detail)
      }else{
        dagen_overbed_kl_detail = rep(NA,3)
        dagen_overbedden_kl_tot = apply(res$df_bezetting %>% select(contains('Kliniek')) > res$df_capaciteit%>% select(contains('Kliniek')), 2, sum)
        dagen_overbed_ic_detail = rep(NA,3)
        dagen_overbedden_ic_tot = apply(res$df_bezetting %>% select(contains('IC')) > res$df_capaciteit%>% select(contains('IC')), 2, sum)
        
        overbedden_kl_tot    = apply(res$df_bezetting %>% select(contains('Kliniek')) - res$df_capaciteit%>% select(contains('Kliniek')), 2, function_overbedden)
        overbedden_kl_detail = rep(NA,3)
        overbedden_ic_tot    = apply(res$df_bezetting %>% select(contains('IC')) - res$df_capaciteit%>% select(contains('IC')), 2, function_overbedden)
        overbedden_ic_detail = rep(NA,3)
        bezetting_kl_detail = rep(NA,3)
        # bezetting_kl_tot = apply(res$df_bezetting %>% select(contains('Kliniek')) ,2,mean)
      }
      row_results = c(dagen_overbed_kl_detail, dagen_overbedden_kl_tot, overbedden_kl_detail, overbedden_kl_tot, 
                      dagen_overbed_ic_detail, dagen_overbedden_ic_tot, overbedden_ic_detail, overbedden_ic_tot)
      kpis_partial = rbind(kpis_partial, row_results)
    }
    colnames(kpis_partial) = as.vector(sapply(names_kpis,
                                    function(name){sapply(names(res$data_start_zkh), function(name2){paste0(name, name2)})}))
    
      
    kpis_tot[[naam_strategie]] = rbind(kpis_tot[[naam_strategie]], kpis_partial)
    if(nrow(kpis_tot[[naam_strategie]])>upper_bound_sims){
      kpis_tot[[naam_strategie]] = head(kpis_tot[[naam_strategie]],upper_bound_sims)
    }
  }
}

df_means = c()
df_radii = c()
for(naam_strategie in namen_strategien){
  df_means = rbind(df_means, apply(kpis_tot[[naam_strategie]], 2, function(v){mean(as.numeric(v), na.rm = T)} ))
  df_radii = rbind(df_radii, apply(kpis_tot[[naam_strategie]], 2, function(v){qt(0.975, length(v)-1)*sd(as.numeric(v), na.rm = T)/sqrt(length(v[!is.na(v)]))}))
}
df_means_long = as_tibble(df_means) %>% add_column(policy = namen_engels[namen_strategien], .before = 1) %>% pivot_longer(-policy) %>% rename(mean = value)
df_radii_long = as_tibble(df_radii) %>% add_column(policy = namen_engels[namen_strategien], .before = 1) %>% pivot_longer(-policy) %>% rename(radius = value)


df_kpis_long                                 = df_means_long %>% left_join(df_radii_long, c('policy', 'name'))
means_formatted                              =  gsub(" ", "\\\\phantom{0}", format(round(df_kpis_long$mean,2), nsmall = 2))
radii_formatted                              =  gsub(' ', '\\\\phantom{0}', format(round(df_kpis_long$radius,2), nsmall = 2))
df_kpis_long$value                           = sapply(1:length(means_formatted), function(i){paste0('$\\text{',means_formatted[i],' $\\pm$ ', radii_formatted[i], '}$')})
df_kpis_long$value[is.na(df_kpis_long$mean)] = '$\\text{\\phantom{0.0000}-\\phantom{00.00}}$'
df_kpis_long                                 = df_kpis_long %>% add_column(Hospital = rep(c('LUMC','Haga', 'GHZ', 'Region'), length(names_kpis)*length(namen_strategien))) %>%
                                                                add_column(KPI = rep(rep(names_kpis, each = 4),length(namen_strategien)))

results_table <- tabular(RowFactor(Hospital, name = 'Hospital')*RowFactor(KPI, name = 'KPI') ~
                           Heading()*Factor(policy)*Heading()*(value)*Heading()*(toString)*
                           DropEmpty(empty=""),
                          data = df_kpis_long)


perm = c(2,4,1,3)
results_table = results_table[c(perm + length(perm)*2, perm + length(perm),perm,  perm + length(perm)*3),]
# results_table = results_table[c(2,4,1,3,6,8,5,7,10,12,9,11,14,16,13,15),]

results_table
if(save_results){
  write(tables::latex.tabular(results_table)[[1]], file = 'regionale_info/resultaten spreiding ziekenhuizen/res_tabel_spreiding_ziekenhuizen.tex')
}



path = 'regionale_info/resultaten spreiding ziekenhuizen/simulaties original/'
files <- list.files(path = path ,pattern = '.rds')
loadres = load_obj(paste0(path, tail(files,1)))

i_sim=4
naam_strategie = 'optimale verdeelsleutel' #'individuele ziekenhuizen'
simres = loadres$simulatie_resultaten[[naam_strategie]][[i_sim]]

#bepaal verdeelsleutel
historische_verdeelsleutel = as.numeric(loadres$simulatie_resultaten[['individuele ziekenhuizen']][[1]]$df_verdeelsleutel[1,-1])[c(2,3,1)]
df_verdeelsleutel = simres$df_verdeelsleutel
df_verdeelsleutel[,-1] = simres$df_verdeelsleutel[,-1]/(as.matrix(apply(simres$df_verdeelsleutel[,-1], 1, sum))%*%ones(1,3))
indc_all_0 = apply(df_verdeelsleutel[,-1], 1, function(v){any(is.na(v))})
df_verdeelsleutel[indc_all_0,-1] = ones(sum(indc_all_0),1)%*%historische_verdeelsleutel

#bepaal historische capaciteit & bezetting
df_capaciteit =simres$df_capaciteit
df_bezetting = simres$df_bezetting


df_tot =  df_bezetting %>% left_join(df_verdeelsleutel, by ='datum') %>% left_join(df_capaciteit, by ='datum')

panes = c('Ward' = 'Kliniek', 'ICU'='IC', 'Allocation'='verdeelsleutel')
df_tot_long = pivot_longer(df_tot, c(-datum))
df_tot_long = df_tot_long %>% add_column(pane = factor(sapply(df_tot_long$name, function(name1){names(panes)[sapply(panes, function(name2){grepl(name2, name1)})]}),
                                                       levels = c('Ward', 'ICU', 'Allocation')))


colors = c( '#2b83ba','#3ded97', '#fdae80')
linetypes = c('solid', 'solid', 'solid')
linewidths = c(1.1*5/24, 1.1*5/12,1.1)
unq_names = unique(df_tot_long$name)
colorspec = c()
linespec  = c()
widthspec = c()
names_colorspec = c()
simuleerdata_init = simres$simuleerdata_init
for(i in 1:length(names(simuleerdata_init))){
  zkh = names(simuleerdata_init)[i]
  color = colors[i]
  linetype  = linetypes[i]
  linewidth = linewidths[i]
  names_zkh = unq_names[sapply(unq_names, function(name){grepl(zkh,name)})]
  names_colorspec = c(names_colorspec, names_zkh)
  colorspec = c(colorspec, rep(color, length(names_zkh)))
  linespec  = c(linespec , rep(linetype, length(names_zkh)))
  widthspec = c(widthspec, rep(linewidth, length(names_zkh)))
}
names(colorspec) = names_colorspec
names(linespec)  = names_colorspec
names(widthspec) = names_colorspec
colorspec = rev(colorspec)
labels = c('Ward' = 'Occupancy Wards', 'ICU' = 'Occupancy ICU', 'Allocation' = "Allocation probability") 



df_plot_bez_kl = df_bezetting %>% select('datum' | contains('Kliniek'))  %>%
  pivot_longer(-datum)
df_plot_cap_kl = df_capaciteit %>% select('datum' | contains('Kliniek'))  %>%
  pivot_longer(-datum)

plot_kl = ggplot() +  geom_line(data = df_plot_cap_kl, aes(x = datum, y = value, color = name, size = name), linetype = 'dashed')+ 
  geom_step(data = df_plot_bez_kl, aes(x = datum, y = value, color = name, linetype = name, size = name))+
  theme_bw()+ scale_colour_manual(values = colorspec)+
  scale_size_manual(values = widthspec)+
  scale_linetype_manual(values = linespec) +
  theme(legend.position = 'none')+xlab('Date') + ylab('Occupancy Ward') +
  scale_y_continuous(breaks = integer_breaks())+ylim(c(0, 61))


df_plot_bez_ic = df_bezetting %>% select('datum' | contains('IC')) %>% 
  pivot_longer(-datum)
df_plot_cap_ic =  df_capaciteit %>% select('datum' | contains('IC')) %>% 
  pivot_longer(-datum)
plot_ic = ggplot() +geom_line(data = df_plot_cap_ic, aes(x = datum, y = value, color = name, size = name), linetype = 'dashed')+
  geom_step(data = df_plot_bez_ic, aes(x = datum, y = value, color = name, linetype = name, size = name))+ theme_bw()+
  scale_colour_manual(values = colorspec)+
  scale_linetype_manual(values = linespec) +
  scale_size_manual(values = widthspec)+
  theme(legend.position = 'none')+xlab('Date')+ylab('Occupancy ICU')+
  scale_y_continuous(breaks = integer_breaks())+ylim(c(0, 32))

df_plot_verdeel = df_verdeelsleutel  %>% pivot_longer(-datum)
plot_verdeel = ggplot(df_plot_verdeel, aes(x=datum)) + geom_step(aes(y = value, color = name, linetype = name, size = name))+ theme_bw()+
  scale_colour_manual(values = colorspec)+
  scale_linetype_manual(values = linespec) +
  scale_size_manual(values = widthspec)+
  theme(legend.position = 'none')+xlab('Date')+ylab('Allocation Probability')+
  scale_y_continuous(labels = scales::percent, limits = c(0,1))




naam_strategie = 'individuele ziekenhuizen' #'optimale verdeelsleutel'
simres = loadres$simulatie_resultaten[[naam_strategie]][[i_sim]]

#bepaal verdeelsleutel
historische_verdeelsleutel = as.numeric(loadres$simulatie_resultaten[['individuele ziekenhuizen']][[1]]$df_verdeelsleutel[1,-1])[c(2,3,1)]
df_verdeelsleutel = simres$df_verdeelsleutel
df_verdeelsleutel[,-1] = simres$df_verdeelsleutel[,-1]/(as.matrix(apply(simres$df_verdeelsleutel[,-1], 1, sum))%*%ones(1,3))
indc_all_0 = apply(df_verdeelsleutel[,-1], 1, function(v){any(is.na(v))})
df_verdeelsleutel[indc_all_0,-1] = ones(sum(indc_all_0),1)%*%historische_verdeelsleutel

#bepaal historische capaciteit & bezetting
df_capaciteit =simres$df_capaciteit
df_bezetting = simres$df_bezetting

df_plot_bez_kl = df_bezetting %>% select('datum' | contains('Kliniek'))  %>%
  pivot_longer(-datum)
df_plot_cap_kl = df_capaciteit %>% select('datum' | contains('Kliniek'))  %>%
  pivot_longer(-datum)

plot_kl_vast = ggplot() +  geom_line(data = df_plot_cap_kl, aes(x = datum, y = value, color = name, size = name), linetype = 'dashed')+ 
  geom_step(data = df_plot_bez_kl, aes(x = datum, y = value, color = name, linetype = name, size = name))+
  theme_bw()+ scale_colour_manual(values = colorspec)+
  scale_size_manual(values = widthspec)+
  scale_linetype_manual(values = linespec) +
  theme(legend.position = 'none')+xlab('Date') + ylab('Occupancy Ward') +
  scale_y_continuous(breaks = integer_breaks())+ylim(c(0, 61))


df_plot_bez_ic = df_bezetting %>% select('datum' | contains('IC')) %>% 
  pivot_longer(-datum)
df_plot_cap_ic =  df_capaciteit %>% select('datum' | contains('IC')) %>% 
  pivot_longer(-datum)
plot_ic_vast = ggplot() +geom_line(data = df_plot_cap_ic, aes(x = datum, y = value, color = name, size = name), linetype = 'dashed')+
  geom_step(data = df_plot_bez_ic, aes(x = datum, y = value, color = name, linetype = name, size = name))+ theme_bw()+
  scale_colour_manual(values = colorspec)+
  scale_linetype_manual(values = linespec) +
  scale_size_manual(values = widthspec)+
  theme(legend.position = 'none')+xlab('Date')+ylab('Occupancy ICU')+
  scale_y_continuous(breaks = integer_breaks())+ylim(c(0, 32))

df_plot_verdeel = df_verdeelsleutel  %>% pivot_longer(-datum)
plot_verdeel_vast = ggplot(df_plot_verdeel, aes(x=datum)) + geom_step(aes(y = value, color = name, linetype = name, size = name))+ theme_bw()+
  scale_colour_manual(values = colorspec)+
  scale_linetype_manual(values = linespec) +
  scale_size_manual(values = widthspec)+
  theme(legend.position = 'none')+xlab('Date')+ylab('Allocation Probability')+
  scale_y_continuous(labels = scales::percent, limits = c(0,1))


plot_verloop_vdslt = plot_grid(plot_kl, plot_ic, plot_verdeel, plot_kl_vast, plot_ic_vast, plot_verdeel_vast, ncol = 2, align = "v", byrow = F)
ggsave(paste0('regionale_info/resultaten spreiding ziekenhuizen/plot_verloop_verdeelsleutel.pdf'),plot_verloop_vdslt, device = cairo_pdf, width = 175*9/8, height = 175*5/4, units = 'mm')




