fit_besmettingen_opnames_old = function(data_besmettingen, horizon = 0, max_peaks = 2, print_ = F, plot_ = F){

  if(horizon == 0){
    log_besm = head(fit_curve_krijg_voorspellingen(data_besmettingen, horizon=1, max_peaks = max_peaks),-1)
    data_besmettingen$log_besm = log_besm$log
  }else{
    log_besm = fit_curve_krijg_voorspellingen(data_besmettingen, horizon=horizon, max_peaks = max_peaks)
    data_besmettingen$log_besm = head(log_besm, - horizon)$log
  }
  
  
  #fit een optimale vertraging en uitfiltering van de besmettingen op de aankomstdata
  MSE_min = Inf
  lag_opt = 0
  coef_opt = 1
  for(lag in seq(2,12)){
    opnames = tail(data_besmettingen$Opnames, -lag)
    besm = head(data_besmettingen$log_besm, -lag)
    OLS_scale = as.numeric(t(besm)%*%opnames/t(besm)%*%besm)
    MSE = mean((opnames - besm*OLS_scale)^2)
    if(MSE < MSE_min){
      MSE_min = MSE
      lag_opt = lag
      coef_opt = OLS_scale
    }
  }
  
  if(print_){
    print(paste('optimale vertraging:',lag_opt))
    print(paste('optimale fitering:',coef_opt))
  }
  
  datum_voorsp_aankomst_besm = tail(log_besm$datum, -lag_opt)
  voorsp_aankomst_besm       = head(log_besm$log*coef_opt, -lag_opt)
  df_voorsp_aankomst         = tibble(datum = datum_voorsp_aankomst_besm, besmettingen = voorsp_aankomst_besm)
  
  df_tot = tibble(datum = seq(min(data_besmettingen$datum), max(data_besmettingen$datum) + days(horizon), by = 'day'))
  df_tot = quiet(df_tot %>% left_join(data_besmettingen ) %>% left_join(df_voorsp_aankomst))
  
  #EWMA toevoegen van aankomsten  
  df_tot = df_tot %>% mutate(ewma_opn = ewma(Opnames, 0.2)) 
  
  if(plot_){
    plot(dates, ewma_opn)
    lines(dates, voorsp, type = 'l', col = 'red')
  }
  
  df_tot = df_tot %>% rename(nieuwe_opn = Opnames)%>% mutate( fit = NA, obs = NA, lwr = NA, upr = NA)
  return(df_tot)
  
}

fit_besmettingen_opnames = function(data_besmettingen, horizon = 0,  print_ = F, plot_ = F, lag_opt = 7, coef_ewma = 0.2){
  assertthat::assert_that(horizon <= lag_opt)
  #voeg ewma toe
  data_besmettingen = data_besmettingen %>% mutate(pred_besm = ewma(Besmettingen, coef_ewma))
  
  #fit een optimale vertraging en uitfiltering van de besmettingen op de aankomstdata
  opnames = tail(data_besmettingen$Opnames, -lag_opt)
  besm = head(data_besmettingen$pred_besm, -lag_opt)
  OLS_scale = as.numeric(t(besm)%*%opnames/t(besm)%*%besm)
  
  datum_voorsp_aankomst_besm = data_besmettingen$datum + days(lag_opt)
  voorsp_aankomst_besm       = data_besmettingen$pred_besm*OLS_scale
  df_voorsp_aankomst         = tibble(datum = datum_voorsp_aankomst_besm, besmettingen = voorsp_aankomst_besm) %>% filter(datum <= max(data_besmettingen$datum) + days(horizon))
  
  df_tot = suppressMessages(df_voorsp_aankomst %>% left_join(data_besmettingen))
  
  df_tot = df_tot %>% rename(nieuwe_opn = Opnames)%>% mutate( fit = NA, obs = NA, lwr = NA, upr = NA)
  return(df_tot)
}





