rm(list = ls())
Sys.setlocale("LC_ALL", "English")
source('R/utils.R')
source('R/00_maak_datasets.R')
require(reshape2)
require(tibbletime)
library(xtable)
library(scales)
library(tables)

rep.col<-function(x,n){
  matrix(rep(x,each=n), ncol=n, byrow=TRUE)
}
colorshorizons = c('0' = 'black','1'='orange','2'='cyan','3' = 'blue','5'='purple','7'='green' )
colorsscenarios = c('log'='orange','echt'='cyan','besmettingen' = 'purple')
stylesscenarios = c('log'='solid','echt'='solid','besmettingen' = 'solid')
sizesscenarios = c('log'=1.2,'echt'=1.2,'besmettingen' = 0.6)
ordersscenarios = c('log'=2,'echt'=3,'besmettingen' = 1)
english_names_methods = c('log' = 'Richards\' Curve', 'echt'= 'True Arrivals', 'besmettingen' = 'Reg. Infections')

maak_plots_maxima_meerdere_scenarios = function(summaries,scenarios, horizon,  taal  ='EN', save_plots = F, CI_val = 0.95,
                                                max_date = as.Date(Sys.time())){
  
  #voeg voorspellingen samen
  preds_ic = c()
  preds_kl = c()
  dates = c()
  realisatie_ic = c()
  realisatie_kl = c()
  for(datestring in names(summaries)[as.Date(names(summaries)) <= max_date]){
    dates = c(dates, toString(as.Date(datestring) + days(horizon)))
    preds_date_ic = c()
    preds_date_kl = c()
    for(scenario in scenarios){
      maximum_ic = mean(summaries[[datestring]]$maxima_IC[[scenario]][1+(horizon)*24,])
      ub_maximum_ic = quantile(summaries[[datestring]]$maxima_IC[[scenario]][1+(horizon)*24,], 1-(1-CI_val)/2)
      lb_maximum_ic = quantile(summaries[[datestring]]$maxima_IC[[scenario]][1+(horizon)*24,], 1-(1+CI_val)/2)
      preds_date_ic = c(preds_date_ic, c(maximum_ic, ub_maximum_ic, lb_maximum_ic))
      
      maxima_kl = mean(summaries[[datestring]]$maxima_kl[[scenario]][1+(horizon)*24,])
      ub_maxima_kl = quantile(summaries[[datestring]]$maxima_kl[[scenario]][1+(horizon)*24,], 1-(1-CI_val)/2)
      lb_maxima_kl = quantile(summaries[[datestring]]$maxima_kl[[scenario]][1+(horizon)*24,], 1-(1+CI_val)/2)
      preds_date_kl = c(preds_date_kl, c(maxima_kl, ub_maxima_kl, lb_maxima_kl))
    }
    preds_kl = rbind(preds_kl, preds_date_kl)
    preds_ic = rbind(preds_ic, preds_date_ic)
    realisatie_ic = c(realisatie_ic, summaries[[datestring]]$pred_df_ic %>% filter(datum == datestring) %>% pull('avg_log'))
    realisatie_kl = c(realisatie_kl, summaries[[datestring]]$pred_df_kl %>% filter(datum == datestring) %>% pull('avg_log'))
  }
  colnames(preds_kl) = Reduce(append, sapply(scenarios, function(str){
    c(paste0('voorspelling maximum Kliniek ', str),
      paste0('Bovengrens voorspelling maximum Kliniek ', str),paste0('Ondergrens voorspelling maximum Kliniek ', str))}))
  
  colnames(preds_ic) = Reduce(append, sapply(scenarios, function(str){
    c(paste0('voorspelling maximum IC ', str),paste0('Bovengrens voorspelling maximum IC ', str),
      paste0('Ondergrens voorspelling maximum IC ', str))}))
  
  pred_tbl_ic = as_tibble(preds_ic) %>% add_column(datum = as.Date(dates), .before = 1)
  pred_tbl_kl = as_tibble(preds_kl) %>% add_column(datum = as.Date(dates), .before = 1)
  
  #bereken realisatie
  historische_maximale_bezetting  = list(hist_bezetting_ic = tibble(datum = as.Date(names(summaries)[as.Date(names(summaries)) <= max_date]), bezetting = realisatie_ic),
                                         hist_bezetting_kl = tibble(datum = as.Date(names(summaries)[as.Date(names(summaries)) <= max_date]), bezetting = realisatie_kl))
  maxfn = rollify(max, window = horizon)
  historische_maximale_bezetting$hist_bezetting_ic = historische_maximale_bezetting$hist_bezetting_ic %>% mutate(bezetting = maxfn(bezetting))
  historische_maximale_bezetting$hist_bezetting_kl = historische_maximale_bezetting$hist_bezetting_kl %>% mutate(bezetting = maxfn(bezetting))
  
  daadwerkelijke_maxima_IC = historische_maximale_bezetting$hist_bezetting_ic%>% filter(as.Date(datum) %in% pred_tbl_ic$datum)
  daadwerkelijke_maxima_Kliniek = historische_maximale_bezetting$hist_bezetting_kl %>% filter(as.Date(datum) %in% pred_tbl_kl$datum)
  
  #voeg realisatie bij voorspellingen
  voorspeltabel_maxima_ic = pred_tbl_ic %>% left_join( daadwerkelijke_maxima_IC, by = 'datum')%>% rename(`daadwerkelijk maximum` = bezetting) %>% 
    filter(!is.na(`daadwerkelijk maximum`))
  voorspeltabel_maxima_kl = pred_tbl_kl   %>% left_join( daadwerkelijke_maxima_Kliniek, by = 'datum') %>% rename(`daadwerkelijk maximum` = bezetting) %>% 
    filter(!is.na(`daadwerkelijk maximum`))
  
  #haal fout/coverage resultaten op voor IC
  errors_ic = voorspeltabel_maxima_ic %>% select(starts_with('Voorspelling maximum IC')) - replicate(length(scenarios), voorspeltabel_maxima_ic$`daadwerkelijk maximum`)
  indc_covg_ic = voorspeltabel_maxima_ic %>% select(starts_with('Bovengrens Voorspelling maximum IC')) >= replicate(length(scenarios),voorspeltabel_maxima_ic$`daadwerkelijk maximum`) &
    voorspeltabel_maxima_ic%>% select(starts_with('Ondergrens Voorspelling maximum IC')) <= replicate(length(scenarios),voorspeltabel_maxima_ic$`daadwerkelijk maximum`) 
  
  ic_eval = tibble(
    type = rep(paste0('Max. ',horizon,'d. ah.'), length(scenarios)),
    scenario = as.vector(english_names_methods[scenarios]),
    bias = apply(errors_ic,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_ic,2,function(v){mean(abs(v), na.rm=T)}),
    CR = apply(indc_covg_ic,2,function(v){mean(abs(v), na.rm=T)}))
  
  
  #haal fout/coverage resultaten op voor kliniek
  errors_kl = voorspeltabel_maxima_kl %>% select(starts_with('Voorspelling maximum Kliniek')) - replicate(length(scenarios), voorspeltabel_maxima_kl$`daadwerkelijk maximum`)
  indc_covg_kl = voorspeltabel_maxima_kl %>% select(starts_with('Bovengrens Voorspelling maximum Kliniek')) >= replicate(length(scenarios),voorspeltabel_maxima_kl$`daadwerkelijk maximum`) &
    voorspeltabel_maxima_kl%>% select(starts_with('Ondergrens Voorspelling maximum Kliniek')) <= replicate(length(scenarios),voorspeltabel_maxima_kl$`daadwerkelijk maximum`) 
  
  kl_eval = tibble(
    type = rep(paste0('Max. ',horizon,'d. ah.'), length(scenarios)),
    scenario = as.vector(english_names_methods[scenarios]),
    bias = apply(errors_kl,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_kl,2,function(v){mean(abs(v), na.rm=T)}),
    CR = apply(indc_covg_kl,2,function(v){mean(abs(v), na.rm=T)}))
  
  #voeg samen
  kl_eval_long = pivot_longer(kl_eval , -c(type, scenario)) %>% add_column(dept = 'Ward')
  ic_eval_long = pivot_longer(ic_eval , -c(type, scenario)) %>% add_column(dept = 'ICU')
  tot_eval = full_join(kl_eval_long, ic_eval_long)   %>% mutate(value = round(value,2))
  
  #genereer plots
  names_pred_curves = colnames(voorspeltabel_maxima_kl %>% select(starts_with('Voorspelling maximum')))
  colorsscenarios_kl = colorsscenarios[scenarios]
  styles_kl          = stylesscenarios[scenarios]
  sizes_kl           = sizesscenarios[scenarios]
  names(sizes_kl) = names_pred_curves
  names(styles_kl)   = names_pred_curves
  names(colorsscenarios_kl) = names_pred_curves
  plot_data_kl = melt(voorspeltabel_maxima_kl %>% select(!starts_with('Ondergrens')& !starts_with('Bovengrens') & !contains('daadwerkelijk')) %>% drop_na(), id.vars = 'datum')
   
    
  plot_voorspellingen_kliniek = ggplot() + geom_line(data = voorspeltabel_maxima_kl, aes(x = datum, y = `daadwerkelijk maximum`), size = 1.2, color = 'red')+
    ylim(0, NA) +
    xlab("Date") +
    ylab(paste0("highest occupancy at ward over the last ",horizon, ' days'))   +
    geom_line(data = plot_data_kl, aes(x = datum, y = value, colour = variable, linetype = variable, size = variable))+
    scale_colour_manual(values = colorsscenarios_kl)+
    scale_linetype_manual(values =styles_kl)+
    scale_size_manual(values =  sizes_kl)+
    theme_bw()+theme(legend.position = 'none')+ylim(c(0, max(voorspeltabel_maxima_kl %>% select(-datum))+2))+scale_y_continuous(breaks = integer_breaks())
  
  
  names_pred_curves = colnames(voorspeltabel_maxima_ic %>% select(starts_with('Voorspelling maximum')))
  colorsscenarios_ic = colorsscenarios[scenarios]
  names(colorsscenarios_ic) = names_pred_curves
  styles_ic          = stylesscenarios[scenarios]
  names(styles_ic)   = names_pred_curves
  sizes_ic           = sizesscenarios[scenarios]
  names(sizes_ic)    = names_pred_curves
  
  plot_data_ic = melt(voorspeltabel_maxima_ic %>% select(!starts_with('Ondergrens')& !starts_with('Bovengrens') & !contains('daadwerkelijk')) %>% drop_na(), id.vars = 'datum')
  plot_voorspellingen_ic = ggplot() + geom_line(data = voorspeltabel_maxima_ic, aes(x = datum, y = `daadwerkelijk maximum`), size = 1.2, color = 'red')+
    ylim(0, NA) +
    xlab("Date") +
    ylab(paste0("highest occupancy at ICU over the last ",horizon, ' days'))   +
    geom_line(data = plot_data_ic, aes(x = datum, y = value, colour = variable, linetype = variable, size = variable))+
    scale_colour_manual(values = colorsscenarios_ic)+
    scale_linetype_manual(values =styles_ic)+
    scale_size_manual(values =  sizes_ic)+
    theme_bw()+theme(legend.position = 'none')+ylim(c(0, max(voorspeltabel_maxima_ic %>% select(-datum))+2))+scale_y_continuous(breaks = integer_breaks())
  
  print(ggplotly(plot_voorspellingen_kliniek))
  print(ggplotly(plot_voorspellingen_ic))
  
  if(save_plots){
    ggsave(paste0('R/data analyse/data/maxima_plot_ic_', ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),horizon_plot, '.pdf'),
           plot_voorspellingen_ic, device = cairo_pdf, width = 125*3/4, height = 125*3/4, units = 'mm')
    ggsave(paste0('R/data analyse/data/maxima_plot_kl_', ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),horizon_plot, '.pdf'),
           plot_voorspellingen_kliniek, device = cairo_pdf, width = 125*3/4, height = 125*3/4, units = 'mm')
  }
  
  return(mget(c('tot_eval', 'voorspeltabel_maxima_ic', 'voorspeltabel_maxima_kl')))
}

maak_plots_voorspellingen_meerdere_scenarios = function(summaries, scenarios,horizons , horizon_plot,  save_plots = F, 
                                                        CI_plot = F, realisatie_erbij = T,max_date = as.Date(Sys.time())){
  #haal voorspellingen op + boven/ondergrens
  copy_scenarios = scenarios
  horizons = c(0, horizons) #voeg horizon 0 toe om huidige bezetting uit te rekenen
  for(i in seq(length(horizons))){
    horizon = horizons[i]
    if(horizon == 0){
      scenarios = scenarios[1]
    }else{
      scenarios = copy_scenarios
    }
    preds_ic = c()
    lb_ic = c()
    ub_ic = c()
    preds_kl = c()
    lb_kl = c()
    ub_kl = c()
    dates = c()
    for(datestring in names(summaries)[as.Date(names(summaries)) <= max_date]){
      
      df_ic = summaries[[datestring]]$pred_df_ic %>% select(datum, sapply(scenarios, function(str){paste0('avg_', str)}))
      row_ic = as.numeric(df_ic[df_ic$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
      preds_ic = rbind(preds_ic, row_ic)
      
      df_lb_ic = summaries[[datestring]]$pred_df_ic %>% select(datum, sapply(scenarios, function(str){paste0('lbCI95%_', str)}))
      row_lb_ic = as.numeric(df_lb_ic[df_lb_ic$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
      lb_ic = rbind(lb_ic, row_lb_ic)
      
      df_ub_ic = summaries[[datestring]]$pred_df_ic %>% select(datum, sapply(scenarios, function(str){paste0('ubCI95%_', str)}))
      row_ub_ic = as.numeric(df_ub_ic[df_ub_ic$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
      ub_ic = rbind(ub_ic, row_ub_ic)
      
      df_kl = summaries[[datestring]]$pred_df_kl %>% select(datum, sapply(scenarios, function(str){paste0('avg_', str)}))
      row_kl = as.numeric(df_kl[df_kl$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
      preds_kl = rbind(preds_kl, row_kl)
      
      df_lb_kl = summaries[[datestring]]$pred_df_kl %>% select(datum, sapply(scenarios, function(str){paste0('lbCI95%_', str)}))
      row_lb_kl = as.numeric(df_lb_kl[df_lb_kl$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
      lb_kl = rbind(lb_kl, row_lb_kl)
      
      df_ub_kl = summaries[[datestring]]$pred_df_kl %>% select(datum, sapply(scenarios, function(str){paste0('ubCI95%_', str)}))
      row_ub_kl = as.numeric(df_ub_kl[df_ub_kl$datum == as.Date(datestring) + days(horizon), ] %>% select(-datum))
      ub_kl = rbind(ub_kl, row_ub_kl)
      
      dates = c(dates, toString(as.Date(datestring) + days(horizon)))
    }
    
    colnames(preds_ic) = sapply(colnames(df_ic %>% select(-datum)), function(str){paste('Predicted occupancy ICU ',str ,' (', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    colnames(preds_kl) = sapply(colnames(df_kl %>% select(-datum)), function(str){paste('Predicted occupancy Ward ',str ,'(', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    
    colnames(lb_ic) = sapply(colnames(df_ic %>% select(-datum)), function(str){paste('Predicted occupancy ICU ',str ,' (', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    colnames(lb_kl) = sapply(colnames(df_kl %>% select(-datum)), function(str){paste('Predicted occupancy Ward ',str ,'(', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    
    colnames(ub_ic) = sapply(colnames(df_ic %>% select(-datum)), function(str){paste('Predicted occupancy ICU ',str ,' (', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    colnames(ub_kl) = sapply(colnames(df_kl %>% select(-datum)), function(str){paste('Predicted occupancy Ward ',str ,'(', horizon, ifelse(horizon>1,' days ahead)', ' day ahead)'))})
    
    
    df_ic = as_tibble(preds_ic) %>% add_column(datum = as.Date(dates), .before = 1)
    ub_ic = as_tibble(ub_ic) %>% add_column(datum = as.Date(dates), .before = 1)
    lb_ic = as_tibble(lb_ic) %>% add_column(datum = as.Date(dates), .before = 1)
    df_kl = as_tibble(preds_kl) %>% add_column(datum = as.Date(dates), .before = 1)
    ub_kl = as_tibble(ub_kl) %>% add_column(datum = as.Date(dates), .before = 1)
    lb_kl = as_tibble(lb_kl) %>% add_column(datum = as.Date(dates), .before = 1)
    
    if(i ==1){
      pred_tbl_ic = df_ic
      ub_tbl_ic = ub_ic
      lb_tbl_ic = lb_ic
      pred_tbl_kl = df_kl
      ub_tbl_kl = ub_kl
      lb_tbl_kl = lb_kl
    }else{
      pred_tbl_ic = pred_tbl_ic %>% left_join(df_ic, by = 'datum') 
      lb_tbl_ic = lb_tbl_ic %>% left_join(lb_ic, by = 'datum')
      ub_tbl_ic = ub_tbl_ic %>% left_join(ub_ic, by = 'datum')
      pred_tbl_kl = pred_tbl_kl %>% left_join(df_kl, by = 'datum')
      lb_tbl_kl = lb_tbl_kl %>% left_join(lb_kl, by = 'datum')
      ub_tbl_kl = ub_tbl_kl %>% left_join(ub_kl, by = 'datum')
    }
  }
  horizons = horizons[-1] # horizon 0 weer weghalen
  
  voorspeltabel_benutting_ic = pred_tbl_ic %>%  rename(`Realised occupancy at ICU` = !!paste('Predicted occupancy ICU ', scenarios[1],' ( 0  day ahead)'))%>% rename(date = datum)
  voorspeltabel_benutting_kl = pred_tbl_kl %>% rename(`Realised occupancy at Ward` = !!paste('Predicted occupancy Ward ', scenarios[1],'( 0  day ahead)'))%>% rename(date = datum)
  
  pred_tbl_ic = pred_tbl_ic %>% rename(date = datum)
  lb_tbl_ic = lb_tbl_ic %>% rename(date = datum)
  ub_tbl_ic = ub_tbl_ic %>% rename(date = datum)
  pred_tbl_kl = pred_tbl_kl %>% rename(date = datum)
  lb_tbl_kl = lb_tbl_kl %>% rename(date = datum)
  ub_tbl_kl = ub_tbl_kl%>% rename(date = datum)
  
  #plot data in voorspeltabel kliniek
  colnames_plot_kl = as.vector(sapply(scenarios, function(str){paste('Predicted occupancy Ward ',str ,'(', horizon_plot, ifelse(horizon_plot>1,' days ahead)', ' day ahead)'))}))
  if(realisatie_erbij){
    colnames_plot_kl = c('Realised occupancy at Ward', colnames_plot_kl)
  }
  df_pred_plot_kl <- melt(voorspeltabel_benutting_kl %>% select(contains( c('date',colnames_plot_kl))) %>% drop_na(),  id.vars = 'date')
  df_ub_plot_kl <- melt(ub_tbl_kl%>% select(contains(c("date",colnames_plot_kl ))) ,  id.vars = 'date') %>% rename( Bovengrens = value)
  df_lb_plot_kl <- melt(lb_tbl_kl%>% select(contains(c("date",colnames_plot_kl ))) ,  id.vars = 'date') %>% rename(Ondergrens = value)
  df_plot_kl = df_pred_plot_kl %>% left_join(df_ub_plot_kl, by = c('date', 'variable')) %>% left_join(df_lb_plot_kl, by = c('date', 'variable')) %>% rename(scenario = variable)
  
  
  #plot data in voorspeltabel IC
  colnames_plot_ic = as.vector(sapply(scenarios, function(str){paste('Predicted occupancy ICU ',str ,' (', horizon_plot, ifelse(horizon_plot>1,' days ahead)', ' day ahead)'))}))
  if(realisatie_erbij){
    colnames_plot_ic = c('Realised occupancy at ICU', colnames_plot_ic)
  }
  
  df_pred_plot_ic <- melt(voorspeltabel_benutting_ic%>% select(contains(c("date",colnames_plot_ic )))%>%drop_na() ,  id.vars = 'date')
  df_ub_plot_ic <- melt(ub_tbl_ic %>% select(contains(c("date",colnames_plot_ic ))),  id.vars = 'date') %>% rename( Bovengrens = value)
  df_lb_plot_ic <- melt(lb_tbl_ic %>% select(contains(c("date",colnames_plot_ic ))) ,  id.vars = 'date') %>% rename(Ondergrens = value)
  df_plot_ic = df_pred_plot_ic %>% left_join(df_ub_plot_ic, by = c('date', 'variable')) %>%
    left_join(df_lb_plot_ic, by = c('date', 'variable'))%>% rename(scenario = variable)
  
  if(realisatie_erbij){
    colors_plot = c('red', colorsscenarios[scenarios])
    styles_plot = c('solid', stylesscenarios[scenarios])
    sizes_plot = c(1.2, sizesscenarios[scenarios])
  }else{
    colors_plot = colorsscenarios[scenarios]
    styles_plot = stylesscenarios[scenarios]
    sizes_plot = sizesscenarios[scenarios]
  }
  names(colors_plot) = colnames_plot_kl
  names(styles_plot) = colnames_plot_kl
  names(sizes_plot) = colnames_plot_kl
  
  # df_plot_kl$style = sapply(df_plot_kl$scenario, function(entry){styles_scenarios[sapply(names(styles_scenarios), function(string){grepl(string, entry)})]})
  # 
  
  plot_voorspellingen_kliniek = ggplot(df_plot_kl, aes(date,value)) + geom_line(aes(colour = scenario, linetype = scenario, size = scenario))+
    ylab('occupancy at Ward')+ theme_bw()+
    theme(legend.position = 'none')+
    scale_colour_manual(values =colors_plot)+
    scale_linetype_manual(values = styles_plot)+
    scale_size_manual(values = sizes_plot)+ylim(c(0, max(df_plot_kl$value)+2))+
    scale_y_continuous(breaks = integer_breaks())
  # ggtitle(paste0(horizon_plot, ' days ahead forecasts'))
  
  
  names(colors_plot) = colnames_plot_ic
  names(styles_plot) = colnames_plot_ic
  names(sizes_plot) = colnames_plot_ic
  
  plot_voorspellingen_ic = ggplot(df_plot_ic, aes(date,value)) + geom_line(aes(colour = scenario, linetype = scenario, size = scenario))+
    ylab('occupancy at ICU')+ theme_bw()+
    theme(legend.position = 'none')+
    scale_colour_manual(values = colors_plot)+
    scale_linetype_manual(values = styles_plot)+
    scale_size_manual(values = sizes_plot)+ylim(c(0, max(df_plot_ic$value)+2))+
    scale_y_continuous(breaks = integer_breaks())
  # ggtitle(paste0(horizon_plot, ' days ahead forecasts'))
  
  print(ggplotly(plot_voorspellingen_kliniek))
  print(ggplotly(plot_voorspellingen_ic))
  
  if('echt' %in% scenarios){horizon_plot = paste0(horizon_plot, '_besmettingen_vs_echt')}
  if(save_plots){
    ggsave(paste0('R/data analyse/data/EWplot_voorspellingen_ic_',ifelse(CI_plot, 'CI_',''), ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),horizon_plot, '.pdf'),plot_voorspellingen_ic, device = cairo_pdf, width = 125*3/4, height = 125*3/4, units = 'mm')
    ggsave(paste0('R/data analyse/data/EWplot_voorspellingen_kl_',ifelse(CI_plot, 'CI_',''), ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),horizon_plot, '.pdf'),plot_voorspellingen_kliniek, device = cairo_pdf, width = 125*3/4, height = 125*3/4, units = 'mm')
  }
  
  #haal fout/coverage resultaten op voor IC
  errors_ic = voorspeltabel_benutting_ic[-seq(max(horizons)),] %>% select(-date, -`Realised occupancy at ICU`) - voorspeltabel_benutting_ic[-seq(max(horizons)),rep("Realised occupancy at ICU", ncol(voorspeltabel_benutting_ic) - 2)]
  coverage_ic = lb_tbl_ic[,-c(1,2)]  <= replicate( length(scenarios)*length(horizons), voorspeltabel_benutting_ic %>% pull('Realised occupancy at ICU')) & 
    ub_tbl_ic[,-c(1,2)] >= replicate( length(scenarios)*length(horizons), voorspeltabel_benutting_ic %>% pull('Realised occupancy at ICU'))
  
  ic_eval = tibble(
    type = rep(sapply(horizons, function(days){paste0(days, ifelse(days == 1, ' day ah.', ' days ah.'))}), each = length(scenarios)),
    scenario = rep(as.vector(english_names_methods[scenarios]),  length(horizons)),
    bias = apply(errors_ic,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_ic,2,function(v){mean(abs(v), na.rm=T)}),
    CR  = apply(coverage_ic, 2, function(v){mean(v, na.rm = T)}))
  
  #haal fout/coverage resultaten op voor Kliniek
  errors_kl = voorspeltabel_benutting_kl[-seq(max(horizons)),] %>% select(-date, -`Realised occupancy at Ward`) - voorspeltabel_benutting_kl[-seq(max(horizons)),rep("Realised occupancy at Ward", ncol(voorspeltabel_benutting_kl) - 2)]
  coverage_kl = lb_tbl_kl[,-c(1,2)]  <= replicate( length(scenarios)*length(horizons), voorspeltabel_benutting_kl %>% pull('Realised occupancy at Ward')) & 
    ub_tbl_kl[,-c(1,2)] >= replicate( length(scenarios)*length(horizons), voorspeltabel_benutting_kl %>% pull('Realised occupancy at Ward'))
  
  kl_eval = tibble(
    type = rep(sapply(horizons, function(days){paste0(days, ifelse(days == 1, ' day ah.', ' days ah.'))}), each = length(scenarios)),
    scenario = rep(as.vector(english_names_methods[scenarios]),  length(horizons)),
    bias = apply(errors_kl,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_kl,2,function(v){mean(abs(v), na.rm=T)}),
    CR   = apply(coverage_kl, 2, function(v){mean(v, na.rm = T)})
  )
  
  #make the table long (for writing to  LaTeX)
  ic_eval_long = pivot_longer(ic_eval, -c(type, scenario)) %>% add_column(dept ='ICU', .before = 1)
  kl_eval_long = pivot_longer(kl_eval, -c(type, scenario))%>% add_column(dept ='Ward', .before = 1)
  tot_eval = full_join(ic_eval_long, kl_eval_long) %>% mutate(value = round(value,2))
  
  #make weekly average predictions 
  ma_pred_wk = rollify(mean, window = 7) # get weekly moving average fn
  voorspelling_ma_ic = ma_pred_wk(voorspeltabel_benutting_ic$`Realised occupancy at ICU`)
  voorspelling_ma_kl = ma_pred_wk(voorspeltabel_benutting_kl$`Realised occupancy at Ward`)
  
  #make weekly average predictions 
  ma_pred_wk = rollify(mean, window = 7) # get weekly moving average fn
  voorspelling_ma_ic = ma_pred_wk(voorspeltabel_benutting_ic$`Realised occupancy at ICU`)
  voorspelling_ma_kl = ma_pred_wk(voorspeltabel_benutting_kl$`Realised occupancy at Ward`)
  
  
  errors_ic_ma =  sapply(horizons, function(horizon){c(rep(NA, horizon), head(voorspelling_ma_ic, length(voorspelling_ma_ic)- horizon))})[-seq(max(horizons)),] - voorspeltabel_benutting_ic[-seq(max(horizons)),rep("Realised occupancy at ICU", length(horizons))]
  ic_eval_ma = tibble(
    type = sapply(horizons, function(days){paste0(days, ifelse(days == 1, ' day ah.', ' days ah.'))}),
    bias = apply(errors_ic_ma,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_ic_ma,2,function(v){mean(abs(v), na.rm=T)}))
  
  errors_kl_ma =  sapply(horizons, function(horizon){c(rep(NA, horizon), head(voorspelling_ma_kl, length(voorspelling_ma_kl)- horizon))})[-seq(max(horizons)),] - voorspeltabel_benutting_kl[-seq(max(horizons)),rep("Realised occupancy at Ward", length(horizons))]
  kl_eval_ma = tibble(
    type = sapply(horizons, function(days){paste0(days, ifelse(days == 1, ' day ah.', ' days ah.'))}),
    bias = apply(errors_kl_ma,2,function(v){mean((v), na.rm=T)}),
    MAE = apply(errors_kl_ma,2,function(v){mean(abs(v), na.rm=T)}))
  
  #make the table long (for writing to  LaTeX)
  horizons_int = rep(horizons, each = ncol(ic_eval)-1)
  ic_eval_long_ma = pivot_longer(ic_eval_ma, -type) 
  kl_eval_long_ma = pivot_longer(kl_eval_ma, -type)
  departments   = rep(c("ICU", "Ward"), each = nrow(ic_eval_long_ma))
  tot_eval_ma = full_join(ic_eval_long_ma, kl_eval_long_ma) %>% add_column(dept = departments, .before = 1) %>% mutate(value = round(value,2))
  
  return(mget(c('tot_eval', 'tot_eval_ma', 'voorspeltabel_benutting_kl', 'voorspeltabel_benutting_ic')))
}



if(sys.nframe() == 0){
  # hier stel je param in voor plots, eerst moet "haal_voorspellingen_op.R" met deze parameters gedraaid worden en er moet dan een .rds file weggeschreven zijn met deze parameter instellingen
  ziekenhuis = 'Haga'
  scenarios = c('log','besmettingen')#
  horizons <- c(1,2,3,5,7)
  bool_save = T
  horizon_CI_plot = 3
  horizon_plot = 3
  app = '_4'
  horizon_max = 3
  max_date = as.Date('2021-01-31')
  aantal_dagen = 7
  realisatie_erbij = T
  overplaatsingen_er_uit = T
  load(paste0('R/data analyse/data/rolling_preds_',ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),aantal_dagen,app,'.Rds'))
  summaries = result$summaries
  
  # # #maak plots en haal foutentabel op voor sim voorspeller
  res_dag = maak_plots_voorspellingen_meerdere_scenarios(summaries,  scenarios, horizons, horizon_plot , save_plots = bool_save, CI_plot = F, realisatie_erbij = T ,
                                                         max_date = max_date)
  res_max = maak_plots_maxima_meerdere_scenarios(summaries,scenarios, horizon_max,  taal = 'EN' , save_plots = bool_save, max_date = max_date)


  #maak plots en haal foutentabel op voor sim voorspeller
  scenarios = c('echt','besmettingen')
  res_dag = maak_plots_voorspellingen_meerdere_scenarios(summaries,  scenarios, horizons, horizon_plot , save_plots = bool_save, CI_plot = F, realisatie_erbij = F ,
                                                         max_date = max_date)

  # #maak plots en haal foutentabel op voor sim voorspeller
  scenarios = c('besmettingen','log', 'echt')
  res_dag = maak_plots_voorspellingen_meerdere_scenarios(summaries,  scenarios, horizons, horizon_plot , save_plots = F, CI_plot = F, realisatie_erbij = realisatie_erbij , max_date = max_date)
  res_max = maak_plots_maxima_meerdere_scenarios(summaries,scenarios, horizon_max,  taal = 'EN' , save_plots = F, max_date = max_date)
  res_max_5 = maak_plots_maxima_meerdere_scenarios(summaries,scenarios, 5,  taal = 'EN' , save_plots = F, max_date = max_date)
  res_max_7 = maak_plots_maxima_meerdere_scenarios(summaries,scenarios, 7,  taal = 'EN' , save_plots = F, max_date = max_date)

  res_forecaster = full_join(full_join(full_join(res_dag$tot_eval, res_max$tot_eval), res_max_5$tot_eval), res_max_7$tot_eval) %>% rename(method = scenario)
  results_table <- tabular(RowFactor(method, name = 'method')*RowFactor(type, name = 'type') ~
                             Heading()*Factor(dept,
                                              levelnames =  c("ICU",
                                                              "Ward" ))*
                             Heading()*Factor(name,
                                              levelnames = c('bias', 'CR', 'MAE' ))*Heading()*(value)*Heading()*(mean)*
                             DropEmpty(empty=""),
                           data = res_forecaster)
  # results_table =  results_table[,c(11, 10, 12, 8, 7, 9, 5, 4, 6, 2, 1, 3, 17,16,18,14,13,15)]
  incr = length(horizons)+3
  results_table =  results_table[c(seq(incr)+incr, seq(incr), seq(incr)+2*incr),c(c(2,1,3)+3, c(2,1,3))]

  if(bool_save){write(tables::latex.tabular(results_table)[[1]], file = paste0('R/data analyse/data/foutentabel_',ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen', '_overplaatsingen'),'.tex'))}
}
