rm(list=ls())
library(tibbletime)
library(tibble)
library(dplyr)
library(tidyr)
source("R/utils.R") # basisfuncties + packages inladen
source('R/functies.R')
source('R/functies_condprobs.R')
source("R/00_maak_datasets.R", encoding = "UTF-8")
source('R/data analyse/maak_voorspellingen_pred.R')
source('R/data analyse/maak_voorspellingen_sim.R')
get_val_horizon =  function(DESres, horizon){
  rij_bezetting_horizon = DESres[DESres$date >= DESres$date[1] + days(horizon), ][1,]
  return(rij_bezetting_horizon)
}


if(sys.nframe() == 0){
  seed = as.numeric(Sys.time())
  random_seed = .Random.seed
  set.seed(seed) # seed is nu in ieder geval altijd anders (afhankelijk van de huidige datumtijd incl seconden)
  # tic()
  # voorspel rollend 2 weken vooruit op basis van de data beschikbaar op dat moment 
  # Kies ziekenhuis
  scenarios = c('log','besmettingen', 'echt')
  ziekenhuis <- "Haga"
  aantal_dagen <- 7 #maximale horizon
  overplaatsingen_er_uit = T
  sim =T
  
  naam_voluit = ifelse(ziekenhuis == 'etz', 'Elisabeth TweeSteden Ziekenhuis',
                ifelse(ziekenhuis == 'rs', 'Rijnstate', 'Ziekenhuis Onbekend'))
  
  increment = 1 #days
  
  # laad data in voor ziekenhuis
  source('R/00_maak_datasets.R')
  list2env(maak_datasets(ziekenhuis), envir = .GlobalEnv)
  if(overplaatsingen_er_uit){
    res = haal_paden_in_uitplaatsingen_er_uit(Kliniekset, ICset)
    Kliniekset = res[[1]]
    ICset = res[[2]]
    arrival_dates = Kliniekset %>% filter(Herkomst %in% c("Eigen woonomgeving"))
    
    Aankomst_kliniek = Kliniekset %>% filter(Herkomst != 'Van IC'&
                                               Herkomst != "Uit eigen ziekenhuis"&
                                               Herkomst != "Ander ziekenhuis") %>%
      select(startdatum) %>% 
      mutate(aantal = 1) %>%
      group_by(startdatum) %>% 
      summarise(aantal_kliniek = sum(aantal), .groups = 'drop') %>%
      ungroup()
    
    Aankomst_IC = ICset %>% filter(Herkomst != 'Van Kliniek'&
                                     Herkomst != "Uit eigen ziekenhuis"&
                                     Herkomst != "Ander ziekenhuis") %>%
      select(startdatum) %>% 
      mutate(aantal = 1) %>%
      group_by(startdatum) %>% 
      summarise(aantal_ic = sum(aantal), .groups = 'drop') %>%
      ungroup()
    
    #merging the arrival information
    aankomst_totaal = tibble(startdatum = seq(min(c(Aankomst_IC$startdatum, Aankomst_kliniek$startdatum)), as.Date(datetime_pred), by = 'days'))
    aankomst_totaal = left_join(aankomst_totaal, Aankomst_IC, by = 'startdatum') %>% left_join(Aankomst_kliniek, by = 'startdatum')%>%
    replace_na(replace = list(aantal_ic = 0, aantal_kliniek = 0)) %>% rename(datum = startdatum)
    
    #haal extreem vroege aankomsten weg
    aankomst_totaal = aankomst_totaal[aankomst_totaal$datum >= as.Date('2020-03-03'),]
    
    aankomst_totaal = aankomst_totaal %>%
      add_column(aantal_cumulatief_ic = cumsum(aankomst_totaal$aantal_ic), .before = 3)
    
    aankomst_totaal = aankomst_totaal %>%
      add_column(aantal_cumulatief_kliniek = cumsum(aankomst_totaal$aantal_kliniek), .after  = 4)
    
    df_nieuwe_opnames = aankomst_totaal %>%
      add_column(aantal = aankomst_totaal$aantal_ic + aankomst_totaal$aantal_kliniek,
                 aantal_cumulatief = aankomst_totaal$aantal_cumulatief_ic +aankomst_totaal$aantal_cumulatief_kliniek, 
                 .before = 2)%>% 
      mutate(
        datum = date(datum),
        num = 1:nrow(.)
      )
  }
  
  # verzamel aankomsttijden per patiënt-type 
  aankomsten_per_type = list()
  aankomsten_per_type[['KL_IC']] = Kliniekset %>% filter(vertrek_type == "Kliniek - vertrek naar IC") %>%
                                                  filter(!(Herkomst %in% c("Van IC", 'Ander ziekenhuis' , 'Ziekenhuis buitenland')))%>%
                                                  arrange(startdatumtijd) %>% select(startdatumtijd)
  aankomsten_per_type[['KL_rest']] = Kliniekset %>% filter(vertrek_type != "Kliniek - vertrek naar IC") %>%
                                                  filter(!(Herkomst %in% c("Van IC", 'Ander ziekenhuis' , 'Ziekenhuis buitenland')))%>%
                                                  arrange(startdatumtijd) %>% select(startdatumtijd)
  aankomsten_per_type[['IC_KL']] = ICset %>% filter(vertrek_type == "IC - vertrek naar Kliniek") %>%
                                                  filter(!(Herkomst %in% c("Van Kliniek", 'Ander ziekenhuis' , 'Ziekenhuis buitenland')))%>%
                                                  arrange(startdatumtijd) %>% select(startdatumtijd)
  aankomsten_per_type[['IC_rest']] = ICset %>% filter(vertrek_type != "IC - vertrek naar Kliniek") %>%
                                                    filter(!(Herkomst %in% c("Van Kliniek", 'Ander ziekenhuis' , 'Ziekenhuis buitenland')))%>%
                                                    arrange(startdatumtijd) %>% select(startdatumtijd)
  save(aankomsten_per_type, file = 'predictions/aankomsten_ptype_Haga.rds')
  
  
  start_date = as.Date('2020-09-01')
  end_datetime = as.Date(datetime_pred) + hours(10)

  # end_date = min(as.Date(datetime_pred), as.Date('2020-07-20')) #minimum van einddatum eerste golf en datum opnamebestand
  df_nieuwe_opnames = df_nieuwe_opnames[df_nieuwe_opnames$datum <= end_datetime,]
  Kliniekset        = maak_partiele_set(Kliniekset, end_datetime, name_dept = 'Kliniek', name_other_dept = 'IC') 
  ICset             = maak_partiele_set(ICset, end_datetime, name_dept = 'IC', name_other_dept = 'Kliniek')
  Kliniekset$BehandelBeperking = 'Geen behandelbeperking'
  ICset$BehandelBeperking = 'Geen behandelbeperking'
  
  date = start_date
  summaries = list()
  dates = c()
  while(date<= as.Date(end_datetime)){
    print(date)
    dates                       = c(dates, toString(date + days(aantal_dagen)))
    datetime                    = date + hours(10)
    
    df_nieuwe_opnames_partial   = df_nieuwe_opnames[df_nieuwe_opnames$datum <= datetime,]
    Kliniekset_partial          = maak_partiele_set(Kliniekset, datetime, name_dept = 'Kliniek', name_other_dept = 'IC')
    ICset_partial               = maak_partiele_set(ICset, datetime, name_dept = 'IC', name_other_dept = 'Kliniek')
   
    summaries[[toString(date)]] = maak_voorspellingen_sim(Kliniekset_partial, ICset_partial , df_nieuwe_opnames_partial , ziekenhuis, aantal_dagen,
                                                          scenarios, voorspeldatumtijd = datetime, fit_vanaf_golf_twee = T)
 
    date = date + days(increment)
  }
  
  #sla file met voorspellingen op onder nieuwe naam
  app ='_1'
  num = 0
  name = paste0('R/data analyse/data/rolling_preds_',ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),aantal_dagen,ifelse(sim, '', '_det'),app,'.Rds')
  while(file.exists(name)){
    num = num + 1
    app = paste0('_',num)
    name = paste0('R/data analyse/data/rolling_preds_',ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),aantal_dagen,ifelse(sim, '', '_det'),app,'.Rds')
  }
  result = list(summaries = summaries, list(seed = seed, random_seed = random_seed))
  save(result, file = name)
}

# rm(list=ls())
# library(tibbletime)
# library(tibble)
# library(dplyr)
# library(tidyr)
# source("R/utils.R") # basisfuncties + packages inladen
# source('R/functies.R')
# source('R/functies_condprobs.R')
# source("R/00_maak_datasets.R", encoding = "UTF-8")
# source('R/data analyse/maak_voorspellingen_pred.R')
# source('R/data analyse/maak_voorspellingen_sim.R')
# get_val_horizon =  function(DESres, horizon){
#   rij_bezetting_horizon = DESres[DESres$date >= DESres$date[1] + days(horizon), ][1,]
#   return(rij_bezetting_horizon)
# }
# 
# 
# if(sys.nframe() == 0){
#   set.seed(as.numeric(Sys.time())) # seed is nu in ieder geval altijd anders (afhankelijk van de huidige datumtijd incl seconden)
#   # tic()
#   # voorspel rollend 2 weken vooruit op basis van de data beschikbaar op dat moment 
#   # Kies ziekenhuis
#   scenarios = c('log','besmettingen', 'echt')
#   ziekenhuis <- "Haga"
#   aantal_dagen <- 7 #maximale horizon
#   overplaatsingen_er_uit = T
#   sim =T
#   
#   naam_voluit = ifelse(ziekenhuis == 'etz', 'Elisabeth TweeSteden Ziekenhuis',
#                        ifelse(ziekenhuis == 'rs', 'Rijnstate', 'Ziekenhuis Onbekend'))
#   
#   increment = 1 #days
#   
#   # laad data in voor ziekenhuis
#   list2env(maak_datasets(ziekenhuis), envir = .GlobalEnv)
#   start_date = as.Date('2020-09-01') 
#   end_datetime = as.Date(datetime_pred) + hours(10) 
#   
#   # end_date = min(as.Date(datetime_pred), as.Date('2020-07-20')) #minimum van einddatum eerste golf en datum opnamebestand
#   df_nieuwe_opnames = df_nieuwe_opnames[df_nieuwe_opnames$datum <= end_datetime,]
#   Kliniekset        = maak_partiele_set(Kliniekset, end_datetime, name_dept = 'Kliniek', name_other_dept = 'IC') 
#   ICset             = maak_partiele_set(ICset, end_datetime, name_dept = 'IC', name_other_dept = 'Kliniek')
#   Kliniekset$BehandelBeperking = 'Geen behandelbeperking'
#   ICset$BehandelBeperking = 'Geen behandelbeperking'
#   
#   date = start_date
#   summaries = list()
#   dates = c()
#   while(date<= as.Date(end_datetime)){
#     print(date)
#     dates                       = c(dates, toString(date + days(aantal_dagen)))
#     datetime                    = date + hours(10)
#     
#     df_nieuwe_opnames_partial   = df_nieuwe_opnames[df_nieuwe_opnames$datum <= datetime,]
#     Kliniekset_partial          = maak_partiele_set(Kliniekset, datetime, name_dept = 'Kliniek', name_other_dept = 'IC')
#     ICset_partial               = maak_partiele_set(ICset, datetime, name_dept = 'IC', name_other_dept = 'Kliniek')
#     
#     summaries[[toString(date)]] = maak_voorspellingen_sim(Kliniekset_partial, ICset_partial , df_nieuwe_opnames_partial , ziekenhuis, aantal_dagen,
#                                                           scenarios, datetime, overplaatsingen_er_uit = overplaatsingen_er_uit, fit_vanaf_golf_twee = T)
#     
#     date = date + days(increment)
#   }
#   
#   #sla file met voorspellingen op onder nieuwe naam
#   app ='_1'
#   num = 0
#   name = paste0('R/data analyse/data/rolling_preds_',ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),aantal_dagen,ifelse(sim, '', '_det'),app,'.Rds')
#   while(file.exists(name)){
#     num = num + 1
#     app = paste0('_',num)
#     name = paste0('R/data analyse/data/rolling_preds_',ziekenhuis,ifelse(overplaatsingen_er_uit,'_geen_overplaatsingen_hor=', '_overplaatsingen_hor='),aantal_dagen,ifelse(sim, '', '_det'),app,'.Rds')
#   }
#   save(summaries, file = name)
# }



