

maak_voorspellingen_sim = function(Kliniekset, ICset, df_nieuwe_opnames, ziekenhuis, aantal_dagen, scenarios, voorspeldatumtijd, fit_vanaf_golf_twee = T,
                                   overplaatsingen_er_uit=F){
  
  laatste_datumtijd_opnamebestand <- voorspeldatumtijd
  laatste_datum_opnamebestand <- as.Date(voorspeldatumtijd)
  if(fit_vanaf_golf_twee){
     start_tweede_golf = as.Date('2020-08-07') #start datum 2e golf (alle data daarvoor wordt NIET gebruikt voor schatten aankomstproces Richard's curve aangezien de dynamiek nu anders is)
  }else{
    start_tweede_golf = as.Date('2000-01-01') #erg vroege datum, geen data er uit
    
  }
 
  
  # laad functies om aankomsten te voorspellen, evalueer op nieuwe opnames
  source("R/01_voorspel_instroom.R", encoding = "UTF-8")
  voorspelde_opnames = voorspel_instroom(df_nieuwe_opnames, aantal_dagen = aantal_dagen, scenarios, voorspeldatum = laatste_datum_opnamebestand, exclude_until = start_tweede_golf)
  
  # laad functies om aankomsten te splitsen per afdeling, evalueer op voorspelde en geobserveerde opnames
  source("R/02_voorspel_instroom_ic_kl.R", encoding = "UTF-8")
  list2env(voorspel_instroom_ic_kl(voorspelde_opnames, df_nieuwe_opnames, ziekenhuis, voorspeldatum = laatste_datum_opnamebestand), envir = environment())
  
  # laad functies survival analyse op ligduren, evalueer op ICset en Kliniekset
  source("R/03_survival_analyse.R", encoding = "UTF-8")
  list2env( survival_analyse(ziekenhuis, ICset, Kliniekset, assertions = F), envir= environment())

  #laad functie om systeeminfo op te halen voor DES, evalueer
  source('R/get_system_information.R', encoding = "UTF-8")
  list2env(get_system_information(Kliniekset, ICset, preds_opnames_ic, preds_opnames_kl, 
                                  condprobs_ic, condprobs_kl, condprobs_ic_kl, condprobs_kl_ic, condprobs_kl_ic_gbhbp, laatste_datumtijd_opnamebestand ,
                                  overplaatsingen_er_uit = overplaatsingen_er_uit), envir = environment())
  
  
  if('echt' %in% scenarios){
    aankomsten_per_type = load_obj(paste0('predictions/aankomsten_ptype_',ziekenhuis,'.rds') )
    for(ptype in names(aankomsten_per_type)){
      aankomsten_ptype = aankomsten_per_type[[ptype]] %>% filter(startdatumtijd >= voorspeldatumtijd & startdatumtijd <= voorspeldatumtijd + days(aantal_dagen + 1)) %>% 
                                        mutate(diff_abs = as.numeric(difftime(startdatumtijd, laatste_datum_opnamebestand , unit = 'days')))
      
      list_scenario_arrival_rates$echt[[ptype]] = list(cuml_intensity_NHPP = get_function_next(as.vector(aankomsten_ptype %>% pull(diff_abs))),
                                                       icuml_intensity_NHPP = Inf)
      # print(aankomsten_ptype %>% arrange(startdatumtijd))
    }
  }
  
  # draai DES
  source('R/DES.R')
  source('R/perform_simulation_study.R', encoding = "UTF-8")
  simresults = perform_simulation_study(laatste_datumtijd_opnamebestand, aantal_dagen, list_scenario_arrival_rates, 
                                        departure_info, current_patient_info, patienttype_info, scenarios)
  
  # vat de simulatiestudie samen in een voorspelling van de gemiddelde en maximale benutting 
  source('R/summarise_simulations.R', encoding = "UTF-8")
  res = summarise_simulations(simresults, scenarios, aantal_dagen, voorspeldatum  = laatste_datum_opnamebestand, CI_val = 0.95, all_hor_max = T, hour_eval = 10)
  

return(res)

}



# maak_voorspellingen_sim = function(Kliniekset, ICset, df_nieuwe_opnames, ziekenhuis, aantal_dagen, scenarios, voorspeldatumtijd, fit_vanaf_golf_twee = T, overplaatsingen_er_uit=F){
#   
#   laatste_datumtijd_opnamebestand <- voorspeldatumtijd
#   laatste_datum_opnamebestand <- as.Date(voorspeldatumtijd)
#   if(fit_vanaf_golf_twee){
#     start_tweede_golf = as.Date('2020-08-07') #start datum 2e golf (alle data daarvoor wordt NIET gebruikt voor schatten aankomstproces Richard's curve aangezien de dynamiek nu anders is)
#   }else{
#     start_tweede_golf = as.Date('2000-01-01') #erg vroege datum, geen data er uit
#     
#   }
#   
#   
#   # laad functies om aankomsten te voorspellen, evalueer op nieuwe opnames
#   source("R/01_voorspel_instroom.R", encoding = "UTF-8")
#   voorspelde_opnames = voorspel_instroom(df_nieuwe_opnames, aantal_dagen = aantal_dagen, scenarios, voorspeldatum = laatste_datum_opnamebestand, exclude_until = start_tweede_golf)
#   
#   # laad functies om aankomsten te splitsen per afdeling, evalueer op voorspelde en geobserveerde opnames
#   source("R/02_voorspel_instroom_ic_kl.R", encoding = "UTF-8")
#   list2env(voorspel_instroom_ic_kl(voorspelde_opnames, df_nieuwe_opnames, ziekenhuis, voorspeldatum = laatste_datum_opnamebestand), envir = environment())
#   
#   # laad functies survival analyse op ligduren, evalueer op ICset en Kliniekset
#   source("R/03_survival_analyse.R", encoding = "UTF-8")
#   list2env( survival_analyse(ziekenhuis, ICset, Kliniekset), envir= environment())
#   
#   #laad functie om systeeminfo op te halen voor DES, evalueer
#   source('R/get_system_information.R', encoding = "UTF-8")
#   list2env(get_system_information(Kliniekset, ICset, preds_opnames_ic, preds_opnames_kl, 
#                                   condprobs_ic, condprobs_kl, condprobs_ic_kl, condprobs_kl_ic, condprobs_kl_ic_gbhbp, laatste_datumtijd_opnamebestand ,
#                                   overplaatsingen_er_uit = overplaatsingen_er_uit), envir = environment())
#   
#   
#   if('echt' %in% scenarios){
#     aankomsten_per_type = load_obj(paste0('predictions/aankomsten_ptype_',ziekenhuis,'.rds') )
#     for(ptype in names(aankomsten_per_type)){
#       aankomsten_ptype = aankomsten_per_type[[ptype]] %>% filter(startdatumtijd >= voorspeldatumtijd & startdatumtijd <= voorspeldatumtijd + days(aantal_dagen + 1)) %>% 
#         mutate(diff_abs = as.numeric(difftime(startdatumtijd, laatste_datum_opnamebestand , unit = 'days')))
#       
#       list_scenario_arrival_rates$echt[[ptype]] = list(cuml_intensity_NHPP = get_function_next(as.vector(aankomsten_ptype %>% pull(diff_abs))),
#                                                        icuml_intensity_NHPP = Inf)
#       # print(aankomsten_ptype %>% arrange(startdatumtijd))
#     }
#   }
#   
#   # draai DES
#   source('R/DES.R')
#   source('R/perform_simulation_study.R', encoding = "UTF-8")
#   simresults = perform_simulation_study(laatste_datumtijd_opnamebestand, aantal_dagen, list_scenario_arrival_rates, 
#                                         departure_info, current_patient_info, patienttype_info, scenarios)
#   
#   # vat de simulatiestudie samen in een voorspelling van de gemiddelde en maximale benutting 
#   source('R/summarise_simulations.R', encoding = "UTF-8")
#   res = summarise_simulations(simresults, scenarios, aantal_dagen, voorspeldatum  = laatste_datum_opnamebestand, CI_val = 0.95, all_hor_max = T, hour_eval = 10)
#   
#   
#   return(res)
#   
# }
# 
# 





