package_list = c('crayon','reticulate', 'boot','minpack.lm','plotly', 'parallel', 'pracma', 'padr', 'dplyr', 'doParallel', 'Hmisc', 'ggplot2', 'tibble', 'purrr', 'pander', 'scales',
  'tidyverse', 'lubridate','investr','FlexParamCurve', 'survival', 'survminer', 'readr', 'readxl', 'tictoc', 'tibble', "grid", 'naniar'
)

for(package_name in package_list){
  if(!(package_name %in% rownames(installed.packages()))){
    install.packages(package_name, dependencies = T)
  }
  do.call(library, list(package = package_name, character.only = TRUE))}
library(boot)
library(plotly)
library('parallel')
library('pracma')
library('padr')
library('dplyr')
library('doParallel')
library('Hmisc')
library('coda')
library('ggplot2') 
library('tibble') 
library('purrr')
library('pander') 
library('scales')
library('tidyverse')
library('minpack.lm')
library('lubridate')
library('investr')
library('FlexParamCurve') 
library('survival')
library('survminer') 
library('readr')
library('readxl') 
library('tictoc')
library(tibble)
library(grid)

replace_null_with_na = function(v){
  v[strcmp(v,'NULL')] = NA
  return(v)
}

#pad naar pandoc instellen
path = Sys.getenv("RSTUDIO_PANDOC")
Sys.setenv(RSTUDIO_PANDOC=path)

cdf_df_to_fn = function(df){
  #input : df with column ligduur and column cdf at ligduur, output: list of cdf and icdf functions
  return(list(cdf = get_simple_function(df$ligduur, df$cdf),
              icdf = get_inv_simple_fn(df$ligduur, df$cdf)))
}

voorspel_lgc2 <- function(mod, df, horizon = 7, level_pi = .80) {
  # Haal de naam van de afhankelijke variabele op
  dep_naam <- all.vars(summary(mod)$formula)[1]

  # Maak een nieuw data frame
  num <- 1:(nrow(df) + horizon)
  datum_nieuw <- map_dfr(list(x = 1:horizon), ~ max(df$datum) + days(.))
  datum <- c(df$datum, datum_nieuw)
  df_pred <- tibble(datum, num)

  # Voeg de afhankelijke variabele toe
  dep <- c(pull(df, dep_naam), rep(NA, horizon))
  df_pred[dep_naam] <- dep

  # Maak voorspellingen en transformeer ze terug
  # N.B.: Ik heb hier voor predictie-intervallen gekozen, dat lijkt me in dat geval geschikter dan
  # betrouwbaarheidsintervallen. Eens?
  pred <- try(predFit(mod, df_pred, interval = "prediction", level = level_pi) %>%
    as_tibble(), silent = T)
  if(is.character(pred)){
    pred <- tibble(fit = predFit(mod, df_pred))
    pred$upr =  rep(Inf, nrow(pred))
    pred$lwr =  rep(0, nrow(pred))
  }
  pred$upr = sapply(pred$upr, function(x){max(0,x)})
  pred$lwr = sapply(pred$lwr, function(x){max(0,x)})
  pred$datum <- df_pred$datum
  pred$obs <- pull(df_pred, dep_naam)

  pred = pred %>% mutate( nieuwe_opn = obs, log = fit, log_upr = upr, log_lwr = lwr) 
  return(pred)
}

# voorspel_lgc2 <- function(mod, df, horizon = 7, level_pi = .80) {
#   # Haal de naam van de afhankelijke variabele op
#   dep_naam <- all.vars(summary(mod)$formula)[1]
#   
#   # Maak een nieuw data frame
#   num <- 1:(nrow(df) + horizon)
#   datum_nieuw <- map_dfr(list(x = 1:horizon), ~ max(df$datum) + days(.))
#   datum <- c(df$datum, datum_nieuw)
#   df_pred <- tibble(datum, num)
#   
#   # Voeg de afhankelijke variabele toe
#   dep <- c(pull(df, dep_naam), rep(NA, horizon))
#   df_pred[dep_naam] <- dep
#   
#   # Maak voorspellingen en transformeer ze terug
#   # N.B.: Ik heb hier voor predictie-intervallen gekozen, dat lijkt me in dat geval geschikter dan
#   # betrouwbaarheidsintervallen. Eens?
#   pred <- predFit(mod, df_pred, interval = "prediction", level = level_pi) %>%
#     as_tibble()
#   pred$datum <- df_pred$datum
#   pred$obs <- pull(df_pred, dep_naam)
#   pred$upr = sapply(pred$upr, function(x){max(0,x)})
#   pred$lwr = sapply(pred$lwr, function(x){max(0,x)})
#   pred = pred %>% rename(nieuwe_opn = obs, log = fit, log_upr = upr, log_lwr = lwr)
#   return(pred)
# }


maak_partiele_set = function(dept_set, datetime, name_dept, name_other_dept){
  #muteer een kliniekset/ICset zodanig dat alleen de beschikbare data vóór "date" meegenomen is.  
  
  dept_set = dept_set[as.POSIXct(dept_set$startdatumtijd, tz = 'UTC') <= datetime, ] # haal aankomsten na date weg
  for(i in seq(nrow(dept_set))){
    #als pat aangekomen is vóór date en weggaat na "date", zet einddatumtijd naar "date" en geef aan dat patiënt toen nog lag
    if(  is.na(dept_set[i,]$einddatumtijd)|dept_set[i,]$einddatumtijd > datetime){
      dept_set[i,]$einddatumtijd = datetime
      dept_set[i, ]$einddatum = as.Date(datetime)
      dept_set[i, ]$vertrek_type = paste0(name_dept, ' - Nog opgenomen')
    }
  }
  dept_set$ligduur = sapply(difftime(dept_set$einddatum , dept_set$startdatum, unit = 'days') + 1, as.integer)  # obtaining LOS
  
  #redetermining event variable for survival analysis
  dept_set$event = ifelse(dept_set$vertrek_type == paste0(name_dept, " - ander ziekenhuis"), 0,
                          ifelse(dept_set$vertrek_type == paste0(name_dept, " - Nog opgenomen"), 0, 1))
  return(dept_set)
}

integer_breaks <- function(n = 5, ...) {
  fxn <- function(x) {
    breaks <- floor(pretty(x, n, ...))
    names(breaks) <- attr(breaks, "labels")
    breaks
  }
  return(fxn)
}

weird_quantfun = function(v, num_thr){
  # find largest threshold b s.t. sum_i v_i*(v_i>b) >num_thr
  v = sort(v, decreasing = F)
  i = length(v)
  while(sum(v[v>v[i]] - v[i])<= num_thr){
    i = i-1
  }
  return(v[i])
}

roaz_regio = function(ziekenhuis){
  if(ziekenhuis %in% c('LUMC','Haga','GHZ')){
    return('Netwerk Acute Zorg West')
  }else{
    if(ziekenhuis %in% c('rs')){
      return("Acute Zorgregio Oost")
    }else{
      if(ziekenhuis %in% c('ETZ')){
        return('Netwerk Acute Zorg Brabant')
      }else{
        warning('voeg roaz regio voor ziekenhuis toe in utils.R')
        return(NA)
      }
    }}
}

confradius = function(v, level = 0.95){
  return(quantile(v, 1/2+level/2)[[1]] - quantile(v, 1/2 - level/2)[[1]])
}

haal_paden_overplaatsingen_er_uit = function(Kliniekset, ICset){
  colname_pat_nrs = colnames(Kliniekset)[1] # dit is niet bij elk ziekenhuis hetzelfde volgens mij
  
  #vind nummers pat die een keer overgeplaatst zijn
  pat_nr_overgeplaatst = c(Kliniekset %>% filter(Herkomst == 'Ander ziekenhuis')%>% pull(colname_pat_nrs), ICset %>% filter(Herkomst == 'Ander ziekenhuis')%>% pull(colname_pat_nrs)) 
  
  #haal ze uit kliniekset/icset
  Kliniekset = Kliniekset[!is.element(Kliniekset %>% pull(colname_pat_nrs),  pat_nr_overgeplaatst), ]
  ICset = ICset[!is.element(ICset %>% pull(colname_pat_nrs),  pat_nr_overgeplaatst), ]
  
  return(mget(c('Kliniekset', 'ICset')))
}

haal_paden_in_uitplaatsingen_er_uit = function(Kliniekset, ICset){
  colname_pat_nrs = colnames(Kliniekset)[1] # dit is niet bij elk ziekenhuis hetzelfde volgens mij
  
  #vind nummers pat die een keer overgeplaatst zijn
  pat_nr_overgeplaatst = c(Kliniekset %>% filter((Herkomst %in% c('Ander ziekenhuis' , 'Ziekenhuis buitenland')))%>% pull(colname_pat_nrs),
                           ICset  %>% filter((Herkomst %in% c('Ander ziekenhuis' , 'Ziekenhuis buitenland')))%>% pull(colname_pat_nrs),
                           ICset  %>% filter((Bestemming %in% c('Ander ziekenhuis' , 'Ziekenhuis buitenland')))%>% pull(colname_pat_nrs),
                           Kliniekset %>% filter((Bestemming %in% c('Ander ziekenhuis' , 'Ziekenhuis buitenland')))%>% pull(colname_pat_nrs)) 
  
  #haal ze uit kliniekset/icset
  Kliniekset = Kliniekset[!is.element(Kliniekset %>% pull(colname_pat_nrs),  pat_nr_overgeplaatst), ]
  ICset = ICset[!is.element(ICset %>% pull(colname_pat_nrs),  pat_nr_overgeplaatst), ]
  
  return(mget(c('Kliniekset', 'ICset')))
}

quiet <- function(x) { 
  sink(tempfile()) 
  on.exit(sink()) 
  invisible(force(x)) 
} 

ewma <- compiler::cmpfun(function(x, a) {
  #ewma function
  n <- length(x)
  s <- rep(NA,n)
  s[1] <- x[1]
  if (n > 1) {
    for (i in 2:n) {
      s[i] <- a * x[i] + (1 - a) * s[i-1]
    }
  }
  return(s)
})

ma <- function(x, n ){as.numeric(stats::filter(x, rep(1 / n, n), sides = 1))}

weights_exp = function(num, coef){
  # return exponential weights
  weights_unn = rep(coef,num)^(seq(num)-1)
  return(weights_unn/sum(weights_unn))
}



fit_curve_krijg_voorspellingen = function(df, horizon, max_peaks = Inf, var = 'Besmettingen', alpha_ewma = 0.5){
  level_pi   = 0.80
  dg_extrpl  = 7
  
  #num moet itereren over aantal rijen in df
  df$num = seq(nrow(df))
  # df = df %>% mutate(!!var := ewma(df%>% pull(var), alpha_ewma)) # take moving average
  df_full = quiet(full_join(df, tibble(datum = seq(max(df$datum) + days(1),max(df$datum) + days(dg_extrpl) , by ='day'),
                                       num   = seq(max(df$num)+1, max(df$num) + dg_extrpl )) %>% 
                              add_column(!!var := tail(ewma(df %>% pull(var), alpha_ewma),1))))
  
  ## model 1: linear fit through data, this will always work
  mod1      = try(nls(as.formula(paste0( var,'~ alpha + beta * num')),
                      df_full, start = list(alpha = 0, beta = 1), control = nls.control(maxiter = 1000)), silent = T)
  pred_tot1  = try(head(voorspel_lgc2(mod1, df, horizon = horizon, level_pi = level_pi),-1), silent = T)
  
  if( is.character(pred_tot1)){
    crit1 = Inf
  }else{
    crit1 = 0 # will be Inf if any other model fits
  }
  
  ## model 2: increasing exponential (early data)
  #find good init estimate by fitting a linear model on the log of data
  df_nonzero = df %>% filter(var > 0)
  x = cbind(ones(nrow(df_nonzero),1), df_nonzero$num)
  y = log(df_nonzero %>% pull(var))
  coefs = pinv(t(x)%*%x)%*%t(x)%*%y
  
  #using init estimate to fit exponential
  alpha.0 <- exp(coefs[1])
  beta.0 <- abs(coefs[2])
  start <- list(alpha = alpha.0, beta = beta.0, gamma = 0 )
  
  #fit, intercept is included to handle zeros
  mod2 = try(nlsLM(as.formula(paste0( var,'~ alpha * exp(beta * num) + gamma')),
                 df_full, start = start, control = nls.control(maxiter = 1000), lower = c(beta = 0, alpha = 0, gamma =- Inf), algorithm ='port'), silent = T)
  
  if(is.character(mod2)){
    crit2 = Inf
  }else{
    MSE =  try(mean((df %>% pull(var) - predFit(mod2, df))^2), silent = T)
    pred_tot2 =  try(voorspel_lgc2(mod2, df, horizon= horizon , level_pi = level_pi), silent = T)
    
    if(is.character(MSE) | is.character(pred_tot2)){
      crit2 = Inf
    }else{
      crit2 = MSE
    }}
  
  #model 3: logistic curve
  peakinfo = findpeaks(df %>% pull(var), npeaks = 1) # find peaks with min distance 1 month
  if(is.null(peakinfo)){
    mod3  = character()
    crit3 = Inf
  }else{
    #fit
    mod3 = try(nlsLM(as.formula(paste0(var, ' ~ der_LC(num, Asym, xmid, scal)')), df_full, start = list(Asym = peakinfo[1], xmid = peakinfo[2], scal = 1),
                     control = nls.lm.control(maxiter = 1000)), silent = T)
    
    if(is.character(mod3)){
      crit3 = Inf
    }else{
      MSE =  try(mean((df %>% pull(var) - predFit(mod3, df))^2), silent = T)
      pred_tot3 =  try(voorspel_lgc2(mod3, df, horizon= horizon , level_pi = level_pi), silent = T)
      if(is.character(MSE) | is.character(pred_tot3)){ #test if model can predict
        crit3 = Inf
      }else{
        crit3 = MSE
      }}}
  
  #model 4: mixture of Richards' curves
  k=1 # num curves
  crit4 = Inf
  mod4  = character()
  while(k <= max_peaks){
    peakinfo = findpeaks(df %>% pull(var), npeaks = k, minpeakdistance = 14) # find peaks with min distance 2 weeks
    
    if(is.null(peakinfo)){
      break
    }else{
      if(dim(peakinfo)[[1]] < k){
        break
      }}
    
    # generate starting values for parameters and model to fit
    startlist = list()
    formula = as.formula(paste0(var, ' ~ der_RC(num, A_1, delta_1, t0_1, k_1)'))
    for(i in seq(k)){
      app = paste0('_',i)
      startlist[[paste0('A',app)]] = peakinfo[i,1]
      startlist[[paste0('delta',app)]] = 1
      startlist[[paste0('t0',app)]] = peakinfo[i,2]
      startlist[[paste0('k',app)]] = 1
      if(i>1){
        formula = update(formula,   as.formula(paste0('~ . + der_RC(num, A',app,',delta', app, ', t0',app,',k',app,')')))
      }
    }
    
    #fit
    upper_list = unlist(startlist)*Inf
    for(i in seq(k)){
      upper_list[paste0('delta_',i)] = 5
    }
    mod_new = try(nlsLM(formula, data=df_full, start = startlist, control = nls.lm.control(maxiter = 1000), lower = unlist(startlist)*0,
                        upper =upper_list ), silent = T)
    
    if(is.character(mod_new)){
      break
    }else{
      crit_new =  try(mean((df %>% pull(var) - predFit(mod_new, df))^2), silent = T)
      pred_tot4 =  try(voorspel_lgc2(mod_new, df, horizon= horizon, level_pi = level_pi), silent = T)
      if(is.character(crit_new) | crit_new > crit4 | is.character(pred_tot4)){
        break
      }else{
        crit4 = crit_new
        mod4  = mod_new
      }
    }
    k = k+1
  }
  
  if(min(c(crit2,crit3,crit4)) < Inf){crit1 = Inf} #never return linear extrapolation if other models resulted in fit
  mods = list(mod1,mod2,mod3, mod4)
  crit = c(crit1,crit2,crit3, crit4 )
  bestmod = which(crit == min(crit))
  preds_lgc = voorspel_lgc2(mods[[bestmod]], df, horizon = horizon, level_pi = level_pi)
  return(preds_lgc)
}


# fit_curve_krijg_voorspellingen = function(df, horizon, max_peaks = Inf, var = 'Besmettingen', alpha_ewma = 0.5){
#   level_pi = 0.80
#   lambda   = 0.8
#   optmethod = 'CG'
#   
#   #num moet itereren over aantal rijen in df, split in test/train
#   df$num = seq(nrow(df))
#   df = df %>% mutate(!!var := ewma(df%>% pull(var), alpha_ewma)) # take moving average
#   idx_train = seq(ceiling(nrow(df)*0.8))
#   df_train = df[idx_train,]
#   df_test = df[-idx_train,]
#   
#   ## model 1: linear fit through data, this is the "if everything else fails" model
#   mod1      = try(nls(as.formula(paste0( var,'~ alpha + beta * num')),
#                       df_full, start = list(alpha = 0, beta = 1), control = nls.control(maxiter = 1000)), silent = T)
#   pred_tot1  = try(head(voorspel_lgc2(mod1, df, horizon = horizon, level_pi = level_pi),-1), silent = T)
#   
#   if( is.character(pred_tot1)){
#     crit1 = Inf
#   }else{
#     crit1 = 0 # will be Inf if any other model fits
#   }
#   
#   ## model 2: increasing exponential (early data)
#   
#   #find good init estimate by fitting a linear model on the log of data
#   df_nonzero = df_train %>% filter(var > 0)
#   x = cbind(ones(nrow(df_nonzero),1), df_nonzero$num)
#   y = df_nonzero %>% pull(var)
#   coefs = pinv(t(x)%*%x)%*%t(x)%*%y
#   
#   #using init estimate to fit exponential
#   alpha.0 <- exp(coefs[1])
#   beta.0 <- abs(coefs[2])
#   start <- list(alpha = alpha.0, beta = beta.0, gamma = 0 )
#   
#   #fit, intercept is included to handle zeros
#   mod2 = try(nls(as.formula(paste0( var,'~ alpha * exp(beta * num) + gamma')),
#                  df_train, start = start, control = nls.control(maxiter = 1000), lower = list(beta = 0), algorithm ='port'), silent = T)
#   
#   if(is.character(mod2)){
#     crit2 = Inf
#   }else{
#     
#     fopt = function(x){
#       mod21 = try(nlsLM(as.formula(paste0( var,'~ alpha * exp(beta * num) + gamma')),
#                      df_train, start = start, control = nls.control(maxiter = 1000), lower = c(beta = 0, alpha = 0, gamma = -Inf), algorithm ='port',
#                      weights = weights_exp(nrow(df_train), x)), silent = T)
#       if(is.character(mod21)){
#         return(10^10)
#       }else{
#         return(lambda*mean((df_test %>% pull(var) - predFit(mod21, df_test))^2)+
#                (1-lambda)*mean((df_train %>% pull(var) - predFit(mod21, df_train))^2))
#       }
#     }
#     
#     optres = optim(2, fopt, method = optmethod)
#     
#     tuning_param = optres$par
#     crit2        = optres$value
#     mod2 = try(nlsLM(as.formula(paste0( var,'~ alpha * exp(beta * num) + gamma')),
#                    df, start = start, control = nls.control(maxiter = 1000), lower = c(beta = 0, alpha = 0, gamma = -Inf), algorithm ='port',
#                    weights = weights_exp(nrow(df), tuning_param)), silent = T)
#     
#     pred_tot2 =  try(voorspel_lgc2(mod2, df, horizon= horizon , level_pi = level_pi), silent = T)
#     
#     }
#   
#   
#   #model 3: logistic curve
#   peakinfo = findpeaks(df_train %>% pull(var), npeaks = 1) # find peaks with min distance 1 month
#   if(is.null(peakinfo)){
#     mod3  = character()
#     crit3 = Inf
#   }else{
#     #fit
#     mod3 = try(nlsLM(as.formula(paste0(var, ' ~ der_LC(num, Asym, xmid, scal)')), df_train, start = list(Asym = peakinfo[1], xmid = peakinfo[2], scal = 1),
#                      control = nls.lm.control(maxiter = 1000)), silent = T)
#     
#     if(is.character(mod3)){
#       crit3 = Inf
#     }else{
#       fopt = function(x){
#         mod31 = try(nlsLM(as.formula(paste0(var, ' ~ der_LC(num, Asym, xmid, scal)')), df_train, start = list(Asym = peakinfo[1], xmid = peakinfo[2], scal = 1),
#                          control = nls.lm.control(maxiter = 1000), weights = weights_exp(nrow(df_train), x)), silent = T)
#         if(is.character(mod31)){
#           return(10^10)
#         }else{
#           return(lambda*mean((df_test %>% pull(var) - predFit(mod31, df_test))^2)+
#                 (1-lambda)*mean((df_train %>% pull(var) - predFit(mod31, df_train))^2))
#         }
#       }
#       
#       optres = optim(1, fopt, method = optmethod)
#       tuning_param = optres$par
#       crit3 = optres$value
#       
#       peakinfo = findpeaks(df_train %>% pull(var), npeaks = 1)
#       mod3 = try(nlsLM(as.formula(paste0(var, ' ~ der_LC(num, Asym, xmid, scal)')), df, start = list(Asym = peakinfo[1], xmid = peakinfo[2], scal = 1),
#                        control = nls.lm.control(maxiter = 1000), weights = weights_exp(nrow(df), tuning_param)), silent = T)
#       pred_tot3 =  try(voorspel_lgc2(mod3, df, horizon= horizon , level_pi = level_pi), silent = T)
#       }}
#   
#   #model 4: mixture of Richards' curves
#   k=1 # num curves
#   crit4 = Inf
#   mod4  = character()
#   while(k <= max_peaks){
#     peakinfo = findpeaks(df_train %>% pull(var), npeaks = k, minpeakdistance = 14) # find peaks with min distance 2 weeks
#     
#     if(is.null(peakinfo)){
#       break
#     }else{
#       if(dim(peakinfo)[[1]] < k){
#         break
#       }}
#     
#     # generate starting values for parameters and model to fit
#     startlist = list()
#     formula = as.formula(paste0(var, ' ~ der_RC(num, A_1, delta_1, t0_1, k_1)'))
#     for(i in seq(k)){
#       app = paste0('_',i)
#       startlist[[paste0('A',app)]] = peakinfo[i,1]
#       startlist[[paste0('delta',app)]] = 1
#       startlist[[paste0('t0',app)]] = peakinfo[i,2]
#       startlist[[paste0('k',app)]] = 1
#       if(i>1){
#         formula = update(formula,   as.formula(paste0('~ . + der_RC(num, A',app,',delta', app, ', t0',app,',k',app,')')))
#       }
#     }
#     
#     #fit
#     upper_list = unlist(startlist)*Inf
#     for(i in seq(k)){
#       upper_list[paste0('delta_',i)] = 5
#     }
#     mod_new = try(nlsLM(formula, data=df_train, start = startlist, control = nls.lm.control(maxiter = 1000), lower = unlist(startlist)*0,
#                         upper =upper_list ), silent = T)
#     
#     if(is.character(mod_new)){
#       break
#     }else{
#       fopt = function(x){
#         
#         #for some reason I have to redefine formula.. 
#         formula = as.formula(paste0(var, ' ~ der_RC(num, A_1, delta_1, t0_1, k_1)'))
#         for(i in seq(k)){
#           app = paste0('_',i)
#           if(i>1){
#             formula = update(formula,   as.formula(paste0('~ . + der_RC(num, A',app,',delta', app, ', t0',app,',k',app,')')))
#           }
#         }
#         
#         mod_new_part = try(nlsLM(formula, data=df_train, start = startlist, control = nls.lm.control(maxiter = 1000), lower = unlist(startlist)*0,
#                             upper =upper_list, weights =  weights_exp(nrow(df_train),x)), silent = T)
#         
#         if(is.character(mod_new_part)){
#           return(10^10)
#         }else{
#           return(lambda*mean((df_test %>% pull(var) - predFit(mod_new_part, df_test))^2)+
#                 (1-lambda)*mean((df_train %>% pull(var) - predFit(mod_new_part, df_train))^2))
#         }
#       }
#       
#       optres = optim(1, fopt, method = optmethod)
#       tuning_param = optres$par
#       crit_new  = optres$value
#       
#       peakinfo = findpeaks(df %>% pull(var), npeaks = k, minpeakdistance = 14) # find peaks with min distance 2 weeks
#       
#       # generate starting values for parameters and model to fit
#       startlist = list()
#       formula = as.formula(paste0(var, ' ~ der_RC(num, A_1, delta_1, t0_1, k_1)'))
#       for(i in seq(k)){
#         app = paste0('_',i)
#         startlist[[paste0('A',app)]] = peakinfo[i,1]
#         startlist[[paste0('delta',app)]] = 1
#         startlist[[paste0('t0',app)]] = peakinfo[i,2]
#         startlist[[paste0('k',app)]] = 1
#         if(i>1){
#           formula = update(formula,   as.formula(paste0('~ . + der_RC(num, A',app,',delta', app, ', t0',app,',k',app,')')))
#         }
#       }
#       
#       mod_new = try(nlsLM(formula, data=df, start = startlist, control = nls.lm.control(maxiter = 1000), lower = unlist(startlist)*0,
#                           upper =upper_list , weights = weights_exp(nrow(df), tuning_param)), silent = T)
#       pred_tot4 =  try(voorspel_lgc2(mod_new, df, horizon= horizon, level_pi = level_pi), silent = T)
#       if(is.character(crit_new) | crit_new > crit4 | is.character(pred_tot4)){
#         break
#       }else{
#         crit4 = crit_new
#         mod4  = mod_new
#       }
#     }
#     k = k+1
#   }
#   
#   if(min(c(crit2,crit3,crit4)) < Inf){crit1 = Inf} #never return linear extrapolation if other models resulted in fit
#   mods = list(mod1,mod2,mod3, mod4)
#   crit = c(crit1,crit2,crit3, crit4 )
#   bestmod = which(crit == min(crit))
#   preds_lgc = voorspel_lgc2(mods[[bestmod]], df, horizon = horizon, level_pi = level_pi)
#   return(preds_lgc)
# }



load_obj <- function(f)
  #https://stackoverflow.com/questions/5577221/how-can-i-load-an-object-into-a-variable-name-that-i-specify-from-an-r-data-file
{
  env <- new.env()
  nm <- load(f, env)[1]
  env[[nm]]
}
get_function_next = function(vector){
  if(length(vector) == 0){ return(function(x){Inf})}
  else{
    return(function(x){ifelse(x>=max(vector), Inf, min(vector[vector>x]))})
  }
}
rnhpois = function(t, cuml_intensity, inv_cuml_intensity){
  # sample the next nonhomogeneous Poisson arrival time at time t given the cumulative intensity function and the inverse
  # cumulative intensity function. Theory is described in 
  # Daley, D. J., & Vere-Jones, D. (2007). An introduction to the theory of point processes: volume II: general theory and structure. Springer Science & Business Media.
  # Basically, when transforming the time axis with the cumulative intensity Lambda(s) = int_0^s lambda(u)du, the resulting process on this time axis is 
  # a unit-rate homogeneous PP. Hence, we sample an unit rate homogeneous PP on this time scale by transforming the last event-time with Lambda, incrementing it with
  # an exponential random variable, and taking the inverse of Lambda of the result.
  if(typeof(inv_cuml_intensity) == 'double'){
    #this is a workaround if we have deterministic arrivals, let then the inv_cuml_intensity be equal to Inf and let the cuml_intensity_function then return the next absolute arrival time in days
    return(cuml_intensity(t))
  }else{
    
    result = tryCatch(
      {ttime = rexp(1) + cuml_intensity(t)
      if(ttime > cuml_intensity(Inf)){
        return(Inf)
      }else{
        result = inv_cuml_intensity(ttime)
      }},
      warning = function(cond){return(Inf)}
    )
    
    return(result)} }

first = function(v){
  return(v[1])
}


add_title_DES <- function(title, naam_voluit) {
  ggtitle(
    paste0(title, " - ", naam_voluit)
  )
}

laad_rds_DES <- function(naam, ziekenhuis = "etz") {
  if (basename(getwd()) == "rmarkdown") {
    map <- "../DES/rds/"
  } else {
    map <- "DES/rds/"
  }
  
  bestand <- paste0(map, ziekenhuis, "_", naam, ".rds")
  
  read_rds(bestand)
}

der_RC = function(t, A,delta,t0, k){
  A = abs(A)
  delta = abs(delta)
  k = abs(k)
  return(A*k*exp(-k*(t - t0))*(1 + delta*exp(-k *(t - t0)))^(-1 - 1/delta))
}
der_RC = Vectorize(der_RC, vectorize.args = c('t'))

der_LC = function(t, A, t0, k){
  return(A*k*exp(-k*(t - t0))*(1 + exp(-k *(t - t0)))^(-2))
}
der_LC = Vectorize(der_LC, vectorize.args = c('t'))


colors <- c("Model (Richards curve)" = rgb(0, 176/(176+246),246/(176+246)),
            "Exponentieel" = rgb(163/(165+163), 165/(163+165),0),
            "Constante Instroom" = 'purple',
            "Bovengrens Model (Richards curve)" = 'red',
            'Tweede Piek' = 'blue')

names_cols <- c('log' = "Model (Richards curve)",
              'log_upr' = "Bovengrens Model (Richards curve)",
              'vlak' = 'Constante Instroom',
              'piek' = 'Tweede Piek',
              'exp' = 'Exponentieel')

schrijf_rds_DES <- function(object, ziekenhuis) {
  bestand <- paste0("DES/rds/", ziekenhuis, "_", deparse(substitute(object)), ".rds")
  
  write_rds(object, bestand)
}

floor_at_0 = Vectorize(function(x){max(x,0)})

rewrite_date = Vectorize(function(datestring){
  # rewrites date in format d(d)-m(m)-yyyy hh:mm in the format yyyy-mm-dd hh:mm
  
  #split the string such that we obtain vector c(d(d), m(m), yyyy, hh:mm)
  splitted_string = str_split(datestring, '-')
  splitted_string = c(splitted_string[[1]][1],splitted_string[[1]][2] , unlist(str_split(splitted_string[[1]][3], ' ')))
  
  #adding zero to day number if it only has one number
  if(nchar(splitted_string[1]) == 1){
    splitted_string[1] = paste0('0', splitted_string[1])
  }
  
  #adding zero to month number if it only has one number
  if(nchar(splitted_string[2]) == 1){
    splitted_string[2] = paste0('0', splitted_string[2])
  }
  
  return(paste0(splitted_string[3],'-',splitted_string[2],'-',splitted_string[1],' ', splitted_string[4],':00'))
  
})


Poisregr_logistic = function(dates, counts){
  # fit and return a 4-parameter Richards curve using Poisson regression on count data 
  # input should be the dates and counts on these dates
  
  der_logistic = function(t, A,k,i){
    #returns the derivative of a 3 param logistic curve at time point t (integer) RC(t) =  A / (1+  exp(-k (t-i)))
    return((A*k*exp(-k*(t - i)))/(1 + exp(-k*(t - i)))^2)
  }
  negloglik_logistic = function(datenum,counts, A,k,i){
    #negative Poisson log likelihood where the expected value is a 4 param RC. datenum are the date numbers (integers)
    expectedcounts = der_logistic(datenum, A,k,i)
    return(sum(expectedcounts) - log(expectedcounts)%*%counts)
  }
  
  #calculate date numbers (sort the dates first and counts accordingly)
  counttbl = tibble(dates = dates, counts = counts) %>% arrange(dates) 
  dates = counttbl$dates
  counts = counttbl$counts
  datenum = c(1,1+cumsum(sapply(diff(dates), as.integer))) #numbering dates
  
  #define optimization scheme
  init = c(1,1,1)
  fopt = function(params){
    negloglik_logistic(datenum, counts, params[1] , params[2], params[3])
  }
  estimate = constrOptim(init, fopt, grad = NULL, ui = identity(3), ci =zeros(1,3))$par
  
  #define Richards curve prediction function (function based on date)
  logcurve = Vectorize(function(date){return(ifelse(date<min(dates), NA, der_logistic(1+as.integer(date - min(dates)),
                                                                                               estimate[1], estimate[2], estimate[3])))})
  
  return(logcurve) 
}

sample_path_nhpois = function(horizon, cuml_intensity, inv_cuml_intensity, cur_time = 0){
  #sample a path with certain horizon from the inhomogeneous poisson proces with given (inverse) 
  # cumulative intensity function 
  jump_times = c()
  while(cur_time<=horizon){
    cur_time = rnhpois(cur_time, cuml_intensity, inv_cuml_intensity)
    if(is.infinite(cur_time)){
      break
    }
    if(cur_time<=horizon){jump_times = c(jump_times, cur_time)}
  }
  counts = as.data.frame(ones(length(jump_times),1))
  rownames(counts) = jump_times
  return(counts)
}

#veiligheidsregio per roaz regio (nodig voor opnames per roaz regio)
veiligheidsreg_per_roaz_regio = list("Netwerk Acute Zorg Brabant" = c("Brabant-Zuidoost","Midden- en West-Brabant","Brabant-Noord") ,
                                     "Acute Zorgregio Oost" = c("Gelderland-Midden", "Gelderland-Zuid",'Noord- en Oost-Gelderland'),  
                                     'Netwerk Acute Zorg West' = c("Haaglanden", "Hollands-Midden") ,  
                                     "Traumazorgnetwerk Midden-Nederland" = c("Utrecht"))

sample_remaining_LOS = function( current_LOS, departure_cdf, departure_icdf){
  # this function obtains a sample from the conditional length of stay CDF, where the condition is that the length of stay (LOS)
  # is larger than "current_LOS".
  return(max(1/(24*60), departure_icdf(runif(1)*(1-departure_cdf(current_LOS-2)) + departure_cdf(current_LOS-2)) - current_LOS+2))
}


get_simple_function = function(xvals, yvals, cdf = FALSE){
  #obtain a "simple function", which returns the last observed yval at xvals
  
  #sort the data
  ordered_data = cbind(xvals,yvals)[order(xvals),]
  xvals = ordered_data[,1]
  yvals = ordered_data[,2]
  
  #return function
  return(Vectorize(function(x){
    if(x< min(xvals)){return(0)
    }else{
      if(cdf  & (x>max(xvals))){return(1)}else{
        return(yvals[which(xvals<=x)[sum(xvals<=x)]])
      }
    }
  }))
}


zero_intensity_info = function(){
  return(list(cuml_intensity_NHPP = function(t){0}, icuml_intensity_NHPP = function(t){Inf}))
}

condprob_transition = function(tibble_cond_probs, tibble_probs, LOS){
  #obtain conditional probability of being in a path corresponding to a transition between departments
  prob_transition = max(tibble_cond_probs$ligduur_01)
  if(LOS == 1){return(prob_transition)}else{
    condprob = as.numeric((prob_transition-tibble_cond_probs$ligduur_01[LOS-1])/(tibble_probs$ligduur_01[LOS-1]))
    if(is.na(condprob)){
      condprob = 0
    }
    return(condprob)}}



get_inv_simple_fn = function(xvals, yvals){
  # creates a "simple function" with jumps at xvals and height yvals at xvals.
  #sort the data
  ordered_data = cbind(xvals,yvals)[order(xvals),]
  xvals = ordered_data[,1]
  yvals = ordered_data[,2]
  
  return(Vectorize(function(p){
    if(p>max(yvals)){return(max(xvals))}else{
  return(min(xvals[which(yvals>=p)]))}}))
}


create_arrival_info = function(horizon, cuml_arrival_intensities){
  if(sum(cuml_arrival_intensities) == 0) {
    return(zero_intensity_info())
  } else {
    arrival_info = list(cuml_intensity_NHPP = approxfun(seq(0, horizon), cuml_arrival_intensities , rule = 2),
                       icuml_intensity_NHPP =approxfun(cuml_arrival_intensities, seq(0,horizon),  rule = 2))
  }
  
}

constantfn= function(val){
  #return constant function
  return(Vectorize(function(t){val}))
}

laad_output_DES <- function(naam, ziekenhuis = "etz", prefix = "") {
  map <- paste0("DES/output/", naam)
  
  if (prefix == "") {
    pattern <- paste0("^", ziekenhuis, "_2020")
  } else {
    pattern <- paste0("^", ziekenhuis, "_", prefix, "_2020")
  }
  
  # Zoek het laatste bestand (op alfabetische volgorde)
  bestand <- tail(list.files(map, pattern, full.names = TRUE), n = 1)
  
  out <- read_delim(bestand, locale = nl, delim = ";", trim_ws = TRUE)
  
  print(paste("Bestand geladen:", bestand))
  
  return(out)
}



#---------------------------------- Unit Tests ----------------------------------------

unittest_NHPP_sampling = function(){
  #check that the number of counts from an inhomogeneous poisson proces indeed follow a poisson distribution
  # with the correct parameter
  horizon = 100
  intervals = seq(0,horizon,length.out = 5)
  cuml_intensity = function(t){20/(1+exp(6-t/10))}
  inv_cuml_intensity = function(t){10*(6-log(20/t-1))}
  M = 10000
  counts_intervals = c()
  for(i in seq(M)){
    path = sample_path_nhpois(horizon, cuml_intensity , inv_cuml_intensity = inv_cuml_intensity )
    jump_times = as.numeric(rownames(path))
    counts_intervals = rbind(counts_intervals, sapply(seq(2, length(intervals)), function(rep){sum((jump_times >= intervals[rep-1])&(jump_times < intervals[rep]))}))
  }
  for(i in seq(length(intervals)-1)){
    #plot empirical probability from counts over intervals vs. theoretical distribution.
    x = sort(unique(counts_intervals[,i]))
    plot(x, sapply(x, function(xval){mean(counts_intervals[,i] == xval)}), lwd = 2, main = paste('num arrivals interval [', toString(intervals[i]),' , ',toString(intervals[i+1]) ,'] vs. theor', sep = ''), ylab = 'probability', xlab = 'counts')
    lines(x, dpois(x, lambda = cuml_intensity(intervals[i+1])-cuml_intensity(intervals[i])), lwd = 2)
  }
}

unittest_get_simple = function(){
  xvals = seq(0,9,1)
  yvals = c(0.1,0.3,0.35,0.4,0.45,0.5,0.55,0.6,0.7,1)
  f = get_simple_function(xvals, yvals)
  g = get_inv_simple_fn(xvals,yvals)
  xx = seq(-1,11,0.01)
  plot(xx, f(xx), type = 'l')
  
  yy = seq(0,1,0.01)
  plot(yy, g(yy))
}

if(sys.nframe()==0){
  
 unittest_get_simple()
  unittest_NHPP_sampling()
  }