ewma <- compiler::cmpfun(function(x, a) {
  #ewma function
    n <- length(x)
    s <- rep(NA,n)
    s[1] <- x[1]
    if (n > 1) {
      for (i in 2:n) {
        s[i] <- a * x[i] + (1 - a) * s[i-1]
      }
    }
    return(s)
  })

vind_start_2e_golf = function(dates, counts){
  #start 2e golf voor besmettingen is (arbitrair) vastgezet op de eerste tijd dat een EWMA met alpha = 0.05 groter is can 0.5
  counts = counts[dates > as.Date('2020-07-01')]
  dates = dates[dates > as.Date('2020-07-01')]
  return(first(dates[ewma(counts, 0.05)>0.5]))
}

voorspel_besmettingen_en_sla_op = function(horizon = 14, start_tweede_golf_regio = as.Date('2020-07-01'), plot_ = F, print_=F, level_pi = 0.8, max_peaks = Inf){
  source('R/utils.R')
  if(!exists('df_besmettingen_tot')){ # read data from website
    df_besmettingen_tot = read_delim('https://data.rivm.nl/covid-19/COVID-19_aantallen_gemeente_per_dag.csv', delim = ';')
  }
  
  #df van besmettingen in ROAZ regio
  df_besmettingen_naz_west = df_besmettingen_tot%>%
                      filter(ROAZ_region == 'Netwerk Acute Zorg West')%>%
                      select(Date_of_publication, Total_reported, Municipality_name)%>%
                      rename(datum = Date_of_publication, Besmettingen = Total_reported, Gemeente = Municipality_name )
  
  #aggregeer op roaz regio
  df_besmettingen_roaz = df_besmettingen_naz_west%>%
                         group_by(datum)%>% summarise(Besmettingen = sum(Besmettingen), .groups = 'drop_last') %>% ungroup()%>%
                          filter(datum >= start_tweede_golf_regio)
  
  
  df_besmettingen_roaz$num = c(1,1+cumsum(as.integer(diff(df_besmettingen_roaz$datum))))
  df_besmettingen_roaz$aantal_cumulatief = cumsum(df_besmettingen_roaz$Besmettingen)
  
  df_pred = fit_curve_krijg_voorspellingen(df_besmettingen_roaz, horizon, level_pi = level_pi, max_peaks = max_peaks)
  write_csv(df_pred %>% select(datum,`Nieuwe Besmettingen`= nieuwe_opn, Ondergrens = log_lwr,Voorspelling = log,  Bovengrens = log_upr), 'predictions/voorspelde_besmettingen_naz_west.csv')
  if(plot_){print(ggplotly(ggplot(df_pred, aes(x = datum)) + geom_line(aes(y = log)) + geom_point(aes(y = nieuwe_opn)) + geom_ribbon(aes(ymin = log_lwr, ymax = log_upr), alpha = 0.2 ,fill= 'red') + ggtitle('regio')))}
  for(gemeente in unique(df_besmettingen_naz_west$Gemeente[!is.na(df_besmettingen_naz_west$Gemeente)])){
    if(print_){print(gemeente)}
    #aggregeer op gemeente niveau
    df_besmettingen_gemeente = df_besmettingen_naz_west%>%filter(Gemeente == gemeente)%>%
      group_by(datum)%>% summarise(Besmettingen = sum(Besmettingen), .groups = 'drop_last') %>% ungroup() %>% filter(datum >= start_tweede_golf_regio)

    #als de 2e golf veel later start voor de gemeente, vind dan de start van 2e golf voor die gemeente
    start_tweede_golf_gemeente = vind_start_2e_golf(df_besmettingen_gemeente$datum, df_besmettingen_gemeente$Besmettingen)
    df_besmettingen_gemeente = df_besmettingen_gemeente %>% filter(datum >= start_tweede_golf_gemeente)

    #reken datum nummers en cumulatieve besmettingen uit
    df_besmettingen_gemeente$num = c(1,1+cumsum(as.integer(diff(df_besmettingen_gemeente$datum))))
    df_besmettingen_gemeente$aantal_cumulatief = cumsum(df_besmettingen_gemeente$Besmettingen)

    #fit de curve, schrijf voorspellingen weg
    df_pred_gemeente = fit_curve_krijg_voorspellingen(df_besmettingen_gemeente, horizon, max_peaks = max_peaks)
    if(plot_){print(ggplotly(ggplot(df_pred_gemeente, aes(x = datum)) + geom_line(aes(y = log)) + geom_point(aes(y = nieuwe_opn)) + geom_ribbon(aes(ymin = log_lwr, ymax = log_upr), alpha = 0.2 ,fill= 'red') + ggtitle(gemeente)))}
    write_csv(df_pred_gemeente %>% select(datum,`Nieuwe Besmettingen`= nieuwe_opn,Ondergrens = log_lwr,Voorspelling = log,  Bovengrens = log_upr), paste0('predictions/voorspelde_besmettingen_',gemeente,'.csv'))
  }

}

